@@ -1,19 +1,19 @@
 import Distribution.Simple
 import Distribution.Simple.LocalBuildInfo
-import Distribution.Simple.BuildPaths
+import Distribution.Simple.BuildPaths(exeExtension)
 import Distribution.Simple.Utils
 import Distribution.Simple.Setup
 import Distribution.PackageDescription hiding (Flag)
 import Control.Monad
-import Data.List(isPrefixOf,intersect)
+import Data.Char(isSpace)
+import Data.List(isPrefixOf,intersect,unfoldr,stripPrefix)
 import Data.Maybe(listToMaybe)
 --import System.IO
 import qualified Control.Exception as E
-import System.Process
+import System.Process(readProcess)
 import System.FilePath
-import System.Directory
-import System.Process
-import System.Exit
+import System.Directory(createDirectoryIfMissing,copyFile,doesDirectoryExist,getDirectoryContents)
+--import System.Exit
 --import Control.Concurrent(forkIO)
 --import Control.Concurrent.Chan(newChan,writeChan,readChan)
 
@@ -23,14 +23,13 @@ tryIOE :: IO a -> IO (Either E.IOException a)
 tryIOE = E.try
 
 main :: IO ()
-main = defaultMainWithHooks simpleUserHooks{ preBuild =gfPreBuild
-                                           , postBuild=gfPostBuild
-                                           , preInst  =gfPreInst
-                                           , postInst =gfPostInst
-                                           , preCopy  =const . checkRGLArgs
-                                           , postCopy =gfPostCopy
-                                           , sDistHook=sdistRGL
---                                         , runTests =testRGL
+main = defaultMainWithHooks simpleUserHooks{ preBuild  = gfPreBuild
+                                           , postBuild = gfPostBuild
+                                           , preInst   = gfPreInst
+                                           , postInst  = gfPostInst
+                                           , preCopy   = const . checkRGLArgs
+                                           , postCopy  = gfPostCopy
+                                           , sDistHook = sdistError
                                            }
   where
     gfPreBuild args  = gfPre args . buildDistPref
@@ -42,75 +41,89 @@ main = defaultMainWithHooks simpleUserHooks{ preBuild =gfPreBuild
          return h
 
     gfPostBuild args flags pkg lbi =
-      do buildRGL args flags pkg lbi
-         let gf = default_gf pkg lbi
-         buildWeb gf args flags pkg lbi
+      do --writeFile "running" ""
+         buildRGL args flags (flags,pkg,lbi)
+--       let gf = default_gf lbi
+--       buildWeb gf (pkg,lbi)
 
     gfPostInst args flags pkg lbi =
-      do installRGL args flags pkg lbi
-         let gf = default_gf pkg lbi
-         installWeb gf args flags pkg lbi
+      do installRGL args flags (pkg,lbi)
+         let gf = default_gf lbi
+         installWeb (pkg,lbi)
 
-    gfPostCopy args flags pkg lbi =
-      do copyRGL args flags pkg lbi
-         let gf = default_gf pkg lbi
-         copyWeb gf args flags pkg lbi
+    gfPostCopy args flags  pkg lbi =
+      do let gf = default_gf lbi
+         copyRGL args flags (pkg,lbi)
+         copyWeb flags (pkg,lbi)
 
 --------------------------------------------------------
 -- Commands for building the Resource Grammar Library
 --------------------------------------------------------
 
 data Mode = AllTenses | Present deriving Show
-all_modes = ["present","alltenses"]
-default_modes = [Present,AllTenses]
+all_modes = ["alltenses","present"]
+default_modes = [AllTenses,Present]
 
 data RGLCommand
   = RGLCommand
       { cmdName   :: String
       , cmdIsDef  :: Bool
-      , cmdAction :: [Mode] -> [String] -> PackageDescription -> LocalBuildInfo -> IO ()
+      , cmdAction :: [Mode] -> [String] -> Info -> IO ()
       }
 
+type Info = (BuildFlags,PackageDescription,LocalBuildInfo)
+bf (i,_,_) = i
+--pd (_,i,_) = i
+lbi (_,_,i) = i
+
 rglCommands =
-  [ RGLCommand "prelude" True  $ \mode args pkg lbi -> do
+  [ RGLCommand "prelude" True  $ \mode args bi -> do
        putStrLn $ "Compiling [prelude]"
-       let prelude_src_dir = rgl_src_dir     </> "prelude"
-           prelude_dst_dir = rgl_dst_dir lbi </> "prelude"
+       let prelude_src_dir = rgl_src_dir    </> "prelude"
+           prelude_dst_dir = rgl_dst_dir (lbi bi) </> "prelude"
        createDirectoryIfMissing True prelude_dst_dir
-       files <- ls prelude_src_dir
-       run_gfc pkg lbi (["-s", "--gfo-dir="++prelude_dst_dir] ++ [prelude_src_dir </> file | file <- files])
-  , RGLCommand "lang"    True  $ \modes args pkg lbi -> do
-       parallel_
-         [do mapM_ (gfc1 mode pkg lbi . lang) (optml mode langsLang args)
-             mapM_ (gfc1 mode pkg lbi . symbol) (optml mode langsAPI args)
-          | mode <- modes]
-  , RGLCommand "compat"  True  $ \modes args pkg lbi -> do
-         mapM_ (gfc modes pkg lbi . compat) (optl langsCompat args)
-  , RGLCommand "api"     True  $ \modes args pkg lbi -> do
-       parallel_
-         [do mapM_ (gfc1 mode pkg lbi . try) (optml mode langsAPI args)
-             mapM_ (gfc1 mode pkg lbi . symbolic) (optml mode langsSymbolic args)
-          | mode <- modes]
-  , RGLCommand "pgf"     False $ \modes args pkg lbi ->
+       files <- list_files prelude_src_dir
+       run_gfc bi (["-s", "--gfo-dir="++prelude_dst_dir] ++ [prelude_src_dir </> file | file <- files])
+
+  , RGLCommand "all"     True  $ gfcp [l,s,c,t,sc]
+  , RGLCommand "lang"    False $ gfcp [l,s]
+  , RGLCommand "api"     False $ gfcp [t,sc]
+  , RGLCommand "compat"  False $ gfcp [c]
+  , RGLCommand "web"     True  $ \ _ _ bi -> buildWeb (default_gf (lbi bi)) bi
+
+  , RGLCommand "pgf"     False $ \modes args bi ->
      parallel_ [
-       do let dir = getRGLBuildDir lbi mode
+       do let dir = getRGLBuildDir (lbi bi) mode
           createDirectoryIfMissing True dir
-          sequence_ [run_gfc pkg lbi ["-s","-make","-name=Lang"++la,
+          sequence_ [run_gfc bi ["-s","-make","-name=Lang"++la,
                                        dir ++ "/Lang" ++ la ++ ".gfo"]
                       | (_,la) <- optl langsPGF args]
-          run_gfc pkg lbi (["-s","-make","-name=Lang"]++
+          run_gfc bi (["-s","-make","-name=Lang"]++
                            ["Lang" ++ la ++ ".pgf"|(_,la)<-optl langsPGF args])
        | mode <- modes]
-  , RGLCommand "demo"    False $ \modes args pkg lbi -> do
+  , RGLCommand "demo"    False $ \modes args bi -> do
        let ls = optl langsDemo args
-       gf (demos "Demo" ls) ["demo/Demo" ++ la ++ ".gf" | (_,la) <- ls] pkg lbi
+       gf bi (demos "Demo" ls) ["demo/Demo" ++ la ++ ".gf" | (_,la) <- ls]
        return ()
-  , RGLCommand "parse"   False $ \mode args pkg lbi -> do
-       mapM_ (gfc mode pkg lbi . parse) (optl langsParse args)
-  , RGLCommand "none"    False $ \mode args pkg lbi -> do
+  , RGLCommand "parse"   False $ \modes args bi ->
+       gfc bi modes (summary parse) (map parse (optl langsParse args))
+  , RGLCommand "none"    False $ \modes args bi ->
        return ()
   ]
   where
+    gfcp cs modes args bi = parallel_ [gfcp' bi mode args cs|mode<-modes]
+
+    gfcp' bi mode args cs = gfcn bi mode (unwords ss) (concat fss)
+      where (ss,fss) = unzip [(summary f,map f as)|c<-cs,let (f,as)=c mode args]
+
+    summary f = f ("*","*")
+
+    l mode args = (lang,optml mode langsLang args)
+    s mode args = (symbol,optml mode langsAPI args)
+    c mode args = (compat,optl langsCompat args)
+    t mode args = (try,optml mode langsAPI args)
+    sc mode args = (symbolic,optml mode langsSymbolic args)
+
     optl = optml AllTenses
     optml mode ls args = getOptLangs (shrink ls) args
       where
@@ -128,31 +141,31 @@ checkRGLArgs args = do
     putStrLn $ "Unrecognised flags: " ++ intercalate ", " args'
   return emptyHookedBuildInfo
 
-buildRGL args flags pkg lbi = do
+buildRGL args flags bi = do
   let cmds = getRGLCommands args
   let modes = getOptMode args
-  mapM_ (\cmd -> cmdAction cmd modes args pkg lbi) cmds
+  mapM_ (\cmd -> cmdAction cmd modes args bi) cmds
 
-installRGL args flags pkg lbi = do
+installRGL args flags bi = do
   let modes = getOptMode args
-  let inst_gf_lib_dir = datadir (absoluteInstallDirs pkg lbi NoCopyDest) </> "lib"
-  copyAll "prelude"   (rgl_dst_dir lbi </> "prelude") (inst_gf_lib_dir </> "prelude")
-  sequence_ [copyAll (show mode) (getRGLBuildDir lbi mode) (inst_gf_lib_dir </> getRGLBuildSubDir lbi mode)|mode<-modes]
+  let inst_gf_lib_dir = datadir (uncurry absoluteInstallDirs bi NoCopyDest) </> "lib"
+  copyAll "prelude"   (rgl_dst_dir (snd bi) </> "prelude") (inst_gf_lib_dir </> "prelude")
+  sequence_ [copyAll (show mode) (getRGLBuildDir (snd bi) mode) (inst_gf_lib_dir </> getRGLBuildSubDir mode)|mode<-modes]
 
-copyRGL args flags pkg lbi = do
+copyRGL args flags bi = do
   let modes = getOptMode args
       dest = case copyDest flags of
                NoFlag -> NoCopyDest
                Flag d -> d
-  let inst_gf_lib_dir = datadir (absoluteInstallDirs pkg lbi dest) </> "lib"
-  copyAll "prelude"   (rgl_dst_dir lbi </> "prelude") (inst_gf_lib_dir </> "prelude")
-  sequence_ [copyAll (show mode) (getRGLBuildDir lbi mode) (inst_gf_lib_dir </> getRGLBuildSubDir lbi mode)|mode<-modes]
+  let inst_gf_lib_dir = datadir (uncurry absoluteInstallDirs bi dest) </> "lib"
+  copyAll "prelude"   (rgl_dst_dir (snd bi) </> "prelude") (inst_gf_lib_dir </> "prelude")
+  sequence_ [copyAll (show mode) (getRGLBuildDir (snd bi) mode) (inst_gf_lib_dir </> getRGLBuildSubDir mode)|mode<-modes]
 
 copyAll s from to = do
   putStrLn $ "Installing [" ++ s ++ "] " ++ to
   createDirectoryIfMissing True to
-  mapM_ (\file -> copyFile (from </> file) (to </> file)) =<< ls from
-
+  mapM_ (\file -> copyFile (from </> file) (to </> file)) =<< list_files from
+{-
 sdistRGL pkg mb_lbi hooks flags = do
   paths <- getRGLFiles rgl_src_dir []
   let pkg' = pkg{dataFiles=paths}
@@ -169,8 +182,14 @@ sdistRGL pkg mb_lbi hooks flags = do
                then return (path : paths)
                else return paths
         else getRGLFiles path paths
+-}
 
-rgl_src_dir     = "lib" </> "src"
+-- | Cabal doesn't know how to correctly create the source distribution, so
+-- we print an error message with the correct instructions when someone tries
+-- `cabal sdist`.
+sdistError _ _ _ _ = fail "Error: Use `make sdist` to create the source distribution file"
+
+rgl_src_dir         = "lib" </> "src"
 rgl_dst_dir lbi = buildDir lbi </> "rgl"
 
 -- the languages have long directory names and short ISO codes (3 letters)
@@ -178,6 +197,7 @@ rgl_dst_dir lbi = buildDir lbi </> "rgl"
 -- don't support all flags needed; they are used in tests
 
 langsCoding = [
+  (("afrikaans","Afr"),""),
   (("amharic",  "Amh"),""),
   (("arabic",   "Ara"),""),
   (("bulgarian","Bul"),""),
@@ -189,17 +209,22 @@ langsCoding = [
   (("estonian", "Est"),""),
   (("finnish",  "Fin"),""),
   (("french",   "Fre"),""),
+  (("ancient_greek","Grc"),""),
   (("greek",    "Gre"),""),
   (("hebrew",   "Heb"),""),
   (("hindi",    "Hin"),"to_devanagari"),
   (("german",   "Ger"),""),
+  (("icelandic","Ice"),""),
   (("interlingua","Ina"),""),
   (("italian",  "Ita"),""),
   (("japanese", "Jpn"),""),
   (("latin",    "Lat"),""),
   (("latvian",    "Lav"),""),
   (("maltese",  "Mlt"),""),
+  (("mongolian","Mon"),""),
+  (("nepali",   "Nep"),""),
   (("norwegian","Nor"),""),
+  (("nynorsk",  "Nno"),""),
   (("persian",  "Pes"),""),
   (("polish",   "Pol"),""),
   (("punjabi",   "Pnb"),""),
@@ -223,15 +248,15 @@ langsLang = langs -- `except` ["Amh","Ara","Lat","Tur"]
 --langsLang = langs `only` ["Fin"] --test
 
 -- languagues that have notpresent marked
-langsPresent = langsLang `except` ["Chi","Gre","Heb","Jpn","Mlt","Nep","Pes","Snd","Tha","Thb","Est"]
+langsPresent = langsLang `except` ["Afr","Chi","Gre","Heb","Ice","Jpn","Mlt","Mon","Nep","Pes","Snd","Tha","Thb","Est"]
 
 -- languages for which to compile Try
 langsAPI  = langsLang `except` langsIncomplete -- ["Ina","Amh","Ara"]
 
-langsIncomplete = ["Amh","Ara","Heb","Ina","Lat","Tur"]
+langsIncomplete = ["Amh","Ara","Grc","Heb","Ina","Lat","Tur"]
 
 -- languages for which to compile Symbolic
-langsSymbolic  = langsAPI `except` ["Jpn"]
+langsSymbolic  = langsAPI `except` ["Afr","Ice","Jpn","Mon","Nep"]
 
 -- languages for which to run demo test
 langsDemo = langsLang `except` ["Ara","Hin","Ina","Lav","Tha"]
@@ -245,19 +270,20 @@ langsPGF = langsLang `except` ["Ara","Hin","Ron","Tha"]
 -- languages for which Compatibility exists (to be extended)
 langsCompat = langsLang `only` ["Cat","Eng","Fin","Fre","Ita","Lav","Spa","Swe"]
 
-gfc modes pkg lbi file = parallel_ [gfc1 mode pkg lbi file | mode<-modes]
-gfc1 mode pkg lbi file = do
-  let dir = getRGLBuildDir lbi mode
+gfc bi modes summary files = 
+    parallel_ [gfcn bi mode summary files | mode<-modes]
+gfcn bi mode summary files = do
+  let dir = getRGLBuildDir (lbi bi) mode
       preproc = case mode of
                   AllTenses -> ""
                   Present   -> "-preproc="++({-rgl_src_dir </>-} "mkPresent")
   createDirectoryIfMissing True dir
-  putStrLn $ "Compiling [" ++ show mode ++ "] " ++ file
-  run_gfc pkg lbi ["-s", "-no-pmcfg", preproc, "--gfo-dir="++dir, file]
+  putStrLn $ "Compiling [" ++ show mode ++ "] " ++ summary
+  run_gfc bi (["-s", "-no-pmcfg", preproc, "--gfo-dir="++dir] ++ files)
 
-gf comm files pkg lbi = do
+gf bi comm files = do
   putStrLn $ "Reading " ++ unwords files
-  let gf = default_gf pkg lbi
+  let gf = default_gf (lbi bi)
   putStrLn ("executing: " ++ comm ++ "\n" ++
             "in " ++ gf)
   out <- readProcess gf ("-s":files) comm
@@ -307,13 +333,14 @@ getOptLangs defaultLangs args =
                    then findLangs langs [l]++ls
                    else ls
 
-getRGLBuildSubDir lbi mode =
+getRGLBuildSubDir mode =
   case mode of
     AllTenses -> "alltenses"
     Present   -> "present"
 
 
-getRGLBuildDir lbi mode = rgl_dst_dir lbi </> getRGLBuildSubDir lbi mode
+getRGLBuildDir :: LocalBuildInfo -> Mode -> FilePath
+getRGLBuildDir lbi mode = rgl_dst_dir lbi </> getRGLBuildSubDir mode
 
 getRGLCommands args =
   let cmds0 = [cmd | arg <- args,
@@ -337,23 +364,15 @@ unlexer abstr ls =
       unlex lla = maybe "" id $ lookup lla langsCoding
 
 -- | Runs the gf executable in compile mode with the given arguments.
-run_gfc :: PackageDescription -> LocalBuildInfo -> [String] -> IO ()
-run_gfc pkg lbi args =
-    do let args' = ["-batch","-gf-lib-path="++rgl_src_dir]
-                   ++ ["+RTS","-K32M","-RTS"] -- not needed with new-comp
+run_gfc :: Info -> [String] -> IO ()
+run_gfc bi args =
+    do let args' = numJobs (bf bi)++["-batch","-gf-lib-path="++rgl_src_dir]
                    ++ filter (not . null) args
-           gf = default_gf pkg lbi
-           gf_cmdline = gf ++ " " ++ unwords (map showArg args')
---     putStrLn $ "Running: " ++ gf_cmdline
-       e <- rawSystem gf args'
-       case e of
-         ExitSuccess   -> return ()
-         ExitFailure i -> do putStrLn $ "Ran: " ++ gf_cmdline
-                             die $ "gf exited with exit code: " ++ show i
-  where
-    showArg arg = if ' ' `elem` arg then "'" ++ arg ++ "'" else arg
+           gf = default_gf (lbi bi)
+       execute gf args'
 
-default_gf pkg lbi = buildDir lbi </> exeName' </> exeNameReal
+default_gf :: LocalBuildInfo -> FilePath
+default_gf lbi = buildDir lbi </> exeName' </> exeNameReal
   where
     exeName' = "gf"
     exeNameReal = exeName' <.> exeExtension
@@ -376,17 +395,18 @@ extractDarcsVersion distFlag =
     versionModulePath = autogenPath</>"DarcsVersion_gf.hs"
     modname = "DarcsVersion_gf"
 
-    askDarcs =
-      do tags <- lines `fmap` readProcess "darcs" ["show","tags"] ""
-         let from = case tags of
-                      [] -> []
-                      tag:_ -> ["--from-tag="++tag]
-         changes <- lines `fmap` readProcess "darcs" ("changes":from) ""
-         let dates = init' (filter ((`notElem` [""," "]).take 1) changes)
-         whatsnew <- tryIOE $ lines `fmap` readProcess "darcs" ["whatsnew","-s"] ""
-         return (listToMaybe tags,listToMaybe dates,
-                 length dates,either (const 0) length whatsnew)
-
+askDarcs =
+  do flip unless (fail "no _darcs") =<< doesDirectoryExist "_darcs"
+     tags <- lines `fmap` readProcess "darcs" ["show","tags"] ""
+     let from = case tags of
+                  [] -> []
+                  tag:_ -> ["--from-tag="++tag]
+     dates <- (init' . patches) `fmap` readProcess "darcs" ("changes":from) ""
+--   let dates = init' (filter ((`notElem` [""," "]).take 1) changes)
+     whatsnew <- tryIOE $ lines `fmap` readProcess "darcs" ["whatsnew","-s"] ""
+     return (listToMaybe tags,listToMaybe dates,
+             length dates,either (const 0) length whatsnew)
+  where
     init' [] = []
     init' xs = init xs
 
@@ -397,7 +417,7 @@ updateFile path new =
                                  writeFile path new
 
 -- | List files, excluding "." and ".."
-ls path = filter (`notElem` [".",".."]) `fmap` getDirectoryContents path
+list_files path = filter ((/=".").take 1) `fmap` getDirectoryContents path
 
 
 -- | For parallel RGL module compilation
@@ -407,3 +427,14 @@ parallel_ ms = sequence_ ms {-
      ts <- sequence [ forkIO (m >> writeChan c ()) | m <- ms]
      sequence_ [readChan c | _ <- ts]
 --}
+
+patches = paras . lines
+  where
+    paras = unfoldr para
+    para ls = case break null $ dropWhile null ls of
+                ([],[]) -> Nothing
+                (xs,ys) -> Just (info xs,ys)
+
+    info = unwords . map dropHeaders . filter (\l->not $ any (`isPrefixOf` l) [" ","patch "])
+    dropHeaders = dropWhile isSpace . dropPrefix "Author: " . dropPrefix "Date: "
+    dropPrefix pre l = maybe l id (stripPrefix pre l)
