@@ -1,5 +1,5 @@
 lang1 = "Eng"
-lang2 = "Tha"
+lang2 = "Chi"
 
 -- to write a comparison for two languages
 
@@ -14,7 +14,7 @@ mkExx sss = case sss of
   (s1:ss1,s2:ss2) | isMsg s1 -> mkExx (ss1,s2:ss2)
   (s1:ss1,s2:ss2) | isMsg s2 -> mkExx (s1:ss1,ss2)
   (s1:ss1,s2:ss2) | s1 /= s2 -> 
-     putStrLn (drops s1) >> putStrLn (filter (not . (==' ')) (drops s2)) >> putStrLn [] >> mkExx (ss1,ss2) -- show strings
+     putStrLn (drops s1) >> putStrLn (drops s2) >> putStrLn [] >> mkExx (ss1,ss2) -- show strings
   (s1:ss1,s2:ss2) | s1 == s2 && not (isJunk s1) -> 
      putStrLn (drops s1) >> mkExx (ss1,ss2) -- show the term
   (s1:ss1,s2:ss2) | s1 == s2 -> mkExx (ss1,ss2)
