@@ -19,7 +19,7 @@ revealedLang = "Eng"
 -- all languages shown
 apiExxFiles = ["api-examples-" ++ lang ++ ".txt" | lang <- words
 --   "Eng Chi"
-  "Afr Bul Cat Chi Dan Dut Eng Est Fin Fre Ger Gre Hin Ita Jpn Lav Mlt Nep Nor Pes Pnb Pol Ron Rus Snd Spa Swe Tha Urd"
+  "Afr Bul Cat Chi Dan Dut Eng Est Fin Fre Ger Gre Hin Ice Ita Jpn Lav Mlt Mon Nep Nor Nno Pes Pnb Pol Ron Rus Snd Spa Swe Tha Urd"
    ]
 
 main = do
@@ -45,6 +45,8 @@ main = do
   append "%!postproc(html): '(SRC=\"categories.png\")'  '\\1 USEMAP=\"#categories\"'"
   append "%!postproc(html): '#LParadigms'  '<a name=\"RParadigms\"></a>'"
   append "%!postproc(tex): '#LParadigms' ''"
+  append "%!postproc(html): '#quicklinks' '<script src=\"quicklinks.js\"></script>'"
+  append "%!postproc(tex): '#quicklinks' ''"
   delimit $ addToolTips cs
   include "synopsis-intro.txt"
   title "Categories"
@@ -95,6 +97,8 @@ main = do
   space
   append "%%toc"
   space
+  append "#quicklinks"
+  space
   let format = if isLatex then "tex" else "html"
   system $ "txt2tags -t" ++ format ++ " " ++ " --toc " ++ synopsis
   if isLatex then (system $ "pdflatex synopsis.tex") >> return () else return ()
@@ -252,13 +256,16 @@ paradigmFiles = [
   ("German",  srcPath "/german/ParadigmsGer.gf"),
   ("Greek",  srcPath "/greek/ParadigmsGre.gf"),
   ("Hindi", srcPath "/hindi/ParadigmsHin.gf"),
+  ("Icelandic", srcPath "/icelandic/ParadigmsIce.gf"),
 --  ("Interlingua", srcPath "/interlingua/ParadigmsIna.gf"),
   ("Italian",  srcPath "/italian/ParadigmsIta.gf"),
   ("Japanese",  srcPath "/japanese/ParadigmsJpn.gf"),
   ("Latvian",  srcPath "/latvian/ParadigmsLav.gf"),
   ("Maltese",  srcPath "/maltese/ParadigmsMlt.gf"),
+  ("Mongolian",  srcPath "/mongolian/ParadigmsMon.gf"),
   ("Nepali", srcPath "/nepali/ParadigmsNep.gf"),
   ("Norwegian", srcPath "/norwegian/ParadigmsNor.gf"),
+  ("Nynorsk", srcPath "/nynorsk/ParadigmsNno.gf"),
   ("Polish", srcPath "/polish/ParadigmsPol.gf"),
   ("Punjabi", srcPath "/punjabi/ParadigmsPnb.gf"),
   ("Romanian", srcPath "/romanian/ParadigmsRon.gf"),
@@ -333,14 +340,13 @@ readFileC cod file = do
   case cod of
     "utf8" -> readFile file
     _ -> do
-      system $ "iconv -f ISO-8859-1 -t UTF-8 " ++ file ++ " >" ++ tmp
+      system $ "iconv -f " ++ cod ++ " -t UTF-8 " ++ file ++ " >" ++ tmp
       readFile tmp
 
 -- 'intelligently' determine the coding of a file
+---- AR 5/6/2016: now utf8 is used for all languages except Bul, where no characters are shown in documentation anyway
 coding file = case language file of
-  "Pol" -> "utf8"
-  "Rus" -> "utf8"
-  "Mlt" -> "utf8"
-  _     -> "latin1"
+  "Bul" -> "CP1251" --- "ISO-8859-1"
+  _     -> "utf8"
 
 language = reverse . take 3 . drop 3 . reverse
