@@ -6,8 +6,9 @@ module Main where
 -- 3. $ runghc ../MkLang.hs Ger Dut
 -- AR 6/11/2009
 
-import System
-import List
+import System.Cmd
+import System.Environment
+import Data.List
 
 main = do
   xx <- getArgs 
@@ -21,7 +22,7 @@ change xx = case xx of
      system "ls *.gf > files.tmp"
      files <- readFile "files.tmp" >>= return . lines
      mapM_ (changeIdents from to) files
-     mapM_ commentOut files
+----     mapM_ commentOut files
   comment -> do
      files <- readFile "files.tmp" >>= return . lines
      mapM_ commentOut files
