@@ -21,10 +21,12 @@ import GF.Speech.PrRegExp
 
 import Data.Maybe
 import System.FilePath
-import Text.PrettyPrint
+import GF.Text.Pretty
 
 -- top-level access to code generation
 
+-- | Export a PGF to the given 'OutputFormat'. For many output formats,
+-- additional 'Options' can be used to control the output.
 exportPGF :: Options
           -> OutputFormat 
           -> PGF 
