@@ -23,19 +23,20 @@ import GF.Grammar.Predef
 import GF.Grammar.Lockfield (isLockLabel)
 import GF.Data.BacktrackM
 import GF.Data.Operations
-import GF.Infra.UseIO (IOE,ePutStr,ePutStrLn)
+import GF.Infra.UseIO (ePutStr,ePutStrLn) -- IOE,
 import GF.Data.Utilities (updateNthM) --updateNth
-import GF.Compile.Compute.ConcreteNew(GlobalEnv,normalForm,resourceValues,ppL)
+import GF.Compile.Compute.ConcreteNew(normalForm,resourceValues)
 import qualified Data.Map as Map
 import qualified Data.Set as Set
 import qualified Data.List as List
 --import qualified Data.IntMap as IntMap
 import qualified Data.IntSet as IntSet
-import Text.PrettyPrint hiding (Str)
+import GF.Text.Pretty
 import Data.Array.IArray
 import Data.Array.Unboxed
 --import Data.Maybe
 --import Data.Char (isDigit)
+import Control.Applicative(Applicative(..))
 import Control.Monad
 import Control.Monad.Identity
 --import Control.Exception
@@ -44,13 +45,13 @@ import Control.Monad.Identity
 ----------------------------------------------------------------------
 -- main conversion function
 
-generatePMCFG :: Options -> SourceGrammar -> Maybe FilePath -> SourceModule -> IOE SourceModule
+--generatePMCFG :: Options -> SourceGrammar -> Maybe FilePath -> SourceModule -> IOE SourceModule
 generatePMCFG opts sgr opath cmo@(cm,cmi) = do
   (seqs,js) <- mapAccumWithKeyM (addPMCFG opts gr cenv opath am cm) Map.empty (jments cmi)
   when (verbAtLeast opts Verbose) $ ePutStrLn ""
   return (cm,cmi{mseqs = Just (mkSetArray seqs), jments = js})
   where
-    cenv = resourceValues gr
+    cenv = resourceValues opts gr
     gr = prependModule sgr cmo
     MTConcrete am = mtype cmi
 
@@ -66,7 +67,7 @@ mapAccumWithKeyM f a m = do let xs = Map.toAscList m
                                    return (a,(k,y):kys)
 
 
-addPMCFG :: Options -> SourceGrammar -> GlobalEnv -> Maybe FilePath -> Ident -> Ident -> SeqSet -> Ident -> Info -> IOE (SeqSet, Info)
+--addPMCFG :: Options -> SourceGrammar -> GlobalEnv -> Maybe FilePath -> Ident -> Ident -> SeqSet -> Ident -> Info -> IOE (SeqSet, Info)
 addPMCFG opts gr cenv opath am cm seqs id (GF.Grammar.CncFun mty@(Just (cat,cont,val)) mlin@(Just (L loc term)) mprn Nothing) = do
 --when (verbAtLeast opts Verbose) $ ePutStr ("\n+ "++showIdent id++" ...")
   let pres  = protoFCat gr res val
@@ -107,7 +108,7 @@ addPMCFG opts gr cenv opath am cm seqs id (GF.Grammar.CncCat mty@(Just (L _ linc
                                                              mprn
                                                              Nothing) = do
   let pcat = protoFCat gr (am,id) lincat
-      pvar = protoFCat gr (identW,cVar) typeStr
+      pvar = protoFCat gr (MN identW,cVar) typeStr
 
       pmcfgEnv0  = emptyPMCFGEnv
 
@@ -147,66 +148,14 @@ addPMCFG opts gr cenv opath am cm seqs id info = return (seqs, info)
 floc opath loc id = maybe (L loc id) (\path->L (External path loc) id) opath
 
 convert opts gr cenv loc term ty@(_,val) pargs =
-    case term' of
-      Error s -> fail $ render $ ppL loc (text $ "Predef.error: "++s)
-      _ -> do {-when (verbAtLeast opts Verbose) $
-                ePutStrLn $
-                  "\n"++take 10000 (renderStyle style{mode=OneLineMode}
-                    (text "term:"<+>ppU 0 term $$
-                     text "eta expanded:"<+>ppU 0 eterm $$
-                     text "normalized:"<+>ppU 0 term'))--}
-              return $ runCnvMonad gr (conv term') (pargs,[])
+  case normalForm cenv loc (etaExpand ty term) of
+    Error s -> fail $ render $ ppL loc ("Predef.error: "++s)
+    term    -> return $ runCnvMonad gr (convertTerm opts CNil val term) (pargs,[])
   where
-    conv t = convertTerm opts CNil val =<< unfactor t
-
-    eterm = expand ty term
-    term' = {-if flag optNewComp opts
-            then-} normalForm cenv loc eterm -- new evaluator
-            --else term -- old evaluator is invoked from GF.Compile.Optimize
-
-expand (context,val) = mkAbs pars . recordExpand val . flip mkApp args
-  where pars = [(Explicit,v) | v <- vars]
-        args = map Vr vars
-        vars = map (\(bt,x,t) -> x) context
-
-recordExpand :: Type -> Term -> Term
-recordExpand typ trm =
-  case typ of
-     RecType tys -> expand trm
-       where
-         n = length tys
-         expand trm =
-           case trm of
-             FV ts -> FV (map expand ts)
-             R rs | length rs==n -> trm
-             _ -> R [assign lab (P trm lab) | (lab,_) <- tys]
-     _ -> trm
-
-unfactor :: Term -> CnvMonad Term
-unfactor t = CM (\gr c -> c (unfac gr t))
-  where
-    unfac gr t =
-      case t of
-        T (TTyped ty) [(PV x,u)] -> let u' = unfac gr u
-                                        vs = allparams ty
-                                    in --trace ("expand single variable table into "++show (length vs)++" branches.\n"++render (ppU 0 t)) $
-                                       V ty [restore x v u' | v <- vs]
-        T (TTyped ty) [(PW  ,u)] -> let u' = unfac gr u
-                                        vs = allparams ty
-                                    in --trace ("expand wildcard table into "++show (length vs)++ "branches.\n"++render (ppU 0 t)) $
-                                       V ty [u' | _ <- vs]
-        T (TTyped ty) _ -> -- convertTerm doesn't handle these tables
-                           ppbug $
-                           sep [text "unfactor"<+>ppU 10 t,
-                                text (show t){-,
-                                fsep (map (ppU 10) (allparams ty))-}]
-        _ -> composSafeOp (unfac gr) t
-      where
-        allparams ty = err bug id (allParamValues gr ty)
-
-        restore x u t = case t of
-                          Vr y | y == x -> u
-                          _             -> composSafeOp (restore x u) t
+    etaExpand (context,val) = mkAbs pars . flip mkApp args
+      where pars = [(Explicit,v) | v <- vars]
+            args = map Vr vars
+            vars = map (\(bt,x,t) -> x) context
 
 pgfCncCat :: SourceGrammar -> Type -> Int -> CncCat
 pgfCncCat gr lincat index =
@@ -247,6 +196,10 @@ newtype CnvMonad a = CM {unCM :: SourceGrammar
                               -> ([ProtoFCat],[Symbol])
                               -> Branch b}
 
+instance Applicative CnvMonad where
+  pure = return
+  (<*>) = ap
+
 instance Monad CnvMonad where
     return a   = CM (\gr c s -> c a s)
     CM m >>= k = CM (\gr c s -> m gr (\a s -> unCM (k a) gr c s) s)
@@ -376,7 +329,7 @@ computeCatRange gr lincat = compute (0,1) lincat
                                      (index,m) = st
                                  in ((index,m*length vs),CPar (m,zip vs [0..]))
 
-ppPath (CProj lbl path) = ppLabel lbl <+> ppPath path
+ppPath (CProj lbl path) = lbl <+> ppPath path
 ppPath (CSel  trm path) = ppU 5 trm   <+> ppPath path
 ppPath CNil             = empty
 
@@ -417,7 +370,7 @@ convertTerm opts sel ctype (Alts s alts)= do CStr s <- convertTerm opts CNil cty
   where
     unSym (CStr [])        = ""
     unSym (CStr [SymKS t]) = t
-    unSym _                = ppbug $ hang (text "invalid prefix in pre expression:") 4 (ppU 0 (Alts s alts))
+    unSym _                = ppbug $ hang ("invalid prefix in pre expression:") 4 (Alts s alts)
 
     unPatt (EPatt p) = fmap Strs (getPatts p)
     unPatt u         = return u
@@ -429,16 +382,22 @@ convertTerm opts sel ctype (Alts s alts)= do CStr s <- convertTerm opts CNil cty
         as <- getPatts a
         bs <- getPatts b
         return [K (s ++ t) | K s <- as, K t <- bs]
-      _ -> fail (render (text "not valid pattern in pre expression" <+> ppPatt Unqualified 0 p))
+      _ -> fail (render ("not valid pattern in pre expression" <+> ppPatt Unqualified 0 p))
 
 convertTerm opts sel ctype (Q (m,f))
-  | m == cPredef &&
-    f == cNonExist                      = return (CStr [SymNE])
   | m == cPredef &&
     f == cBIND                          = return (CStr [SymBIND])
   | m == cPredef &&
     f == cSOFT_BIND                     = return (CStr [SymSOFT_BIND])
-
+  | m == cPredef &&
+    f == cSOFT_SPACE                    = return (CStr [SymSOFT_SPACE])
+  | m == cPredef &&
+    f == cCAPIT                         = return (CStr [SymCAPIT])
+  | m == cPredef &&
+    f == cALL_CAPIT                     = return (CStr [SymALL_CAPIT])
+  | m == cPredef &&
+    f == cNonExist                      = return (CStr [SymNE])
+{-
 convertTerm opts sel@(CProj l _) ctype (ExtR t1 t2@(R rs2))
                     | l `elem` map fst rs2 = convertTerm opts sel ctype t2
                     | otherwise            = convertTerm opts sel ctype t1
@@ -446,10 +405,10 @@ convertTerm opts sel@(CProj l _) ctype (ExtR t1 t2@(R rs2))
 convertTerm opts sel@(CProj l _) ctype (ExtR t1@(R rs1) t2)
                     | l `elem` map fst rs1 = convertTerm opts sel ctype t1
                     | otherwise            = convertTerm opts sel ctype t2
-
+-}
 convertTerm opts CNil ctype t           = do v <- evalTerm CNil t
                                              return (CPar v)
-convertTerm _    sel  _     t           = ppbug (text "convertTerm" <+> sep [parens (text (show sel)),ppU 10 t])
+convertTerm _    sel  _     t           = ppbug ("convertTerm" <+> sep [parens (show sel),ppU 10 t])
 
 convertArg :: Options -> Term -> Int -> Path -> CnvMonad (Value [Symbol])
 convertArg opts (RecType rs)  nr path =
@@ -489,8 +448,8 @@ convertTbl opts (CSel v sub_sel) ctype pt ts = do
   vs <- getAllParamValues pt
   case lookup v (zip vs ts) of
     Just t  -> convertTerm opts sub_sel ctype t
-    Nothing -> ppbug (text "convertTbl:" <+> (text "missing value" <+> ppU 0 v $$
-                                              text "among" <+> vcat (map (ppU 0) vs)))
+    Nothing -> ppbug ( "convertTbl:" <+> ("missing value" <+> v $$
+                                          "among" <+> vcat vs))
 convertTbl opts _ ctype _ _ = bug ("convertTbl: "++show ctype)
 
 
@@ -571,13 +530,13 @@ evalTerm path (V pt ts)    =
          do vs <- getAllParamValues pt
             case lookup trm (zip vs ts) of
               Just t  -> evalTerm path t
-              Nothing -> ppbug $ text "evalTerm: missing value:"<+>ppU 0 trm
-                                 $$ text "among:" <+>fsep (map (ppU 10) vs)
+              Nothing -> ppbug $ "evalTerm: missing value:"<+>trm
+                                 $$ "among:" <+>fsep (map (ppU 10) vs)
 evalTerm path (S term sel) = do v <- evalTerm CNil sel
                                 evalTerm (CSel v path) term
 evalTerm path (FV terms)   = variants terms >>= evalTerm path
 evalTerm path (EInt n)     = return (EInt n)
-evalTerm path t            = ppbug (text "evalTerm" <+> parens (ppU 0 t))
+evalTerm path t            = ppbug ("evalTerm" <+> parens t)
 --evalTerm path t            = ppbug (text "evalTerm" <+> sep [parens (text (show path)),parens (text (show t))])
 
 getVarIndex x = maybe err id $ getArgIndex x
@@ -654,7 +613,7 @@ restrictProtoFCat path v (PFCat cat f schema) = do
 mkArray    lst = listArray (0,length lst-1) lst
 mkSetArray map = array (0,Map.size map-1) [(v,k) | (k,v) <- Map.toList map]
 
-bug msg = ppbug (text msg)
-ppbug = error . render . hang (text "Internal error in GeneratePMCFG:") 4
+bug msg = ppbug msg
+ppbug msg = error . render $ hang "Internal error in GeneratePMCFG:" 4 msg
 
 ppU = ppTerm Unqualified
