@@ -29,7 +29,7 @@ import GF.Grammar.Unify
 --import GF.Compile.Compute.Abstract
 import GF.Compile.TypeCheck.TC
 
-import Text.PrettyPrint
+import GF.Text.Pretty
 --import Control.Monad (foldM, liftM, liftM2)
 
 -- | invariant way of creating TCEnv from context
@@ -41,7 +41,7 @@ initTCEnv gamma =
 type2val :: Type -> Val
 type2val = VClos []
 
-cont2exp :: Context -> Exp
+cont2exp :: Context -> Term
 cont2exp c = mkProd c eType [] -- to check a context
 
 cont2val :: Context -> Val
@@ -49,7 +49,7 @@ cont2val = type2val . cont2exp
 
 -- some top-level batch-mode checkers for the compiler
 
-justTypeCheck :: SourceGrammar -> Exp -> Val -> Err Constraints
+justTypeCheck :: SourceGrammar -> Term -> Val -> Err Constraints
 justTypeCheck gr e v = do
   (_,constrs0) <- checkExp (grammar2theory gr) (initTCEnv []) e v
   (constrs1,_) <- unifyVal constrs0
@@ -70,10 +70,10 @@ checkContext :: SourceGrammar -> Context -> [Message]
 checkContext st = checkTyp st . cont2exp
 
 checkTyp :: SourceGrammar -> Type -> [Message]
-checkTyp gr typ = err (\x -> [text x]) ppConstrs $ justTypeCheck gr typ vType
+checkTyp gr typ = err (\x -> [pp x]) ppConstrs $ justTypeCheck gr typ vType
 
 checkDef :: SourceGrammar -> Fun -> Type -> Equation -> [Message]
-checkDef gr (m,fun) typ eq = err (\x -> [text x]) ppConstrs $ do
+checkDef gr (m,fun) typ eq = err (\x -> [pp x]) ppConstrs $ do
   (b,cs) <- checkBranch (grammar2theory gr) (initTCEnv []) eq (type2val typ)
   (constrs,_) <- unifyVal cs
   return $ filter notJustMeta constrs
