@@ -13,7 +13,7 @@ import GF.Compile.TypeCheck.Primitives
 
 import Data.List
 import Control.Monad
-import Text.PrettyPrint
+import GF.Text.Pretty
 
 computeLType :: SourceGrammar -> Context -> Type -> Check Type
 computeLType gr g0 t = comp (reverse [(b,x, Vr x) | (b,x,_) <- g0] ++ g0) t
@@ -436,6 +436,10 @@ checkLType gr g trm typ0 = do
       case typ of
         Table arg1 val ->
            do arg' <- checkEqLType gr g arg0 arg1 trm
+              vs1 <- allParamValues gr arg1
+              if length vs1 == length vs
+                 then return ()
+                 else checkError $ text "wrong number of values in table" <+> ppTerm Unqualified 0 trm
               vs' <- map fst `fmap` sequence [checkLType gr g v val|v<-vs]
               return (V arg' vs',typ)
 
@@ -714,4 +718,4 @@ checkLookup x g =
   case [ty | (b,y,ty) <- g, x == y] of
     []     -> checkError (text "unknown variable" <+> ppIdent x)
     (ty:_) -> return ty
--}
\ No newline at end of file
+-}
