@@ -28,7 +28,7 @@ import GF.Grammar.Predef
 import Control.Monad
 --import Data.List (sortBy)
 import Data.Maybe
-import Text.PrettyPrint
+import GF.Text.Pretty
 
 data AExp =
      AVr   Ident Val 
@@ -38,13 +38,15 @@ data AExp =
    | AFloat Double
    | AStr  String
    | AMeta MetaId Val
+   | ALet  (Ident,(Val,AExp)) AExp
    | AApp  AExp AExp Val 
    | AAbs  Ident Val AExp 
    | AProd Ident AExp AExp 
--- | AEqs  [([Exp],AExp)] --- not used
+-- -- | AEqs  [([Exp],AExp)] --- not used
    | ARecType [ALabelling]
    | AR    [AAssign]
    | AP    AExp Label Val
+   | AGlue AExp AExp
    | AData Val
   deriving (Eq,Show)
 
@@ -57,13 +59,13 @@ lookupConst :: Theory -> QIdent -> Err Val
 lookupConst th f = th f
 
 lookupVar :: Env -> Ident -> Err Val
-lookupVar g x = maybe (Bad (render (text "unknown variable" <+> ppIdent x))) return $ lookup x ((identW,uVal):g)
+lookupVar g x = maybe (Bad (render ("unknown variable" <+> x))) return $ lookup x ((identW,VClos [] (Meta 0)):g)
 -- wild card IW: no error produced, ?0 instead.
 
 type TCEnv = (Int,Env,Env)
 
-emptyTCEnv :: TCEnv
-emptyTCEnv = (0,[],[])
+--emptyTCEnv :: TCEnv
+--emptyTCEnv = (0,[],[])
 
 whnf :: Val -> Err Val
 whnf v = ---- errIn ("whnf" +++ prt v) $ ---- debug
@@ -80,7 +82,7 @@ app u v = case u of
   VClos env (Abs _ x e) -> eval ((x,v):env) e
   _ -> return $ VApp u v
   
-eval :: Env -> Exp -> Err Val
+eval :: Env -> Term -> Err Val
 eval env e = ---- errIn ("eval" +++ prt e +++ "in" +++ prEnv env) $ 
              case e of
   Vr x    -> lookupVar env x
@@ -113,10 +115,10 @@ eqVal k u1 u2 = ---- errIn (prt u1 +++ "<>" +++ prBracket (show k) +++ prt u2) $
     _ -> return [(w1,w2) | w1 /= w2]
 -- invariant: constraints are in whnf
 
-checkType :: Theory -> TCEnv -> Exp -> Err (AExp,[(Val,Val)])
+checkType :: Theory -> TCEnv -> Term -> Err (AExp,[(Val,Val)])
 checkType th tenv e = checkExp th tenv e vType
 
-checkExp :: Theory -> TCEnv -> Exp -> Val -> Err (AExp, [(Val,Val)])
+checkExp :: Theory -> TCEnv -> Term -> Val -> Err (AExp, [(Val,Val)])
 checkExp th tenv@(k,rho,gamma) e ty = do
   typ <- whnf ty
   let v = VGen k
@@ -129,7 +131,18 @@ checkExp th tenv@(k,rho,gamma) e ty = do
 	(t',cs) <- checkExp th 
                            (k+1,(x,v x):rho, (x,a'):gamma) t (VClos ((y,v x):env) b)
 	return (AAbs x a' t', cs)
-      _ -> Bad (render (text "function type expected for" <+> ppTerm Unqualified 0 e <+> text "instead of" <+> ppValue Unqualified 0 typ))
+      _ -> Bad (render ("function type expected for" <+> ppTerm Unqualified 0 e <+> "instead of" <+> ppValue Unqualified 0 typ))
+
+    Let (x, (mb_typ, e1)) e2 -> do
+      (val,e1,cs1) <- case mb_typ of
+                        Just typ -> do (_,cs1) <- checkType th tenv typ
+                                       val <- eval rho typ
+                                       (e1,cs2) <- checkExp th tenv e1 val
+                                       return (val,e1,cs1++cs2)
+                        Nothing  -> do (e1,val,cs) <- inferExp th tenv e1
+                                       return (val,e1,cs)
+      (e2,cs2) <- checkExp th (k,rho,(x,val):gamma) e2 typ
+      return (ALet (x,(val,e1)) e2, cs1++cs2)
 
     Prod _ x a b -> do
       testErr (typ == vType) "expected Type"
@@ -141,24 +154,28 @@ checkExp th tenv@(k,rho,gamma) e ty = do
       case typ of
         VRecType ys -> do case [l | (l,_) <- ys, isNothing (lookup l xs)] of
                             [] -> return ()
-                            ls -> fail (render (text "no value given for label:" <+> fsep (punctuate comma (map ppLabel ls))))
+                            ls -> fail (render ("no value given for label:" <+> fsep (punctuate ',' ls)))
                           r <- mapM (checkAssign th tenv ys) xs
                           let (xs,css) = unzip r
                           return (AR xs, concat css)
-        _           -> Bad (render (text "record type expected for" <+> ppTerm Unqualified 0 e <+> text "instead of" <+> ppValue Unqualified 0 typ))
+        _           -> Bad (render ("record type expected for" <+> ppTerm Unqualified 0 e <+> "instead of" <+> ppValue Unqualified 0 typ))
 
     P r l -> do (r',cs) <- checkExp th tenv r (VRecType [(l,typ)])
                 return (AP r' l typ,cs)
 
+    Glue x y -> do cs1 <- eqVal k valAbsFloat typ
+                   (x,cs2) <- checkExp th tenv x typ
+                   (y,cs3) <- checkExp th tenv y typ
+                   return (AGlue x y,cs1++cs2++cs3)
     _ -> checkInferExp th tenv e typ
 
-checkInferExp :: Theory -> TCEnv -> Exp -> Val -> Err (AExp, [(Val,Val)])
+checkInferExp :: Theory -> TCEnv -> Term -> Val -> Err (AExp, [(Val,Val)])
 checkInferExp th tenv@(k,_,_) e typ = do
   (e',w,cs1) <- inferExp th tenv e
   cs2 <- eqVal k w typ
   return (e',cs1 ++ cs2)
       
-inferExp :: Theory -> TCEnv -> Exp -> Err (AExp, Val, [(Val,Val)])
+inferExp :: Theory -> TCEnv -> Term -> Err (AExp, Val, [(Val,Val)])
 inferExp th tenv@(k,rho,gamma) e = case e of
    Vr x -> mkAnnot (AVr x) $ noConstr $ lookupVar gamma x
    Q (m,c) | m == cPredefAbs && isPredefCat c
@@ -172,6 +189,16 @@ inferExp th tenv@(k,rho,gamma) e = case e of
    RecType xs -> do r <- mapM (checkLabelling th tenv) xs
                     let (xs,css) = unzip r
                     return (ARecType xs, vType, concat css)
+   Let (x, (mb_typ, e1)) e2 -> do
+    (val1,e1,cs1) <- case mb_typ of
+                       Just typ -> do (_,cs1) <- checkType th tenv typ
+                                      val <- eval rho typ
+                                      (e1,cs2) <- checkExp th tenv e1 val
+                                      return (val,e1,cs1++cs2)
+                       Nothing  -> do (e1,val,cs) <- inferExp th tenv e1
+                                      return (val,e1,cs)
+    (e2,val2,cs2) <- inferExp th (k,rho,(x,val1):gamma) e2
+    return (ALet (x,(val1,e1)) e2, val2, cs1++cs2)
    App f t -> do
     (f',w,csf) <- inferExp th tenv f 
     typ <- whnf w
@@ -180,8 +207,8 @@ inferExp th tenv@(k,rho,gamma) e = case e of
         (a',csa) <- checkExp th tenv t (VClos env a)
 	b' <- whnf $ VClos ((x,VClos rho t):env) b
 	return $ (AApp f' a' b', b', csf ++ csa)
-      _ -> Bad (render (text "Prod expected for function" <+> ppTerm Unqualified 0 f <+> text "instead of" <+> ppValue Unqualified 0 typ))
-   _ -> Bad (render (text "cannot infer type of expression" <+> ppTerm Unqualified 0 e))
+      _ -> Bad (render ("Prod expected for function" <+> ppTerm Unqualified 0 f <+> "instead of" <+> ppValue Unqualified 0 typ))
+   _ -> Bad (render ("cannot infer type of expression" <+> ppTerm Unqualified 0 e))
 
 checkLabelling :: Theory -> TCEnv -> Labelling -> Err (ALabelling, [(Val,Val)])
 checkLabelling th tenv (lbl,typ) = do
@@ -204,7 +231,7 @@ checkAssign th tenv@(k,rho,gamma) typs (lbl,(Nothing,exp)) = do
     Just val -> do (aexp,cs) <- checkExp th tenv exp val
                    return ((lbl,(val,aexp)),cs)
 
-checkBranch :: Theory -> TCEnv -> Equation -> Val -> Err (([Exp],AExp),[(Val,Val)])
+checkBranch :: Theory -> TCEnv -> Equation -> Val -> Err (([Term],AExp),[(Val,Val)])
 checkBranch th tenv b@(ps,t) ty = errIn ("branch" +++ show b) $ 
                                   chB tenv' ps' ty 
  where 
@@ -223,7 +250,7 @@ checkBranch th tenv b@(ps,t) ty = errIn ("branch" +++ show b) $
           let tenv' = (length binds, sigma ++ rho, binds ++ gamma)
           ((ps',exp),cs2) <- chB tenv' ps2 (VClos ((y,p'):env) b)
 	  return ((p:ps',exp), cs1 ++ cs2) -- don't change the patt
-        _ -> Bad (render (text "Product expected for definiens" <+> ppTerm Unqualified 0 t <+> text "instead of" <+> ppValue Unqualified 0 typ))
+        _ -> Bad (render ("Product expected for definiens" <+> ppTerm Unqualified 0 t <+> "instead of" <+> ppValue Unqualified 0 typ))
     [] -> do
       (e,cs) <- checkExp th tenv t ty
       return (([],e),cs)
@@ -244,12 +271,12 @@ checkBranch th tenv b@(ps,t) ty = errIn ("branch" +++ show b) $
                     where (xss,j,g',k') = foldr p2t ([],i,g,k) xs
      PImplArg p -> p2t p (ps,i,g,k)
      PTilde t -> (t : ps, i, g, k)
-     _ -> error $ render (text "undefined p2t case" <+> ppPatt Unqualified 0 p <+> text "in checkBranch")
+     _ -> error $ render ("undefined p2t case" <+> ppPatt Unqualified 0 p <+> "in checkBranch")
 
   upd x k g = (x, VGen k x) : g --- hack to recognize pattern variables
 
 
-checkPatt :: Theory -> TCEnv -> Exp -> Val -> Err (Binds,[(Val,Val)])
+checkPatt :: Theory -> TCEnv -> Term -> Val -> Err (Binds,[(Val,Val)])
 checkPatt th tenv exp val = do
   (aexp,_,cs) <- checkExpP tenv exp val
   let binds = extrBinds aexp
@@ -282,8 +309,8 @@ checkPatt th tenv exp val = do
            (a',_,csa) <- checkExpP tenv t (VClos env a)
 	   b' <- whnf $ VClos ((x,VClos rho t):env) b
 	   return $ (AApp f' a' b', b', csf ++ csa)
-         _ -> Bad (render (text "Prod expected for function" <+> ppTerm Unqualified 0 f <+> text "instead of" <+> ppValue Unqualified 0 typ))
-     _ -> Bad (render (text "cannot typecheck pattern" <+> ppTerm Unqualified 0 exp))
+         _ -> Bad (render ("Prod expected for function" <+> ppTerm Unqualified 0 f <+> "instead of" <+> ppValue Unqualified 0 typ))
+     _ -> Bad (render ("cannot typecheck pattern" <+> ppTerm Unqualified 0 exp))
 
 -- auxiliaries
 
