@@ -14,52 +14,46 @@
 -- Copyright (c) Aarne Ranta 1998-2000, under GNU General Public License (see GPL)
 -----------------------------------------------------------------------------
 
-module GF.Data.Operations (-- * misc functions
-		   ifNull,
-
-		   -- * the Error monad
-		   Err(..), err, maybeErr, testErr, errVal, errIn, 
+module GF.Data.Operations (
+		   -- ** The Error monad
+		   Err(..), err, maybeErr, testErr, fromErr, errIn, 
 		   lookupErr,
-		   mapPairListM, mapPairsM, pairM,
-		   singleton, --mapsErr, mapsErrTree,
+
+		   -- ** Error monad class
+		   ErrorMonad(..), checks, --doUntil, allChecks, checkAgain,
+                   liftErr,
 		   
-		   -- ** checking
+		   -- ** Checking
 		   checkUnique, unifyMaybeBy, unifyMaybe,
 
-		   -- * binary search trees; now with FiniteMap
-		   BinTree, emptyBinTree, isInBinTree, justLookupTree,
+                   -- ** Monadic operations on lists and pairs
+		   mapPairListM, mapPairsM, pairM,
+
+		   -- ** Binary search trees; now with FiniteMap
+		   BinTree, emptyBinTree, isInBinTree, --justLookupTree,
 		   lookupTree, --lookupTreeMany,
                    lookupTreeManyAll, updateTree,
 		   buildTree, filterBinTree,
-		   --sorted2tree,
                    mapTree, --mapMTree,
                    tree2list,
  
-
-		   -- * printing
-		   indent, (+++), (++-), (++++), (+++++),
+		   -- ** Printing
+		   indent, (+++), (++-), (++++), (+++-), (+++++),
 		   prUpper, prReplicate, prTList, prQuotedString, prParenth, prCurly, 
 		   prBracket, prArgList, prSemicList, prCurlyList, restoreEscapes,
 		   numberedParagraphs, prConjList, prIfEmpty, wrapLines,
 
-		   -- * extra
-		   combinations,
-
-		   -- * topological sorting with test of cyclicity
+		   -- ** Topological sorting
 		   topoTest, topoTest2,
 
-		   -- * the generic fix point iterator
-		   iterFix,
-
-		   -- * chop into separator-separated parts
-		   chunks, readIntArg, 
-
-		   -- * state monad with error; from Agda 6\/11\/2001
-		   STM(..), appSTM, stm, stmr, readSTM, updateSTM, writeSTM, done,
-
-		   -- * error monad class
-		   ErrorMonad(..), checkAgain, checks, allChecks, doUntil,
-                   liftErr
+		   -- ** Misc
+		   ifNull,
+		   combinations, done, readIntArg, --singleton,
+		   iterFix,  chunks,
+{-
+		   -- ** State monad with error; from Agda 6\/11\/2001
+		   STM(..), appSTM, stm, stmr, readSTM, updateSTM, writeSTM,
+-}
                 
 		  ) where
 
@@ -67,7 +61,8 @@ import Data.Char (isSpace, toUpper, isSpace, isDigit)
 import Data.List (nub, partition, (\\))
 import qualified Data.Map as Map
 import Data.Map (Map)
-import Control.Monad (liftM,liftM2)
+--import Control.Applicative(Applicative(..))
+import Control.Monad (liftM,liftM2) --,ap
 
 import GF.Data.ErrM
 import GF.Data.Relation
@@ -82,21 +77,12 @@ ifNull b f xs = if null xs then b else f xs
 
 -- the Error monad
 
--- | analogue of @maybe@
-err :: (String -> b) -> (a -> b) -> Err a -> b 
-err d f e = case e of
-  Ok a -> f a
-  Bad s -> d s
-
--- | add msg s to @Maybe@ failures
+-- | Add msg s to 'Maybe' failures
 maybeErr :: ErrorMonad m => String -> Maybe a -> m a
 maybeErr s = maybe (raise s) return
 
 testErr :: ErrorMonad m => Bool -> String -> m ()
-testErr cond msg = if cond then return () else raise msg
-
-errVal :: a -> Err a -> a
-errVal a = err (const a) id
+testErr cond msg = if cond then done else raise msg
 
 errIn :: ErrorMonad m => String -> m a -> m a
 errIn msg m = handle m (\s -> raise (s ++++ "OCCURRED IN" ++++ msg))
@@ -110,12 +96,9 @@ mapPairListM f xys = mapM (\ p@(x,_) -> liftM ((,) x) (f p)) xys
 mapPairsM :: Monad m => (b -> m c) -> [(a,b)] -> m [(a,c)]
 mapPairsM f xys = mapM (\ (x,y) -> liftM ((,) x) (f y)) xys
 
-pairM :: Monad a => (b -> a c) -> (b,b) -> a (c,c)
+pairM :: Monad m => (b -> m c) -> (b,b) -> m (c,c)
 pairM op (t1,t2) = liftM2 (,) (op t1) (op t2)
 
-singleton :: a -> [a]
-singleton = (:[])
-
 -- checking
 
 checkUnique :: (Show a, Eq a) => [a] -> [String]
@@ -143,21 +126,14 @@ emptyBinTree = Map.empty
 
 isInBinTree :: (Ord a) => a -> BinTree a b -> Bool
 isInBinTree = Map.member
-
-justLookupTree :: (Monad m,Ord a) => a -> BinTree a b -> m b
-justLookupTree = lookupTree (const [])
-
-lookupTree :: (Monad m,Ord a) => (a -> String) -> a -> BinTree a b -> m b
-lookupTree pr x tree = case Map.lookup x tree of
-  Just y -> return y
-  _ -> fail ("no occurrence of element" +++ pr x)
 {-
-lookupTreeMany :: Ord a => (a -> String) -> [BinTree a b] -> a -> Err b
-lookupTreeMany pr (t:ts) x = case lookupTree pr x t of
-  Ok v -> return v
-  _ -> lookupTreeMany pr ts x
-lookupTreeMany pr [] x = Bad $ "failed to find" +++ pr x
+justLookupTree :: (ErrorMonad m,Ord a) => a -> BinTree a b -> m b
+justLookupTree = lookupTree (const [])
 -}
+lookupTree :: (ErrorMonad m,Ord a) => (a -> String) -> a -> BinTree a b -> m b
+lookupTree pr x = maybeErr no . Map.lookup x
+  where no = "no occurrence of element" +++ pr x
+
 lookupTreeManyAll :: Ord a => (a -> String) -> [BinTree a b] -> a -> [b]
 lookupTreeManyAll pr (t:ts) x = case lookupTree pr x t of
   Ok v -> v : lookupTreeManyAll pr ts x
@@ -169,16 +145,10 @@ updateTree (a,b) = Map.insert a b
 
 buildTree :: (Ord a) => [(a,b)] -> BinTree a b
 buildTree = Map.fromList
-{-
-sorted2tree :: Ord a => [(a,b)] -> BinTree a b
-sorted2tree = Map.fromAscList
--}
+
 mapTree :: ((a,b) -> c) -> BinTree a b -> BinTree a c
 mapTree f = Map.mapWithKey (\k v -> f (k,v))
-{-
-mapMTree :: (Ord a,Monad m) => ((a,b) -> m c) -> BinTree a b -> m (BinTree a c)
-mapMTree f t = liftM Map.fromList $ sequence [liftM ((,) k) (f (k,x)) | (k,x) <- Map.toList t]
--}
+
 filterBinTree :: Ord a => (a -> b -> Bool) -> BinTree a b -> BinTree a b
 filterBinTree = Map.filterWithKey
 
@@ -190,13 +160,20 @@ tree2list = Map.toList
 indent :: Int -> String -> String
 indent i s = replicate i ' ' ++ s
 
-(+++), (++-), (++++), (+++++) :: String -> String -> String
+(+++), (++-), (++++), (+++-), (+++++) :: String -> String -> String
 a +++ b   = a ++ " "    ++ b
+
 a ++- ""  = a 
 a ++- b   = a +++ b
+
 a ++++ b  = a ++ "\n"   ++ b
+
+a +++- "" = a
+a +++- b  = a ++++ b
+
 a +++++ b = a ++ "\n\n" ++ b
 
+
 prUpper :: String -> String
 prUpper s = s1 ++ s2' where
  (s1,s2) = span isSpace s
@@ -268,18 +245,24 @@ wrapLines n s@(c:cs) =
 
 --- optWrapLines = if argFlag "wraplines" True then wrapLines 0 else id
 
--- | 'combinations' is the same as @sequence@!!!
+-- | 'combinations' is the same as 'sequence'!!!
 -- peb 30\/5-04
 combinations :: [[a]] -> [[a]]
 combinations t = case t of 
   []    -> [[]]
   aa:uu -> [a:u | a <- aa, u <- combinations uu]
 
--- | topological sorting with test of cyclicity
+{-
+-- | 'singleton' is the same as 'return'!!!
+singleton :: a -> [a]
+singleton = (:[])
+-}
+
+-- | Topological sorting with test of cyclicity
 topoTest :: Ord a => [(a,[a])] -> Either [a] [[a]]
 topoTest = topologicalSort . mkRel'
 
--- | topological sorting with test of cyclicity, new version /TH 2012-06-26
+-- | Topological sorting with test of cyclicity, new version /TH 2012-06-26
 topoTest2 :: Ord a => [(a,[a])] -> Either [[a]] [[a]]
 topoTest2 g0 = maybe (Right cycles) Left (tsort g)
   where
@@ -295,7 +278,7 @@ topoTest2 g0 = maybe (Right cycles) Left (tsort g)
 	  where leaves = map fst ns
 
 
--- | the generic fix point iterator
+-- | Fix point iterator (for computing e.g. transitive closures or reachability)
 iterFix :: Eq a => ([a] -> [a]) -> [a] -> [a]
 iterFix more start = iter start start 
   where
@@ -314,7 +297,7 @@ chunks sep ws = case span (/= sep) ws of
 readIntArg :: String -> Int
 readIntArg n = if (not (null n) && all isDigit n) then read n else 0
 
-
+{-
 -- state monad with error; from Agda 6/11/2001
 
 newtype STM s a = STM (s -> Err (a,s)) 
@@ -330,6 +313,10 @@ stmr f = stm (\s -> return (f s))
 
 instance Functor (STM s) where fmap = liftM
 
+instance Applicative (STM s) where
+  pure = return
+  (<*>) = ap
+
 instance  Monad (STM s) where
   return a    = STM (\s -> return (a,s))
   STM c >>= f = STM (\s -> do 
@@ -345,7 +332,8 @@ updateSTM f = stmr (\s -> ((),f s))
 
 writeSTM :: s -> STM s ()
 writeSTM s = stmr (const ((),s))
-
+-}
+-- | @return ()@
 done :: Monad m => m ()
 done = return ()
 
@@ -361,28 +349,13 @@ instance ErrorMonad Err where
   handle (Bad i) f  = f i
 
 liftErr e = err raise return e
-
+{-
 instance ErrorMonad (STM s) where
   raise msg = STM (\s -> raise msg)
   handle (STM f) g = STM (\s -> (f s) 
                                 `handle` (\e -> let STM g' = (g e) in
                                                     g' s))
-{-
--- error recovery with multiple reporting AR 30/5/2008
-mapsErr :: (a -> Err b) -> [a] -> Err [b]
 
-mapsErr f = seqs . map f where
-  seqs es = case es of
-    Ok v : ms -> case seqs ms of
-      Ok vs -> return (v : vs)
-      b     -> b
-    Bad s : ms -> case seqs ms of
-      Ok vs  -> Bad s
-      Bad ss -> Bad (s +++++ ss)
-    [] -> return []
-
-mapsErrTree :: (Ord a) => ((a,b) -> Err (a,c)) -> BinTree a b -> Err (BinTree a c)
-mapsErrTree f t =  mapsErr f (tree2list t) >>= return . sorted2tree
 -}
 
 -- | if the first check fails try another one
@@ -392,7 +365,7 @@ checkAgain c1 c2 = handle_ c1 c2
 checks :: ErrorMonad m => [m a] -> m a
 checks [] = raise "no chance to pass"
 checks cs = foldr1 checkAgain cs
-
+{-
 allChecks :: ErrorMonad m => [m a] -> m [a]
 allChecks ms = case ms of
   (m: ms) -> let rs = allChecks ms in handle_ (liftM2 (:) m rs) rs
@@ -404,3 +377,4 @@ doUntil cond ms = case ms of
     v <- a
     if cond v then return v else doUntil cond as
   _ -> raise "no result"
+-}
\ No newline at end of file
