@@ -16,7 +16,7 @@ module GF.Data.Utilities(module GF.Data.Utilities, module PGF.Utilities) where
 
 import Data.Maybe
 import Data.List
-import Control.Monad (MonadPlus(..),liftM)
+import Control.Monad (MonadPlus(..),liftM,when)
 import PGF.Utilities
 
 -- * functions on lists
@@ -136,6 +136,10 @@ mapBoth = map . apBoth
 whenMP :: MonadPlus m => Bool -> a -> m a
 whenMP b x = if b then return x else mzero
 
+whenM bm m = flip when m =<< bm
+
+repeatM m = whenM m (repeatM m)
+
 -- * functions on Maybes
 
 -- | Returns true if the argument is Nothing or Just []
