@@ -22,6 +22,7 @@ module GF.Grammar.Lookup (
            lookupResDef, lookupResDefLoc,
            lookupResType, 
            lookupOverload,
+           lookupOverloadTypes,
            lookupParamValues, 
            allParamValues,
            lookupAbsDef, 
@@ -42,7 +43,7 @@ import GF.Grammar.Lockfield
 import Data.List (sortBy)
 --import Data.Maybe (maybe)
 --import Control.Monad
-import Text.PrettyPrint
+import GF.Text.Pretty
 import qualified Data.Map as Map
 
 -- whether lock fields are added in reuse
@@ -59,10 +60,10 @@ lookupIdent c t =
 lookupIdentInfo :: ErrorMonad m => SourceModInfo -> Ident -> m Info
 lookupIdentInfo mo i = lookupIdent i (jments mo)
 
-lookupQIdentInfo :: ErrorMonad m => SourceGrammar -> QIdent -> m Info
+lookupQIdentInfo :: ErrorMonad m => Grammar -> QIdent -> m Info
 lookupQIdentInfo gr (m,c) = flip lookupIdentInfo c =<< lookupModule gr m
 
-lookupResDef :: ErrorMonad m => SourceGrammar -> QIdent -> m Term
+lookupResDef :: ErrorMonad m => Grammar -> QIdent -> m Term
 lookupResDef gr x = fmap unLoc (lookupResDefLoc gr x)
 
 lookupResDefLoc gr (m,c)
@@ -83,9 +84,9 @@ lookupResDefLoc gr (m,c)
         AnyInd _ n        -> look n c
         ResParam _ _      -> return (noLoc (QC (m,c)))
         ResValue _        -> return (noLoc (QC (m,c)))
-        _   -> raise $ render (ppIdent c <+> text "is not defined in resource" <+> ppIdent m)
+        _   -> raise $ render (c <+> "is not defined in resource" <+> m)
 
-lookupResType :: ErrorMonad m => SourceGrammar -> QIdent -> m Type
+lookupResType :: ErrorMonad m => Grammar -> QIdent -> m Type
 lookupResType gr (m,c) = do
   info <- lookupQIdentInfo gr (m,c)
   case info of
@@ -99,9 +100,31 @@ lookupResType gr (m,c) = do
     AnyInd _ n        -> lookupResType gr (n,c)
     ResParam _ _      -> return typePType
     ResValue (L _ t)  -> return t
-    _   -> raise $ render (ppIdent c <+> text "has no type defined in resource" <+> ppIdent m)
+    _   -> raise $ render (c <+> "has no type defined in resource" <+> m)
 
-lookupOverload :: ErrorMonad m => SourceGrammar -> QIdent -> m [([Type],(Type,Term))]
+lookupOverloadTypes :: ErrorMonad m => Grammar -> QIdent -> m [(Term,Type)]
+lookupOverloadTypes gr id@(m,c) = do
+  info <- lookupQIdentInfo gr (m,c)
+  case info of
+    ResOper (Just (L _ ty)) _ -> ret ty
+
+    -- used in reused concrete
+    CncCat _ _ _ _ _ -> ret typeType
+    CncFun (Just (cat,cont,val)) _ _ _ -> do
+          val' <- lock cat val 
+          ret $ mkProd cont val' []
+    ResParam _ _      -> ret typePType
+    ResValue (L _ t)  -> ret t
+    ResOverload os tysts -> do
+            tss <- mapM (\x -> lookupOverloadTypes gr (x,c)) os
+            return $ [(tr,ty) | (L _ ty,L _ tr) <- tysts] ++
+                     concat tss
+    AnyInd _ n   -> lookupOverloadTypes gr (n,c)
+    _            -> raise $ render (c <+> "has no types defined in resource" <+> m)
+  where
+    ret ty = return [(Q id,ty)]
+
+lookupOverload :: ErrorMonad m => Grammar -> QIdent -> m [([Type],(Type,Term))]
 lookupOverload gr (m,c) = do
     info <- lookupQIdentInfo gr (m,c)
     case info of
@@ -112,29 +135,29 @@ lookupOverload gr (m,c) = do
                      concat tss
 
       AnyInd _ n  -> lookupOverload gr (n,c)
-      _   -> raise $ render (ppIdent c <+> text "is not an overloaded operation")
+      _   -> raise $ render (c <+> "is not an overloaded operation")
 
 -- | returns the original 'Info' and the module where it was found
-lookupOrigInfo :: ErrorMonad m => SourceGrammar -> QIdent -> m (Ident,Info)
+lookupOrigInfo :: ErrorMonad m => Grammar -> QIdent -> m (ModuleName,Info)
 lookupOrigInfo gr (m,c) = do
   info <- lookupQIdentInfo gr (m,c)
   case info of
     AnyInd _ n  -> lookupOrigInfo gr (n,c)
     i           -> return (m,i)
 
-allOrigInfos :: SourceGrammar -> Ident -> [(QIdent,Info)]
-allOrigInfos gr m = errVal [] $ do
+allOrigInfos :: Grammar -> ModuleName -> [(QIdent,Info)]
+allOrigInfos gr m = fromErr [] $ do
   mo <- lookupModule gr m
   return [((m,c),i) | (c,_) <- tree2list (jments mo), Ok (m,i) <- [lookupOrigInfo gr (m,c)]]
 
-lookupParamValues :: ErrorMonad m => SourceGrammar -> QIdent -> m [Term]
+lookupParamValues :: ErrorMonad m => Grammar -> QIdent -> m [Term]
 lookupParamValues gr c = do
   (_,info) <- lookupOrigInfo gr c
   case info of
     ResParam _ (Just pvs) -> return pvs
-    _                     -> raise $ render (ppQIdent Qualified c <+> text "has no parameter values defined")
+    _                     -> raise $ render (ppQIdent Qualified c <+> "has no parameter values defined")
 
-allParamValues :: ErrorMonad m => SourceGrammar -> Type -> m [Term]
+allParamValues :: ErrorMonad m => Grammar -> Type -> m [Term]
 allParamValues cnc ptyp =
   case ptyp of
     _ | Just n <- isTypeInts ptyp -> return [EInt i | i <- [0..n]]
@@ -148,51 +171,51 @@ allParamValues cnc ptyp =
        pvs <- allParamValues cnc pt
        vvs <- allParamValues cnc vt
        return [V pt ts | ts <- combinations (replicate (length pvs) vvs)]
-    _ -> raise (render (text "cannot find parameter values for" <+> ppTerm Unqualified 0 ptyp))
+    _ -> raise (render ("cannot find parameter values for" <+> ptyp))
   where
     -- to normalize records and record types
     sortByFst = sortBy (\ x y -> compare (fst x) (fst y))
 
-lookupAbsDef :: ErrorMonad m => SourceGrammar -> Ident -> Ident -> m (Maybe Int,Maybe [Equation])
-lookupAbsDef gr m c = errIn (render (text "looking up absdef of" <+> ppIdent c)) $ do
+lookupAbsDef :: ErrorMonad m => Grammar -> ModuleName -> Ident -> m (Maybe Int,Maybe [Equation])
+lookupAbsDef gr m c = errIn (render ("looking up absdef of" <+> c)) $ do
   info <- lookupQIdentInfo gr (m,c)
   case info of
     AbsFun _ a d _ -> return (a,fmap (map unLoc) d)
     AnyInd _ n     -> lookupAbsDef gr n c
     _              -> return (Nothing,Nothing)
 
-lookupLincat :: ErrorMonad m => SourceGrammar -> Ident -> Ident -> m Type
+lookupLincat :: ErrorMonad m => Grammar -> ModuleName -> Ident -> m Type
 lookupLincat gr m c | isPredefCat c = return defLinType --- ad hoc; not needed?
 lookupLincat gr m c = do
   info <- lookupQIdentInfo gr (m,c)
   case info of
     CncCat (Just (L _ t)) _ _ _ _ -> return t
     AnyInd _ n                    -> lookupLincat gr n c
-    _                             -> raise (render (ppIdent c <+> text "has no linearization type in" <+> ppIdent m))
+    _                             -> raise (render (c <+> "has no linearization type in" <+> m))
 
 -- | this is needed at compile time
-lookupFunType :: ErrorMonad m => SourceGrammar -> Ident -> Ident -> m Type
+lookupFunType :: ErrorMonad m => Grammar -> ModuleName -> Ident -> m Type
 lookupFunType gr m c = do
   info <- lookupQIdentInfo gr (m,c)
   case info of
     AbsFun (Just (L _ t)) _ _ _ -> return t
     AnyInd _ n                  -> lookupFunType gr n c
-    _                           -> raise (render (text "cannot find type of" <+> ppIdent c))
+    _                           -> raise (render ("cannot find type of" <+> c))
 
 -- | this is needed at compile time
-lookupCatContext :: ErrorMonad m => SourceGrammar -> Ident -> Ident -> m Context
+lookupCatContext :: ErrorMonad m => Grammar -> ModuleName -> Ident -> m Context
 lookupCatContext gr m c = do
   info <- lookupQIdentInfo gr (m,c)
   case info of
     AbsCat (Just (L _ co)) -> return co
     AnyInd _ n             -> lookupCatContext gr n c
-    _                      -> raise (render (text "unknown category" <+> ppIdent c))
+    _                      -> raise (render ("unknown category" <+> c))
 
 
 -- this gives all opers and param constructors, also overloaded opers and funs, and the types, and locations
 -- notice that it only gives the modules that are reachable and the opers that are included
 
-allOpers :: SourceGrammar -> [((Ident,Ident),Type,Location)]
+allOpers :: Grammar -> [(QIdent,Type,Location)]
 allOpers gr = 
   [((m,op),typ,loc) |
       (m,mi)  <- maybe [] (allExtends gr) (greatestResource gr),
@@ -214,7 +237,7 @@ allOpers gr =
                     _ -> typ
 
 --- not for dependent types
-allOpersTo :: SourceGrammar -> Type -> [((Ident,Ident),Type,Location)]
+allOpersTo :: Grammar -> Type -> [(QIdent,Type,Location)]
 allOpersTo gr ty = [op | op@(_,typ,_) <- allOpers gr, isProdTo ty typ] where
   isProdTo t typ = eqProd typ t || case typ of
     Prod _ _ a b -> isProdTo t b
