@@ -10,16 +10,16 @@
 
 module GF.Grammar.Predef where
 
-import GF.Infra.Ident(Ident,identS)
+import GF.Infra.Ident(Ident,identS,moduleNameS)
 
 cType = identS "Type"
 cPType = identS "PType"
 cTok = identS "Tok"
 cStr = identS "Str"
 cStrs = identS "Strs"
-cPredefAbs = identS "PredefAbs"
-cPredefCnc = identS "PredefCnc"
-cPredef = identS "Predef"
+cPredefAbs = moduleNameS "PredefAbs"
+cPredefCnc = moduleNameS "PredefCnc"
+cPredef = moduleNameS "Predef"
 cInt = identS "Int"
 cFloat = identS "Float"
 cString = identS "String"
@@ -32,6 +32,9 @@ cUndefinedType = identS "UndefinedType"
 cNonExist = identS "nonExist"
 cBIND = identS "BIND"
 cSOFT_BIND = identS "SOFT_BIND"
+cSOFT_SPACE = identS "SOFT_SPACE"
+cCAPIT = identS "CAPIT"
+cALL_CAPIT = identS "ALL_CAPIT"
 
 isPredefCat :: Ident -> Bool
 isPredefCat c = elem c [cInt,cString,cFloat]
@@ -58,6 +61,7 @@ cRead = identS "read"
 cToStr = identS "toStr"
 cMapStr = identS "mapStr"
 cError = identS "error"
+cTrace = identS "trace"
 
 -- * Hacks: dummy identifiers used in various places.
 -- Not very nice!
