@@ -13,7 +13,7 @@
 -----------------------------------------------------------------------------
 
 module GF.Infra.CheckM
-          (Check, CheckResult, Message, runCheck,
+          (Check, CheckResult, Message, runCheck, runCheck',
 	   checkError, checkCond, checkWarn, checkWarnings, checkAccumError,
 	   checkIn, checkInModule, checkMap, checkMapRecover,
            parallelCheck, accumulateError, commitCheck,
@@ -21,14 +21,16 @@ module GF.Infra.CheckM
 
 import GF.Data.Operations
 --import GF.Infra.Ident
-import GF.Grammar.Grammar(msrc) -- ,Context
-import GF.Grammar.Printer(ppLocation)
+--import GF.Grammar.Grammar(msrc) -- ,Context
+import GF.Infra.Location(ppLocation,sourcePath)
+import GF.Infra.Option(Options,noOptions,verbAtLeast,Verbosity(..))
 
 import qualified Data.Map as Map
-import Text.PrettyPrint
+import GF.Text.Pretty
 import System.FilePath(makeRelative)
 import Control.Parallel.Strategies(parList,rseq,using)
-import Control.Monad(liftM)
+import Control.Monad(liftM,ap)
+import Control.Applicative(Applicative(..))
 
 type Message = Doc
 type Error   = Message
@@ -50,8 +52,12 @@ instance Monad Check where
                  (ws,Success x) -> unCheck (g x) {-ctxt-} ws
                  (ws,Fail msg)  -> (ws,Fail msg)
 
+instance Applicative Check where
+  pure = return
+  (<*>) = ap
+
 instance ErrorMonad Check where
-  raise s = checkError (text s)
+  raise s = checkError (pp s)
   handle f h = handle' f (h . render)
 
 handle' f h = Check (\{-ctxt-} msgs -> case unCheck f {-ctxt-} msgs of
@@ -67,9 +73,9 @@ checkCond s b = if b then return () else checkError s
 
 -- | warnings should be reversed in the end
 checkWarn :: Message -> Check ()
-checkWarn msg = Check $ \{-ctxt-} (es,ws) -> ((es,(text "Warning:" <+> msg) : ws),Success ())
+checkWarn msg = Check $ \{-ctxt-} (es,ws) -> ((es,("Warning:" <+> msg) : ws),Success ())
 
-checkWarnings = mapM_ checkWarn
+checkWarnings ms = mapM_ checkWarn ms
 
 -- | Report a nonfatal (accumulated) error
 checkAccumError :: Message -> Check ()
@@ -93,15 +99,19 @@ commitCheck c =
     list = vcat . reverse
 
 -- | Run an error check, report errors and warnings
-runCheck :: ErrorMonad m => Check a -> m (a,String)
-runCheck c =
+runCheck c = runCheck' noOptions c
+
+-- | Run an error check, report errors and (optionally) warnings
+runCheck' :: ErrorMonad m => Options -> Check a -> m (a,String)
+runCheck' opts c =
     case unCheck c {-[]-} ([],[]) of
-      (([],ws),Success v) -> return (v,render (list ws))
+      (([],ws),Success v) -> return (v,render (wlist ws))
       (msgs   ,Success v) -> bad msgs
       ((es,ws),Fail    e) -> bad ((e:es),ws)
   where
-    bad (es,ws) = raise (render $ list ws $$ list es)
+    bad (es,ws) = raise (render $ wlist ws $$ list es)
     list = vcat . reverse
+    wlist ws = if verbAtLeast opts Normal then list ws else empty
 
 parallelCheck :: [Check a] -> Check [a]
 parallelCheck cs =
@@ -151,6 +161,6 @@ checkIn msg c = Check $ \{-ctxt-} msgs0 ->
 -- | Augment error messages with a relative path to the source module and
 -- an contextual hint (which can be left 'empty')
 checkInModule cwd mi loc context =
-    checkIn (ppLocation relpath loc <> colon $$ nest 2 context)
+    checkIn (ppLocation relpath loc <> ':' $$ nest 2 context)
   where
-    relpath = makeRelative cwd (msrc mi)
+    relpath = makeRelative cwd (sourcePath mi)
