@@ -1,7 +1,7 @@
--- LANGUAGE CPP
 module GF.Infra.Option
     (
-     -- * Option types
+     -- ** Command line options
+     -- *** Option types
      Options, 
      Flags(..), 
      Mode(..), Phase(..), Verbosity(..), 
@@ -9,21 +9,21 @@ module GF.Infra.Option
      SISRFormat(..), Optimization(..), CFGTransform(..), HaskellOption(..),
      Dump(..), Pass(..), Recomp(..),
      outputFormatsExpl, 
-     -- * Option parsing 
+     -- *** Option parsing
      parseOptions, parseModuleOptions, fixRelativeLibPaths,
-     -- * Option pretty-printing
+     -- *** Option pretty-printing
      optionsGFO,
      optionsPGF,
-     -- * Option manipulation
+     -- *** Option manipulation
      addOptions, concatOptions, noOptions,
      modifyFlags,
      helpMessage,
-     -- * Checking specific options
+     -- *** Checking specific options
      flag, cfgTransform, haskellOption, readOutputFormat,
      isLexicalCat, isLiteralCat, renameEncoding, getEncoding, defaultEncoding,
-     -- * Setting specific options
+     -- *** Setting specific options
      setOptimization, setCFGTransform,
-     -- * Convenience methods for checking options    
+     -- *** Convenience methods for checking options
      verbAtLeast, dump
     ) where
 
@@ -73,7 +73,10 @@ errors = raise . unlines
 
 -- Types
 
-data Mode = ModeVersion | ModeHelp | ModeInteractive | ModeRun | ModeCompiler
+data Mode = ModeVersion | ModeHelp
+          | ModeInteractive | ModeRun
+          | ModeInteractive2 | ModeRun2
+          | ModeCompiler
           | ModeServer {-port::-}Int
   deriving (Show,Eq,Ord)
 
@@ -126,6 +129,7 @@ data CFGTransform = CFGNoLR
   deriving (Show,Eq,Ord)
 
 data HaskellOption = HaskellNoPrefix | HaskellGADT | HaskellLexical
+                   | HaskellConcrete | HaskellVariants
   deriving (Show,Eq,Ord)
 
 data Warning = WarnMissingLincat
@@ -172,9 +176,10 @@ data Flags = Flags {
       optDump            :: [Dump],
       optTagsOnly        :: Bool,
       optHeuristicFactor :: Maybe Double,
-      optMetaProb        :: Maybe Double,
-      optMetaToknProb    :: Maybe Double{-,
-      optNewComp         :: Bool-}
+      optCaseSensitive   :: Bool,
+      optPlusAsBind      :: Bool,
+      optJobs            :: Maybe (Maybe Int),
+      optTrace           :: Bool
     }
   deriving (Show)
 
@@ -217,12 +222,11 @@ optionsGFO opts = optionsPGF opts
 
 -- | Pretty-print the options that are preserved in .pgf files.
 optionsPGF :: Options -> [(String,Literal)]
-optionsPGF opts = 
+optionsPGF opts =
          maybe [] (\x -> [("language",LStr x)]) (flag optSpeechLanguage opts)
       ++ maybe [] (\x -> [("startcat",LStr x)]) (flag optStartCat opts)
       ++ maybe [] (\x -> [("heuristic_search_factor",LFlt x)]) (flag optHeuristicFactor opts)
-      ++ maybe [] (\x -> [("meta_prob",LFlt x)]) (flag optMetaProb opts)
-      ++ maybe [] (\x -> [("meta_token_prob",LFlt x)]) (flag optMetaToknProb opts)
+      ++ (if flag optCaseSensitive opts then [] else [("case_sensitive",LStr "off")])
 
 -- Option manipulation
 
@@ -284,15 +288,10 @@ defaultFlags = Flags {
       optDump            = [],
       optTagsOnly        = False,
       optHeuristicFactor = Nothing,
-      optMetaProb        = Nothing,
-      optMetaToknProb    = Nothing{-,
-      optNewComp         =
-#ifdef NEW_COMP
-                           True
-#else
-                           False
-#endif
--}
+      optCaseSensitive   = True,
+      optPlusAsBind      = False,
+      optJobs            = Nothing,
+      optTrace           = False
     }
 
 -- | Option descriptions
@@ -305,8 +304,11 @@ optDescr =
      Option ['v'] ["verbose"] (OptArg verbosity "N") "Set verbosity (default 1). -v alone is the same as -v 2.",
      Option ['q','s'] ["quiet"] (NoArg (verbosity (Just "0"))) "Quiet, same as -v 0.",
      Option [] ["batch"] (NoArg (mode ModeCompiler)) "Run in batch compiler mode.",
+     Option ['j'] ["jobs"] (OptArg jobs "N") "Compile N modules in parallel with -batch (default 1).",
      Option [] ["interactive"] (NoArg (mode ModeInteractive)) "Run in interactive mode (default).",
      Option [] ["run"] (NoArg (mode ModeRun)) "Run in interactive mode, showing output only (no other messages).",
+     Option [] ["cshell"] (NoArg (mode ModeInteractive2)) "Start the C run-time shell.",
+     Option [] ["crun"] (NoArg (mode ModeRun2)) "Start the C run-time shell, showing output only (no other messages).",
      Option [] ["server"] (OptArg modeServer "port") $
        "Run in HTTP server mode on given port (default "++show defaultPort++").",
      Option [] ["document-root"] (ReqArg gfDocuRoot "DIR")
@@ -318,6 +320,8 @@ optDescr =
      Option [] ["make"] (NoArg (liftM2 addOptions (mode ModeCompiler) (phase Link))) "Build .pgf file and other output files and exit.",
      Option [] ["cpu"] (NoArg (cpu True)) "Show compilation CPU time statistics.",
      Option [] ["no-cpu"] (NoArg (cpu False)) "Don't show compilation CPU time statistics (default).",
+--   Option ['t'] ["trace"] (NoArg (trace True)) "Trace computations",
+--   Option [] ["no-trace"] (NoArg (trace False)) "Don't trace computations",
      Option [] ["gfo-dir"] (ReqArg gfoDir "DIR") "Directory to put .gfo files in (default = '.').",
      Option ['f'] ["output-format"] (ReqArg outFmt "FMT") 
         (unlines ["Output format. FMT can be one of:",
@@ -373,10 +377,8 @@ optDescr =
      Option [] ["cse"] (onOff (toggleOptimize OptCSE) True) "Perform common sub-expression elimination (default on).",
      Option [] ["cfg"] (ReqArg cfgTransform "TRANS") "Enable or disable specific CFG transformations. TRANS = merge, no-merge, bottomup, no-bottomup, ...",
      Option [] ["heuristic_search_factor"] (ReqArg (readDouble (\d o -> o { optHeuristicFactor = Just d })) "FACTOR") "Set the heuristic search factor for statistical parsing",
-     Option [] ["meta_prob"] (ReqArg (readDouble (\d o -> o { optMetaProb = Just d })) "PROB") "Set the probability of introducting a meta variable in the parser",
-     Option [] ["meta_token_prob"] (ReqArg (readDouble (\d o -> o { optMetaToknProb = Just d })) "PROB") "Set the probability for skipping a token in the parser",
---     Option [] ["new-comp"] (NoArg (set $ \o -> o{optNewComp = True})) "Use the new experimental compiler.",
---     Option [] ["old-comp"] (NoArg (set $ \o -> o{optNewComp = False})) "Use old trusty compiler.",
+     Option [] ["case_sensitive"] (onOff (\v -> set $ \o -> o{optCaseSensitive=v}) True) "Set the parser in case-sensitive/insensitive mode [sensitive by default]",
+     Option [] ["plus-as-bind"] (NoArg (set $ \o -> o{optPlusAsBind=True})) "Uses of (+) with runtime variables automatically generate BIND (experimental feature).",
      dumpOption "source" Source,
      dumpOption "rebuild" Rebuild,
      dumpOption "extend" Extend,
@@ -385,7 +387,6 @@ optDescr =
      dumpOption "refresh" Refresh,
      dumpOption "opt" Optimize,
      dumpOption "canon" Canon
-
     ]
  where phase       x = set $ \o -> o { optStopAfterPhase = x }
        mode        x = set $ \o -> o { optMode = x }
@@ -395,12 +396,20 @@ optDescr =
            ms = mode . ModeServer
            readPort p = maybe err ms (readMaybe p)
                  where err = fail $ "Bad server port: "++p
+
+       jobs          = maybe (setjobs Nothing) number
+           where
+             number s = maybe err (setjobs . Just) (readMaybe s)
+               where err = fail $ "Bad number of jobs: " ++ s
+             setjobs j = set $ \ o -> o { optJobs = Just j }
+
        verbosity mv  = case mv of
                            Nothing -> set $ \o -> o { optVerbosity = Verbose }
                            Just v  -> case readMaybe v >>= toEnumBounded of
                                         Just i  -> set $ \o -> o { optVerbosity = i }
                                         Nothing -> fail $ "Bad verbosity: " ++ show v
        cpu         x = set $ \o -> o { optShowCPUTime = x }
+--     trace       x = set $ \o -> o { optTrace = x }
        gfoDir      x = set $ \o -> o { optGFODir = Just x }
        outFmt      x = readOutputFormat x >>= \f ->
                          set $ \o -> o { optOutputFormats = optOutputFormats o ++ [f] }
@@ -517,7 +526,9 @@ haskellOptionNames :: [(String, HaskellOption)]
 haskellOptionNames =
     [("noprefix", HaskellNoPrefix),
      ("gadt",     HaskellGADT),
-     ("lexical",  HaskellLexical)]
+     ("lexical",  HaskellLexical),
+     ("concrete", HaskellConcrete),
+     ("variants", HaskellVariants)]
 
 -- | This is for bacward compatibility. Since GHC 6.12 we
 -- started using the native Unicode support in GHC but it
