@@ -1,4 +1,3 @@
-{-# OPTIONS -cpp #-}
 ----------------------------------------------------------------------
 -- |
 -- Module      : UseIO
@@ -13,7 +12,10 @@
 -- (Description of the module)
 -----------------------------------------------------------------------------
 
-module GF.Infra.UseIO(module GF.Infra.UseIO,MonadIO(..),liftErr) where
+module GF.Infra.UseIO(-- ** Files and IO
+                      module GF.Infra.UseIO,
+                      -- *** Reused
+                      MonadIO(..),liftErr) where
 
 import Prelude hiding (catch)
 
@@ -22,7 +24,7 @@ import GF.Infra.Option
 import GF.System.Catch
 import Paths_gf(getDataDir)
 
-import System.Directory
+import GF.System.Directory
 import System.FilePath
 import System.IO
 import System.IO.Error(isUserError,ioeGetErrorString)
@@ -31,30 +33,19 @@ import System.Exit
 import System.CPUTime
 --import System.Cmd
 import Text.Printf
-import Control.Monad
+--import Control.Applicative(Applicative(..))
+import Control.Monad(when,liftM,foldM)
 import Control.Monad.Trans(MonadIO(..))
+import Control.Monad.State(StateT,lift)
 import Control.Exception(evaluate)
 
---putShow' :: Show a => (c -> a) -> c -> IO ()
---putShow' f = putStrLn . show . length . show . f
+--putIfVerb :: MonadIO io => Options -> String -> io ()
+putIfVerb opts msg = when (verbAtLeast opts Verbose) $ putStrLnE msg
 
-putIfVerb :: MonadIO io => Options -> String -> io ()
-putIfVerb opts msg =
-      when (verbAtLeast opts Verbose) $ liftIO $ putStrLn msg
+-- *** GF files path and library path manipulation
 
-putIfVerbW :: MonadIO io => Options -> String -> io ()
-putIfVerbW opts msg =
-      when (verbAtLeast opts Verbose) $ liftIO $ putStr (' ' : msg)
-{-
-errOptIO :: Options -> a -> Err a -> IO a
-errOptIO os e m = case m of
-  Ok x  -> return x
-  Bad k -> do  
-    putIfVerb os k
-    return e
--}
 type FileName = String
-type InitPath = String
+type InitPath = String -- ^ the directory portion of a pathname
 type FullPath = String
 
 gfLibraryPath    = "GF_LIB_PATH"
@@ -67,8 +58,8 @@ getLibraryDirectory opts =
     Nothing   -> liftIO $ catch (getEnv gfLibraryPath)
                                 (\ex -> fmap (</> "lib") getDataDir)
 
-getGrammarPath :: FilePath -> IO [FilePath]
-getGrammarPath lib_dir = do
+getGrammarPath :: MonadIO io => FilePath -> io [FilePath]
+getGrammarPath lib_dir = liftIO $ do
   catch (fmap splitSearchPath $ getEnv gfGrammarPathVar) 
         (\_ -> return [lib_dir </> "alltenses",lib_dir </> "prelude"])     -- e.g. GF_GRAMMAR_PATH
 
@@ -105,9 +96,25 @@ getSubdirs dir = do
                        then return (fpath:fs)
                        else return        fs ) [] fs
 
+--------------------------------------------------------------------------------
 justModuleName :: FilePath -> String
 justModuleName = dropExtension . takeFileName
 
+isGF,isGFO :: FilePath -> Bool
+isGF  = (== ".gf")  . takeExtensions
+isGFO = (== ".gfo") . takeExtensions
+
+gfFile,gfoFile :: FilePath -> FilePath
+gfFile  f = addExtension f "gf"
+gfoFile f = addExtension f "gfo"
+
+gf2gfo :: Options -> FilePath -> FilePath
+gf2gfo = gf2gfo' . flag optGFODir
+
+gf2gfo' gfoDir file = maybe (gfoFile (dropExtension file))
+                            (\dir -> dir </> gfoFile (takeBaseName file))
+                            gfoDir
+--------------------------------------------------------------------------------
 splitInModuleSearchPath :: String -> [FilePath]
 splitInModuleSearchPath s = case break isPathSep s of
   (f,_:cs) -> f : splitInModuleSearchPath cs
@@ -118,33 +125,51 @@ splitInModuleSearchPath s = case break isPathSep s of
 
 --
 
--- * IO monad with error; adapted from state monad
+-- *** Error handling in the IO monad
 
-newtype IOE a = IOE { appIOE :: IO (Err a) }
+-- | Was: @newtype IOE a = IOE { appIOE :: IO (Err a) }@
+type IOE a = IO a
 
-ioe :: IO (Err a) -> IOE a
-ioe = IOE
+--ioe :: IO (Err a) -> IOE a
+--ioe io = err fail return =<< io
 
-instance MonadIO IOE where liftIO io = ioe (io >>= return . return)
+-- | Catch exceptions caused by calls to 'raise' or 'fail' in the 'IO' monad.
+-- To catch all 'IO' exceptions, use 'try' instead.
+tryIOE :: IOE a -> IO (Err a)
+tryIOE ioe = handle (fmap Ok ioe) (return . Bad)
 
-instance ErrorMonad IOE where
-  raise = ioe . return . Bad
-  handle m h = ioe $ err (appIOE . h) (return . Ok) =<< appIOE m
+--runIOE :: IOE a -> IO a
+--runIOE = id
 
+-- instance MonadIO IOE where liftIO io = ioe (io >>= return . return)
+
+-- | Make raise and handle mimic behaviour of the old IOE monad
+instance ErrorMonad IO where
+  raise = fail
+  handle m h = catch m $ \ e -> if isUserError e
+                                then h (ioeGetErrorString e)
+                                else ioError e
+{-
 instance Functor IOE where fmap = liftM
 
+instance Applicative IOE where
+  pure = return
+  (<*>) = ap
+
 instance  Monad IOE where
   return a    = ioe (return (return a))
   IOE c >>= f = IOE $ do 
                   x <- c          -- Err a
                   appIOE $ err raise f x         -- f :: a -> IOE a
   fail = raise
+-}
+
+-- | Print the error message and return a default value if the IO operation 'fail's
+useIOE :: a -> IOE a -> IO a
+useIOE a ioe = handle ioe (\s -> putStrLn s >> return a)
 
 maybeIO io = either (const Nothing) Just `fmap` liftIO (try io)
-
-useIOE :: a -> IOE a -> IO a
-useIOE a ioe = appIOE ioe >>= err (\s -> putStrLn s >> return a) return 
-
+{-
 --foldIOE :: (a -> b -> IOE a) -> a -> [b] -> IOE (a, Maybe String)
 foldIOE f s xs = case xs of
   [] -> return (s,Nothing)
@@ -153,33 +178,65 @@ foldIOE f s xs = case xs of
     case ev of 
       Ok v  -> foldIOE f v xx
       Bad m -> return $ (s, Just m)
-
+-}
 die :: String -> IO a
 die s = do hPutStrLn stderr s
            exitFailure
 
-ePutStr, ePutStrLn, putStrE, putStrLnE :: MonadIO m => String -> m ()
-ePutStr s = liftIO $ hPutStr stderr s
-ePutStrLn s = liftIO $ hPutStrLn stderr s
-putStrLnE s = liftIO $ putStrLn s >> hFlush stdout
-putStrE s = liftIO $ putStr s >> hFlush stdout
+-- *** Diagnostic output
 
-putPointE :: MonadIO m => Verbosity -> Options -> String -> m a -> m a
+class Monad m => Output m where
+  ePutStr, ePutStrLn, putStrE, putStrLnE :: String -> m ()
+
+instance Output IO where
+  ePutStr   s = hPutStr stderr s `catch` oops
+    where oops _ = return () -- prevent crash on character encoding problem
+  ePutStrLn s = hPutStrLn stderr s `catch` oops
+    where oops _ = ePutStrLn "" -- prevent crash on character encoding problem
+  putStrLnE s = putStrLn s >> hFlush stdout
+  putStrE   s = putStr s >> hFlush stdout
+{-
+instance Output IOE where
+  ePutStr   = liftIO . ePutStr
+  ePutStrLn = liftIO . ePutStrLn
+  putStrLnE = liftIO . putStrLnE
+  putStrE   = liftIO . putStrE
+-}
+
+instance Output m => Output (StateT s m) where
+  ePutStr = lift . ePutStr
+  ePutStrLn = lift . ePutStrLn
+  putStrE = lift . putStrE
+  putStrLnE = lift . putStrLnE
+
+--putPointE :: Verbosity -> Options -> String -> IO a -> IO a
 putPointE v opts msg act = do
   when (verbAtLeast opts v) $ putStrE msg
 
-  t1 <- liftIO $ getCPUTime
-  a <- act >>= liftIO . evaluate
-  t2 <- liftIO $ getCPUTime
+  (t,a) <- timeIt act
 
   if flag optShowCPUTime opts
-      then do let msec = (t2 - t1) `div` 1000000000
+      then do let msec = t `div` 1000000000
               putStrLnE (printf " %5d msec" msec)
       else when (verbAtLeast opts v) $ putStrLnE ""
 
   return a
 
--- * File IO
+-- | Because GHC adds the confusing text "user error" for failures caused by
+-- calls to 'fail'.
+ioErrorText e = if isUserError e
+                then ioeGetErrorString e
+                else show e
+
+-- *** Timing
+
+timeIt act =
+  do t1 <- liftIO $ getCPUTime
+     a <- liftIO . evaluate =<< act
+     t2 <- liftIO $ getCPUTime
+     return (t2-t1,a)
+
+-- *** File IO
 
 writeUTF8File :: FilePath -> String -> IO ()
 writeUTF8File fpath content =
@@ -188,9 +245,3 @@ writeUTF8File fpath content =
 
 readBinaryFile path = hGetContents =<< openBinaryFile path ReadMode
 writeBinaryFile path s = withBinaryFile path WriteMode (flip hPutStr s)
-
--- | Because GHC adds the confusing text "user error" for failures caused by
--- calls to fail.
-ioErrorText e = if isUserError e
-                then ioeGetErrorString e
-                else show e
