@@ -37,7 +37,7 @@ import qualified Data.Map as Map
 import Data.Set (Set)
 import qualified Data.Set as Set
 
-import Debug.Trace
+--import Debug.Trace
 
 data SRG = SRG { srgName :: String    -- ^ grammar name
 		 , srgStartCat :: Cat     -- ^ start category name
@@ -88,13 +88,13 @@ setDefaultCFGTransform opts t b = setCFGTransform t b `addOptions` opts
 
 maybeTransform :: Options -> CFGTransform -> (CFG -> CFG) -> (CFG -> CFG)
 maybeTransform opts t f = if cfgTransform opts t then f else id
-
+{-
 traceStats s g = trace ("---- " ++ s ++ ": " ++ stats g {- ++ "\n" ++ prCFRules g ++ "----" -}) g
 
 stats g = "Categories: " ++ show (countCats g)
           ++ ", External categories: " ++ show (Set.size (cfgExternalCats g))
           ++ ", Rules: " ++ show (countRules g)
-
+-}
 makeNonRecursiveSRG :: Options 
                     -> PGF
                     -> CId -- ^ Concrete syntax name.
@@ -129,9 +129,9 @@ renameCats prefix cfg = mapCFGCats renameCat cfg
         badCat c = error ("GF.Speech.SRG.renameCats: " ++ c ++ "\n" ++ prCFG cfg)
 
 cfRulesToSRGRule :: [CFRule] -> SRGRule
-cfRulesToSRGRule rs@(r:_) = SRGRule (lhsCat r) rhs
+cfRulesToSRGRule rs@(r:_) = SRGRule (ruleLhs r) rhs
     where 
-      alts = [((n,Nothing),mkSRGSymbols 0 ss) | CFRule c ss n <- rs]
+      alts = [((n,Nothing),mkSRGSymbols 0 ss) | Rule c ss n <- rs]
       rhs = [SRGAlt p n (srgItem sss) | ((n,p),sss) <- buildMultiMap alts ]
 
       mkSRGSymbols _ [] = []
