@@ -1,5 +1,3 @@
-{-# LANGUAGE CPP #-}
-
 module GF.Text.Coding where
 
 import qualified Data.ByteString as BS
@@ -25,11 +23,7 @@ encodeUnicode enc s =
   where
     translate cod cbuf
       | i < w     = do bbuf <- newByteBuffer 128 WriteBuffer
-#if __GLASGOW_HASKELL__ >= 702
                        (_,cbuf,bbuf) <- cod cbuf bbuf
-#else
-                       (cbuf,bbuf) <- cod cbuf bbuf
-#endif
                        if isEmptyBuffer bbuf
                          then ioe_invalidCharacter1
                          else do let bs = PS (bufRaw bbuf) (bufL bbuf) (bufR bbuf-bufL bbuf)
@@ -54,11 +48,7 @@ decodeUnicodeIO enc (PS fptr l len) = do
   where
     translate cod bbuf cbuf
       | i < w     = do
-#if __GLASGOW_HASKELL__ >= 702
                        (_,bbuf,cbuf) <- cod bbuf cbuf
-#else
-                       (bbuf,cbuf) <- cod bbuf cbuf
-#endif
                        if isEmptyBuffer cbuf
                          then ioe_invalidCharacter2
                          else unpack cod bbuf cbuf
