@@ -6,12 +6,12 @@ import Data.List(sortBy)
 import Data.Function(on)
 import qualified Data.Map as Map
 import Text.JSON(makeObj) --encode
-import Text.PrettyPrint(render,text,(<+>))
+import GF.Text.Pretty(render,(<+>))
 
 import qualified Data.ByteString.UTF8 as UTF8(fromString)
 
 import GF.Infra.Option(optionsGFO)
-import GF.Infra.Ident(showIdent)
+import GF.Infra.Ident(showIdent,ModuleName(..))
 import GF.Grammar.Grammar
 import GF.Grammar.Printer(ppParams,ppTerm,getAbs,TermPrintQual(..))
 import GF.Grammar.Parser(runP,pModDef)
@@ -56,10 +56,10 @@ convAbstract (modid,src) =
            case lookup "startcat" flags of
              Just (LStr cat) -> cat
              _               -> "-"
-     return $ Grammar (convId modid) extends (Abstract startcat cats funs) []
+     return $ Grammar (convModId modid) extends (Abstract startcat cats funs) []
 
 convExtends = mapM convExtend
-convExtend (modid,MIAll) = return (convId modid)
+convExtend (modid,MIAll) = return (convModId modid)
 convExtend _ = fail "unsupported module extension"
 
 convAbsJments jments = foldM convAbsJment ([],[]) (jmentList jments)
@@ -86,6 +86,7 @@ convSimpleType (Vr id) = return (convId id)
 convSimpleType t = fail "unsupported type"
 
 convId = showIdent
+convModId (MN m) = convId m
 
 convConcrete (modid,src) =
   do unless (isModCnc src) $ fail "Concrete syntax expected"
@@ -100,13 +101,13 @@ convConcrete (modid,src) =
          langcode = "" -- !!!
          conc = Concrete langcode opens ps lcs os ls
          abs = Abstract "-" [] [] -- dummy
-     return $ Grammar (convId modid) extends abs [conc]
+     return $ Grammar (convModId modid) extends abs [conc]
 
 convOpens = mapM convOpen
 
 convOpen o =
   case o of
-    OSimple id -> return (convId id)
+    OSimple id -> return (convModId id)
     _ -> fail "unsupported module open"
 
 
@@ -124,11 +125,11 @@ convCncJment (name,jment) =
     ResOper oltyp (Just lterm) -> return $ Op $ Oper lhs rhs
       where
         lhs = i++maybe "" ((" : "++) . render . ppTerm q 0 . unLoc) oltyp
-        rhs = render (text " ="<+>ppTerm q 0 (unLoc lterm))
+        rhs = render (" ="<+>ppTerm q 0 (unLoc lterm))
     ResOverload [] defs -> return $ Op $ Oper lhs rhs
       where
         lhs = i
-        rhs = render $ text " = overload"<+>ppTerm q 0 r
+        rhs = render $ " = overload"<+>ppTerm q 0 r
         r =  R [(lab,(Just ty,fu)) | (L _ ty,L _ fu) <-defs]
         lab = ident2label name
     CncFun _ (Just ldef) pprn _ -> -- ignores printname !!
