@@ -13,6 +13,7 @@
  * <http://graphics.stanford.edu/~seander/bithacks.html>
  */
 
+GU_INTERNAL_DECL
 unsigned gu_ceil2e(unsigned i);
 
 static inline int
@@ -29,20 +30,20 @@ gu_ceildiv(size_t size, size_t div)
 static inline bool
 gu_aligned(uintptr_t addr, size_t alignment)
 {
-	gu_require(alignment == gu_ceil2e(alignment));
+	//gu_require(alignment == gu_ceil2e(alignment));
 	return (addr & (alignment - 1)) == 0;
 }
 
 static inline uintptr_t
 gu_align_forward(uintptr_t addr, size_t alignment) {
-	gu_require(alignment == gu_ceil2e(alignment));
+	//gu_require(alignment == gu_ceil2e(alignment));
 	uintptr_t mask = alignment - 1;
 	return (addr + mask) & ~mask;
 }
 
 static inline uintptr_t
 gu_align_backward(uintptr_t addr, size_t alignment) {
-	gu_require(alignment == gu_ceil2e(alignment));
+	//gu_require(alignment == gu_ceil2e(alignment));
 	return addr & ~(alignment - 1);
 }
 
@@ -103,9 +104,6 @@ gu_tagged(void* ptr, size_t tag) {
 }
 
 #include <gu/exn.h>
-#include <gu/type.h>
-
-extern GU_DECLARE_TYPE(GuIntDecodeExn, abstract);
 
 #define GU_DECODE_2C_(u_, t_, umax_, posmax_, tmin_, err_)	\
 	(((u_) <= (posmax_))					\
@@ -143,7 +141,7 @@ gu_decode_2c64(uint64_t u, GuExn* err)
 			     UINT64_C(0x7fffffffffffffff), INT64_MIN, err);
 }
 
-double
+GU_INTERNAL_DECL double
 gu_decode_double(uint64_t u);
 
 
