@@ -1,4 +1,4 @@
 #include <gu/defs.h>
 
 void* const gu_null = NULL;
-GuStruct* const gu_null_struct = NULL;
+GU_API GuStruct* const gu_null_struct = NULL;
