@@ -1,22 +1,3 @@
-/* 
- * Copyright 2010 University of Helsinki.
- *   
- * This file is part of libgu.
- * 
- * Libgu is free software: you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by the
- * Free Software Foundation, either version 3 of the License, or (at your
- * option) any later version.
- * 
- * Libgu is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
- * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with libgu. If not, see <http://www.gnu.org/licenses/>.
- */
-
 /** @file
  *
  * Miscellaneous macros.
@@ -25,6 +6,33 @@
 #ifndef GU_DEFS_H_
 #define GU_DEFS_H_
 
+// MSVC requires explicit export/import of
+// symbols in DLLs. CMake takes care of this
+// for functions, but not for data/variables.
+#if defined(_MSC_VER)
+#if defined(COMPILING_GU)
+#define GU_API_DECL __declspec(dllexport)
+#define GU_API __declspec(dllexport)
+#else
+#define GU_API_DECL __declspec(dllimport)
+#define GU_API ERROR_NOT_COMPILING_LIBGU
+#endif
+
+#define GU_INTERNAL_DECL
+#define GU_INTERNAL
+
+#define restrict __restrict
+
+#else
+
+#define GU_API_DECL
+#define GU_API
+
+#define GU_INTERNAL_DECL  __attribute__ ((visibility ("hidden")))
+#define GU_INTERNAL       __attribute__ ((visibility ("hidden")))
+#endif
+// end MSVC workaround
+
 #include <stddef.h>
 #include <inttypes.h>
 #include <stdbool.h>
@@ -154,7 +162,7 @@ extern void* const gu_null;
 // Dummy struct used for generic struct pointers
 typedef struct GuStruct GuStruct;
 
-extern GuStruct* const gu_null_struct;
+GU_API_DECL extern GuStruct* const gu_null_struct;
 
 typedef uintptr_t GuWord;
 
