@@ -1,6 +1,6 @@
 #include <gu/enum.h>
 
-void
+GU_API void
 gu_enum_next(GuEnum* en, void* to, GuPool* pool)
 {
 	en->next(en, to, pool);
