@@ -2,7 +2,6 @@
 #define GU_EXN_H_
 
 #include <gu/mem.h>
-#include <gu/type.h>
 
 /** @file
  *
@@ -47,9 +46,7 @@ struct GuExnData
 struct GuExn {
 	/// @privatesection
 	GuExnState state;
-	GuExn* parent;
-	GuKind* catch;
-	GuType* caught;
+	const char* caught;
 	GuExnData data;
 };
 
@@ -59,22 +56,19 @@ struct GuExn {
 
 
 /// Allocate a new local exception frame.
-#define gu_exn(parent_, catch_, pool_) &(GuExn){	\
+#define gu_exn(pool_) &(GuExn){	\
 	.state = GU_EXN_OK,	\
-	.parent = parent_,	\
-	.catch = gu_kind(catch_),	\
 	.caught = NULL,	\
 	.data = {.pool = pool_, .data = NULL} \
 }
 
 
 /// Allocate a new exception frame.
-GuExn*
-gu_new_exn(GuExn* parent, GuKind* catch_kind, GuPool* pool);
+GU_API_DECL GuExn*
+gu_new_exn(GuPool* pool);
 
 
-
-bool
+GU_API_DECL bool
 gu_exn_is_raised(GuExn* err);
 
 static inline void
@@ -83,10 +77,11 @@ gu_exn_clear(GuExn* err) {
 	err->state = GU_EXN_OK;
 }
 
-
-
-GuType*
-gu_exn_caught(GuExn* err);
+#define gu_exn_caught(err, type) \
+	(err->caught && strcmp(err->caught, #type) == 0)
+	
+GU_API_DECL bool
+gu_exn_caught_(GuExn* err, const char* type);
 
 static inline const void*
 gu_exn_caught_data(GuExn* err)
@@ -95,20 +90,20 @@ gu_exn_caught_data(GuExn* err)
 }
 
 /// Temporarily block a raised exception.
-void
+GU_API_DECL void
 gu_exn_block(GuExn* err);
 
 /// Show again a blocked exception.
-void
+GU_API_DECL void
 gu_exn_unblock(GuExn* err);
 
 //@private
-GuExnData*
-gu_exn_raise_(GuExn* err, GuType* type);
+GU_API_DECL GuExnData*
+gu_exn_raise_(GuExn* err, const char* type);
 
 //@private
-GuExnData*
-gu_exn_raise_debug_(GuExn* err, GuType* type,
+GU_API_DECL GuExnData*
+gu_exn_raise_debug_(GuExn* err, const char* type,
 		      const char* filename, const char* func, int lineno);
 
 #ifdef NDEBUG
@@ -122,7 +117,7 @@ gu_exn_raise_debug_(GuExn* err, GuType* type,
 
 /// Raise an exception.
 #define gu_raise(exn, T)		\
-	gu_exn_raise(exn, gu_type(T))
+	gu_exn_raise(exn, #T)
 /**< 
  * @param exn The current exception frame.
  *
@@ -168,9 +163,7 @@ gu_ok(GuExn* exn) {
 
 typedef int GuErrno;
 
-extern GU_DECLARE_TYPE(GuErrno, signed);
-
-void
+GU_API_DECL void
 gu_raise_errno(GuExn* err);
 
 /** @} */
