@@ -1,18 +1,10 @@
 #include <gu/defs.h>
 #include <gu/mem.h>
-#include <gu/type.h>
 #include <gu/map.h>
 #include <gu/assert.h>
 #include <gu/prime.h>
 #include <gu/string.h>
 
-typedef enum {
-	GU_MAP_GENERIC,
-	GU_MAP_ADDR,
-	GU_MAP_WORD,
-	GU_MAP_STRING
-} GuMapKind;
-
 typedef struct GuMapData GuMapData;
 
 struct GuMapData {
@@ -24,7 +16,6 @@ struct GuMapData {
 };
 
 struct GuMap {
-	GuMapKind kind;
 	GuHasher* hasher;
 	size_t key_size;
 	size_t value_size;
@@ -62,17 +53,16 @@ gu_map_entry_is_free(GuMap* map, GuMapData* data, size_t idx)
 {
 	if (idx == data->zero_idx) {
 		return false;
-	} else if (map->kind == GU_MAP_ADDR) {
+	} else if (map->hasher == gu_addr_hasher) {
 		const void* key = ((const void**)data->keys)[idx];
 		return key == NULL;
-	} else if (map->kind == GU_MAP_WORD) {
+	} else if (map->hasher == gu_word_hasher) {
 		GuWord key = ((GuWord*)data->keys)[idx];
 		return key == 0;
-	} else if (map->kind == GU_MAP_STRING) {
+	} else if (map->hasher == gu_string_hasher) {
 		GuString key = ((GuString*)data->keys)[idx];
 		return key == NULL;
 	}
-	gu_assert(map->kind == GU_MAP_GENERIC);
 	const void* key = &data->keys[idx * map->key_size];
 	return gu_map_buf_is_zero(key, map->key_size);
 }
@@ -81,8 +71,57 @@ static bool
 gu_map_lookup(GuMap* map, const void* key, size_t* idx_out)
 {
 	size_t n = map->data.n_entries;
-	switch (map->kind) {
-	case GU_MAP_GENERIC: {
+	if (map->hasher == gu_addr_hasher) {
+		GuHash hash = (GuHash) key;
+		size_t idx = hash % n;
+		size_t offset = (hash % (n - 2)) + 1;
+		while (true) {
+			const void* entry_key =
+				((const void**)map->data.keys)[idx];
+			if (entry_key == NULL && map->data.zero_idx != idx) {
+				*idx_out = idx;
+				return false;
+			} else if (entry_key == key) {
+				*idx_out = idx;
+				return true;
+			}
+			idx = (idx + offset) % n;
+		}
+	} else if (map->hasher == gu_word_hasher) {
+		GuWord w = *(const GuWord*)key;
+		GuHash hash = (GuHash) w;
+		size_t idx = hash % n;
+		size_t offset = (hash % (n - 2)) + 1;
+		while (true) {
+			GuWord entry_key = ((GuWord*)map->data.keys)[idx];
+			if (entry_key == 0 && map->data.zero_idx != idx) {
+				*idx_out = idx;
+				return false;
+			} else if (entry_key == w) {
+				*idx_out = idx;
+				return true;
+			}
+			idx = (idx + offset) % n;
+		}
+	} else if (map->hasher == gu_string_hasher) {
+		GuHasher* hasher = map->hasher;
+		GuEquality* eq = (GuEquality*) hasher;
+		GuHash hash = hasher->hash(hasher, key);
+		size_t idx = hash % n;
+		size_t offset = (hash % (n - 2)) + 1;
+		while (true) {
+			GuString entry_key =
+				((GuString*)map->data.keys)[idx];
+			if (entry_key == NULL && map->data.zero_idx != idx) {
+				*idx_out = idx;
+				return false;
+			} else if (eq->is_equal(eq, key, entry_key)) {
+				*idx_out = idx;
+				return true;
+			}
+			idx = (idx + offset) % n;
+		}
+	} else {
 		GuHasher* hasher = map->hasher;
 		GuEquality* eq = (GuEquality*) hasher;
 		GuHash hash = hasher->hash(hasher, key);
@@ -101,83 +140,18 @@ gu_map_lookup(GuMap* map, const void* key, size_t* idx_out)
 			}
 			idx = (idx + offset) % n;
 		}
-		gu_impossible();
-		break;
-	}
-	case GU_MAP_ADDR: {
-		GuHash hash = (GuHash) key;
-		size_t idx = hash % n;
-		size_t offset = (hash % (n - 2)) + 1;
-		while (true) {
-			const void* entry_key =
-				((const void**)map->data.keys)[idx];
-			if (entry_key == NULL && map->data.zero_idx != idx) {
-				*idx_out = idx;
-				return false;
-			} else if (entry_key == key) {
-				*idx_out = idx;
-				return true;
-			}
-			idx = (idx + offset) % n;
-		}
-		gu_impossible();
-		break;
-	}
-	case GU_MAP_WORD: {
-		GuWord w = *(const GuWord*)key;
-		GuHash hash = (GuHash) w;
-		size_t idx = hash % n;
-		size_t offset = (hash % (n - 2)) + 1;
-		while (true) {
-			GuWord entry_key = ((GuWord*)map->data.keys)[idx];
-			if (entry_key == 0 && map->data.zero_idx != idx) {
-				*idx_out = idx;
-				return false;
-			} else if (entry_key == w) {
-				*idx_out = idx;
-				return true;
-			}
-			idx = (idx + offset) % n;
-		}
-		gu_impossible();
-		break;
-	}
-	case GU_MAP_STRING: {
-		GuHasher* hasher = map->hasher;
-		GuEquality* eq = (GuEquality*) hasher;
-		GuHash hash = hasher->hash(hasher, key);
-		size_t idx = hash % n;
-		size_t offset = (hash % (n - 2)) + 1;
-		while (true) {
-			GuString entry_key =
-				((GuString*)map->data.keys)[idx];
-			if (entry_key == NULL && map->data.zero_idx != idx) {
-				*idx_out = idx;
-				return false;
-			} else if (eq->is_equal(eq, key, entry_key)) {
-				*idx_out = idx;
-				return true;
-			}
-			idx = (idx + offset) % n;
-		}
-		gu_impossible();
-		break;
-	}
-	default:
-		gu_impossible();
 	}
+
 	gu_impossible();
 	return false;
 }
 	
 
 static void
-gu_map_resize(GuMap* map)
+gu_map_resize(GuMap* map, size_t req_entries)
 {
 	GuMapData* data = &map->data;
 	GuMapData old_data = *data;
-	size_t req_entries =
-		gu_twin_prime_sup(GU_MAX(11, map->data.n_occupied * 4 / 3 + 1));
 
 	size_t key_size = map->key_size;
 	size_t key_alloc = 0;
@@ -195,23 +169,16 @@ gu_map_resize(GuMap* map)
 					    GU_MIN(key_alloc / key_size,
 						   value_alloc / value_size)
 					    : key_alloc / key_size);
-	switch (map->kind) {
-	case GU_MAP_GENERIC:
-	case GU_MAP_WORD:
-		memset(data->keys, 0, key_alloc);
-		break;
-	case GU_MAP_ADDR:
+	if (map->hasher == gu_addr_hasher) {
 		for (size_t i = 0; i < data->n_entries; i++) {
 			((const void**)data->keys)[i] = NULL;
 		}
-		break;
-	case GU_MAP_STRING:
+	} else if (map->hasher == gu_string_hasher) {
 		for (size_t i = 0; i < data->n_entries; i++) {
 			((GuString*)data->keys)[i] = NULL;
 		}
-		break;
-	default:
-		gu_impossible();
+	} else {
+		memset(data->keys, 0, key_alloc);
 	}
 
 	gu_assert(data->n_entries > data->n_occupied);
@@ -224,9 +191,9 @@ gu_map_resize(GuMap* map)
 			continue;
 		}
 		void* old_key = &old_data.keys[i * key_size];
-		if (map->kind == GU_MAP_ADDR) {
+		if (map->hasher == gu_addr_hasher) {
 			old_key = *(void**)old_key;
-		} else if (map->kind == GU_MAP_STRING) {
+		} else if (map->hasher == gu_string_hasher) {
 			old_key = (void*) *(GuString*)old_key;
 		}
 		void* old_value = &old_data.values[i * value_size];
@@ -247,13 +214,15 @@ gu_map_maybe_resize(GuMap* map)
 {
 	if (map->data.n_entries <=
 	    map->data.n_occupied + (map->data.n_occupied / 4)) {
-		gu_map_resize(map);
+		size_t req_entries =
+			gu_twin_prime_sup(GU_MAX(11, map->data.n_occupied * 4 / 3 + 1));
+		gu_map_resize(map, req_entries);
 		return true;
 	}
 	return false;
 }
 
-void*
+GU_API void*
 gu_map_find(GuMap* map, const void* key)
 {
 	size_t idx;
@@ -264,14 +233,14 @@ gu_map_find(GuMap* map, const void* key)
 	return NULL;
 }
 
-const void*
+GU_API const void*
 gu_map_find_default(GuMap* map, const void* key)
 {
 	void* p = gu_map_find(map, key);
 	return p ? p : map->default_value;
 }
 
-const void*
+GU_API const void*
 gu_map_find_key(GuMap* map, const void* key)
 {
 	size_t idx;
@@ -282,14 +251,14 @@ gu_map_find_key(GuMap* map, const void* key)
 	return NULL;
 }
 
-bool
+GU_API bool
 gu_map_has(GuMap* ht, const void* key)
 {
 	size_t idx;
 	return gu_map_lookup(ht, key, &idx);
 }
 
-void*
+GU_API void*
 gu_map_insert(GuMap* map, const void* key)
 {
 	size_t idx;
@@ -299,9 +268,9 @@ gu_map_insert(GuMap* map, const void* key)
 			found = gu_map_lookup(map, key, &idx);
 			gu_assert(!found);
 		}
-		if (map->kind == GU_MAP_ADDR) {
+		if (map->hasher == gu_addr_hasher) {
 			((const void**)map->data.keys)[idx] = key;
-		} else if (map->kind == GU_MAP_STRING) {
+		} else if (map->hasher == gu_string_hasher) {
 			((GuString*)map->data.keys)[idx] = key;
 		} else {
 			memcpy(&map->data.keys[idx * map->key_size],
@@ -320,7 +289,7 @@ gu_map_insert(GuMap* map, const void* key)
 	return &map->data.values[idx * map->value_size];
 }
 
-void
+GU_API void
 gu_map_iter(GuMap* map, GuMapItor* itor, GuExn* err)
 {
 	for (size_t i = 0; i < map->data.n_entries && gu_ok(err); i++) {
@@ -329,9 +298,9 @@ gu_map_iter(GuMap* map, GuMapItor* itor, GuExn* err)
 		}
 		const void* key = &map->data.keys[i * map->key_size];
 		void* value = &map->data.values[i * map->value_size];
-		if (map->kind == GU_MAP_ADDR) {
+		if (map->hasher == gu_addr_hasher) {
 			key = *(const void* const*) key;
-		} else if (map->kind == GU_MAP_STRING) {
+		} else if (map->hasher == gu_string_hasher) {
 			key = *(GuString*) key;
 		}
 		itor->fn(itor, key, value, err);
@@ -358,9 +327,9 @@ gu_map_enum_next(GuEnum* self, void* to, GuPool* pool)
 		}
 		en->x.key   = &en->ht->data.keys[i * en->ht->key_size];
 		en->x.value = &en->ht->data.values[i * en->ht->value_size];
-		if (en->ht->kind == GU_MAP_ADDR) {
+		if (en->ht->hasher == gu_addr_hasher) {
 			en->x.key = *(const void* const*) en->x.key;
-		} else if (en->ht->kind == GU_MAP_STRING) {
+		} else if (en->ht->hasher == gu_string_hasher) {
 			en->x.key = *(GuString*) en->x.key;
 		}
 
@@ -371,7 +340,7 @@ gu_map_enum_next(GuEnum* self, void* to, GuPool* pool)
 	en->i = i+1;
 }
 
-GuEnum*
+GU_API GuEnum*
 gu_map_enum(GuMap* ht, GuPool* pool)
 {
 	GuMapEnum* en = gu_new(GuMapEnum, pool);
@@ -381,7 +350,7 @@ gu_map_enum(GuMap* ht, GuPool* pool)
 	return &en->en;
 }
 
-size_t
+GU_API size_t
 gu_map_count(GuMap* map)
 {
 	size_t count = 0;
@@ -396,22 +365,12 @@ gu_map_count(GuMap* map)
 
 static const uint8_t gu_map_no_values[1] = { 0 };
 
-GuMap*
+GU_API GuMap*
 gu_make_map(size_t key_size, GuHasher* hasher,
 	    size_t value_size, const void* default_value,
+	    size_t init_size,
 	    GuPool* pool)
 {
-	GuMapKind kind =
-		((!hasher || hasher == gu_addr_hasher)
-		 ? GU_MAP_ADDR
-		 : (hasher == gu_string_hasher)
-		 ? GU_MAP_STRING
-		 : (key_size == sizeof(GuWord) && hasher == gu_word_hasher)
-		 ? GU_MAP_WORD
-		 : GU_MAP_GENERIC);
-	if (kind == GU_MAP_ADDR || kind == GU_MAP_STRING) {
-		key_size = sizeof(GuWord);
-	}
 	GuMapData data = {
 		.n_occupied = 0,
 		.n_entries = 0,
@@ -426,25 +385,9 @@ gu_make_map(size_t key_size, GuHasher* hasher,
 	map->key_size = key_size;
 	map->value_size = value_size;
 	map->fin.fn = gu_map_finalize;
-	map->kind = kind;
 	gu_pool_finally(pool, &map->fin);
-	gu_map_resize(map);
+
+	init_size = gu_twin_prime_sup(init_size);
+	gu_map_resize(map, init_size);
 	return map;
 }
-
-GuMap*
-gu_map_type_make(GuMapType* mtype, GuPool* pool)
-{
-	size_t key_size = 0;
-	if (mtype->hasher && mtype->hasher != gu_addr_hasher) {
-		key_size = gu_type_size(mtype->key_type);
-	}
-	size_t value_size = gu_type_size(mtype->value_type);
-	return gu_make_map(key_size, mtype->hasher,
-			   value_size, mtype->default_value, pool);
-}
-
-GU_DEFINE_KIND(GuMap, abstract);
-// GU_DEFINE_KIND(GuIntMap, GuMap);
-
-
