@@ -15,33 +15,36 @@ struct GuMapItor {
 
 typedef struct GuMap GuMap;
 
-GuMap*
+GU_API_DECL GuMap*
 gu_make_map(size_t key_size, GuHasher* hasher,
 	    size_t value_size, const void* default_value,
+	    size_t init_size,
 	    GuPool* pool);
 
+#define GU_MAP_DEFAULT_INIT_SIZE 11
+
 #define gu_new_map(K, HASHER, V, DV, POOL)			\
-	(gu_make_map(sizeof(K), (HASHER), sizeof(V), (DV), (POOL)))
+	(gu_make_map(sizeof(K), (HASHER), sizeof(V), (DV), GU_MAP_DEFAULT_INIT_SIZE, (POOL)))
 
 #define gu_new_set(K, HASHER, POOL)			\
-	(gu_make_map(sizeof(K), (HASHER), 0, NULL, (POOL)))
+	(gu_make_map(sizeof(K), (HASHER), 0, NULL, GU_MAP_DEFAULT_INIT_SIZE, (POOL)))
 
 #define gu_new_addr_map(K, V, DV, POOL)				\
-	(gu_make_map(0, NULL, sizeof(V), (DV), (POOL)))
+	(gu_make_map(sizeof(K), gu_addr_hasher, sizeof(V), (DV), GU_MAP_DEFAULT_INIT_SIZE, (POOL)))
 
-size_t
+GU_API_DECL size_t
 gu_map_count(GuMap* map);
 
-void*
+GU_API_DECL void*
 gu_map_find_full(GuMap* ht, void* key_inout);
 
-const void*
+GU_API_DECL const void*
 gu_map_find_default(GuMap* ht, const void* key);
 
 #define gu_map_get(MAP, KEYP, V)		\
 	(*(V*)gu_map_find_default((MAP), (KEYP)))
 
-void*
+GU_API_DECL void*
 gu_map_find(GuMap* ht, const void* key);
 
 #define gu_map_set(MAP, KEYP, V, VAL)				\
@@ -50,13 +53,13 @@ gu_map_find(GuMap* ht, const void* key);
 	*gu_map_set_p_ = (VAL);					\
 	GU_END
 
-const void*
+GU_API_DECL const void*
 gu_map_find_key(GuMap* ht, const void* key);
 
-bool
+GU_API_DECL bool
 gu_map_has(GuMap* ht, const void* key);
 
-void*
+GU_API_DECL void*
 gu_map_insert(GuMap* ht, const void* key);
 
 #define gu_map_put(MAP, KEYP, V, VAL)				\
@@ -65,7 +68,7 @@ gu_map_insert(GuMap* ht, const void* key);
 	*gu_map_put_p_ = (VAL);					\
 	GU_END
 
-void
+GU_API_DECL void
 gu_map_iter(GuMap* ht, GuMapItor* itor, GuExn* err);
 
 typedef struct {
@@ -73,7 +76,7 @@ typedef struct {
 	void* value;
 } GuMapKeyValue;
 
-GuEnum*
+GU_API_DECL GuEnum*
 gu_map_enum(GuMap* ht, GuPool* pool);
 
 typedef GuMap GuIntMap;
@@ -81,45 +84,4 @@ typedef GuMap GuIntMap;
 #define gu_new_int_map(VAL_T, DEFAULT, POOL)		\
 	gu_new_map(int, gu_int_hasher, VAL_T, DEFAULT, POOL)
 
-
-#if defined(GU_TYPE_H_) && !defined(GU_MAP_H_TYPE_)
-#define GU_MAP_H_TYPE_
-
-extern GU_DECLARE_KIND(GuMap);
-
-typedef const struct GuMapType GuMapType, GuType_GuMap;
-
-struct GuMapType {
-	GuType_abstract abstract_base;
-	GuHasher* hasher;
-	GuType* key_type;
-	GuType* value_type;
-	const void* default_value;
-};
-
-GuMap*
-gu_map_type_make(GuMapType* mtype, GuPool* pool);
-
-#define gu_map_type_new(MAP_T, POOL)			\
-	gu_map_type_make(gu_type_cast(gu_type(MAP_T), GuMap), (POOL))
-
-#define GU_TYPE_INIT_GuMap(k_, t_, kt_, h_, vt_, dv_)			\
-	{								\
-		.abstract_base = GU_TYPE_INIT_abstract(k_, t_, _),	\
-		.hasher = h_,						\
-		.key_type = kt_,					\
-		.value_type = vt_,					\
-		.default_value = dv_					\
-	}
-
-#define gu_type__GuIntMap gu_type__GuMap
-
-typedef GuType_GuMap GuType_GuIntMap;
-
-#define GU_TYPE_INIT_GuIntMap(KIND, MAP_T, VAL_T, DEFAULT)	\
-	GU_TYPE_INIT_GuMap(KIND, MAP_T, gu_type(int), gu_int_hasher,	\
-			   VAL_T, DEFAULT)
-
-#endif
-
 #endif // GU_MAP_H_
