@@ -1,28 +1,17 @@
-/* 
- * Copyright 2010 University of Helsinki.
- *   
- * This file is part of libgu.
- * 
- * Libgu is free software: you can redistribute it and/or modify it under
- * the terms of the GNU Lesser General Public License as published by the
- * Free Software Foundation, either version 3 of the License, or (at your
- * option) any later version.
- * 
- * Libgu is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
- * License for more details.
- * 
- * You should have received a copy of the GNU Lesser General Public
- * License along with libgu. If not, see <http://www.gnu.org/licenses/>.
- */
-
 #include <gu/mem.h>
 #include <gu/fun.h>
 #include <gu/bits.h>
 #include <gu/assert.h>
 #include <string.h>
 #include <stdlib.h>
+#if !defined(_WIN32) && !defined(_WIN64)
+#include <sys/mman.h>
+#include <sys/stat.h>
+#endif
+#if !defined(_MSC_VER)
+#include <unistd.h>
+#endif
+#include <fcntl.h>
 
 #ifdef USE_VALGRIND
 #include <valgrind/valgrind.h>
@@ -34,7 +23,7 @@
 static const size_t
 // Maximum request size for a chunk. The actual maximum chunk size
 // may be somewhat larger.
-gu_mem_chunk_max_size = 1024 * sizeof(void*),
+    gu_mem_chunk_max_size = 1024 * sizeof(void*),
 
 // number of bytes to allocate in the pool when it is created
 	gu_mem_pool_initial_size = 24 * sizeof(void*),
@@ -101,7 +90,7 @@ gu_mem_padovan(size_t min)
 	return a;
 }
 
-void*
+GU_API void*
 gu_mem_buf_realloc(void* old_buf, size_t min_size, size_t* real_size_out)
 {
 	size_t min_blocks = ((min_size + gu_malloc_overhead - 1) /
@@ -112,13 +101,14 @@ gu_mem_buf_realloc(void* old_buf, size_t min_size, size_t* real_size_out)
 	*real_size_out = buf ? size : 0;
 	return buf;
 }
-void*
+
+GU_API void*
 gu_mem_buf_alloc(size_t min_size, size_t* real_size_out)
 {
 	return gu_mem_buf_realloc(NULL, min_size, real_size_out);
 }
 
-void
+GU_API void
 gu_mem_buf_free(void* buf)
 {
 	gu_mem_free(buf);
@@ -139,18 +129,20 @@ struct GuFinalizerNode {
 	GuFinalizer* fin;
 };
 
-enum GuPoolFlags {
-	GU_POOL_LOCAL = 1 << 0
+enum GuPoolType {
+	GU_POOL_HEAP,
+	GU_POOL_LOCAL,
+	GU_POOL_MMAP
 };
 
 struct GuPool {
 	uint8_t* curr_buf; // actually GuMemChunk*
 	GuMemChunk* chunks;
 	GuFinalizerNode* finalizers;
-	uint16_t flags;
-	uint16_t left_edge;
-	uint16_t right_edge;
-	uint16_t curr_size;
+	uint16_t type;
+	size_t left_edge;
+	size_t right_edge;
+	size_t curr_size;
 	uint8_t init_buf[];
 };
 
@@ -160,7 +152,7 @@ gu_init_pool(uint8_t* buf, size_t sz)
 	gu_require(gu_aligned((uintptr_t) (void*) buf, gu_alignof(GuPool)));
 	gu_require(sz >= sizeof(GuPool));
 	GuPool* pool = (GuPool*) buf;
-	pool->flags = 0;
+	pool->type = GU_POOL_HEAP;
 	pool->curr_size = sz;
 	pool->curr_buf = (uint8_t*) pool;
 	pool->chunks = NULL;
@@ -171,15 +163,15 @@ gu_init_pool(uint8_t* buf, size_t sz)
 	return pool;
 }
 
-GuPool*
+GU_API GuPool*
 gu_local_pool_(uint8_t* buf, size_t sz)
 {
 	GuPool* pool = gu_init_pool(buf, sz);
-	pool->flags |= GU_POOL_LOCAL;
+	pool->type = GU_POOL_LOCAL;
 	return pool;
 }
 
-GuPool* 
+GU_API GuPool* 
 gu_new_pool(void)
 {
 	size_t sz = GU_FLEX_SIZE(GuPool, init_buf, gu_mem_pool_initial_size);
@@ -188,9 +180,60 @@ gu_new_pool(void)
 	return pool;
 }
 
+GU_API GuPool* 
+gu_mmap_pool(char* fpath, void* addr, size_t size, void**pptr)
+{
+#if !defined(_WIN32) && !defined(_WIN64)
+	int prot = PROT_READ;
+	int fd = open(fpath, O_RDONLY);
+	if (fd < 0) {
+		if (errno == ENOENT) {
+			fd = open(fpath, O_RDWR | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR);
+			if (fd < 0)
+				return NULL;
+
+			if (ftruncate(fd, size) < 0) {
+				close(fd);
+				return NULL;
+			}
+
+			prot |= PROT_WRITE;
+		} else {
+			return NULL;
+		}
+	}
+
+	void *ptr = mmap(addr, size, prot, MAP_SHARED | MAP_FIXED, fd, 0);
+	if (ptr == MAP_FAILED) {
+		close(fd);
+		return NULL;
+	}
+
+	gu_require(ptr == addr);
+
+	*pptr = (prot & PROT_WRITE) ? NULL : ptr;
+
+	size_t sz = GU_FLEX_SIZE(GuPool, init_buf, sizeof(int));
+	uint8_t* buf = gu_mem_buf_alloc(sz, &sz);
+	GuPool* pool = gu_init_pool(buf, size);
+
+	uint8_t* pfd = pool->init_buf;
+	*((int*) pfd) = fd;
+
+	pool->type = GU_POOL_MMAP;
+	pool->curr_buf = ptr;
+	pool->left_edge = 0;
+
+	return pool;
+#else
+	return NULL;
+#endif
+}
+
 static void
 gu_pool_expand(GuPool* pool, size_t req)
 {
+	gu_require(pool->type != GU_POOL_MMAP);
 	size_t real_req = GU_MAX(req, GU_MIN(((size_t)pool->curr_size) + 1,
 					     gu_mem_chunk_max_size));
 	gu_assert(real_req >= sizeof(GuMemChunk));
@@ -201,7 +244,6 @@ gu_pool_expand(GuPool* pool, size_t req)
 	pool->curr_buf = (uint8_t*) chunk;
 	pool->left_edge = offsetof(GuMemChunk, data);
 	pool->right_edge = pool->curr_size = size;
-	// size should always fit in uint16_t
 	gu_assert((size_t) pool->right_edge == size); 
 }
 
@@ -220,7 +262,6 @@ static void*
 gu_pool_malloc_aligned(GuPool* pool, size_t pre_align, size_t pre_size,
 		       size_t align, size_t size) 
 {
-	gu_require(size <= gu_mem_max_shared_alloc);
 	size_t pos = gu_mem_advance(pool->left_edge, pre_align, pre_size,
 				    align, size);
 	if (pos > (size_t) pool->right_edge) {
@@ -241,7 +282,7 @@ gu_pool_avail(GuPool* pool)
 	return (size_t) pool->right_edge - (size_t) pool->left_edge;
 }
 
-void*
+GU_API void*
 gu_pool_malloc_unaligned(GuPool* pool, size_t size)
 {
 	if (size > gu_pool_avail(pool)) {
@@ -254,7 +295,7 @@ gu_pool_malloc_unaligned(GuPool* pool, size_t size)
 	return addr;
 }
 
-void*
+GU_API void*
 gu_malloc_prefixed(GuPool* pool, size_t pre_align, size_t pre_size,
 		   size_t align, size_t size)
 {
@@ -267,7 +308,8 @@ gu_malloc_prefixed(GuPool* pool, size_t pre_align, size_t pre_size,
 	}
 	size_t full_size = gu_mem_advance(offsetof(GuMemChunk, data),
 					  pre_align, pre_size, align, size);
-	if (full_size > gu_mem_max_shared_alloc) {
+	if (full_size > gu_mem_max_shared_alloc &&
+	    pool->type != GU_POOL_MMAP) {
 		GuMemChunk* chunk = gu_mem_alloc(full_size);
 		chunk->next = pool->chunks;
 		pool->chunks = chunk;
@@ -286,34 +328,29 @@ gu_malloc_prefixed(GuPool* pool, size_t pre_align, size_t pre_size,
 	return ret;
 }
 
-void*
+GU_API void*
 gu_malloc_aligned(GuPool* pool, size_t size, size_t align)
 {
-	if (align == 0) {
-		align = gu_alignof(GuMaxAlign);
-	}
-	void* ret = gu_malloc_prefixed(pool, 1, 0, align, size);
-	return ret;
+	return gu_malloc_prefixed(pool, 1, 0, align, size);
 }
 
-
-void 
+GU_API void 
 gu_pool_finally(GuPool* pool, GuFinalizer* finalizer)
 {
+	gu_require(pool->type != GU_POOL_MMAP);
 	GuFinalizerNode* node = gu_new(GuFinalizerNode, pool);
 	node->next = pool->finalizers;
 	node->fin = finalizer;
 	pool->finalizers = node;
 }
 
-void
+GU_API void
 gu_pool_free(GuPool* pool)
 {
 	GuFinalizerNode* node = pool->finalizers;
 	while (node) {
-		GuFinalizerNode* next = node->next;
 		node->fin->fn(node->fin);
-		node = next;
+		node = node->next;
 	}
 	GuMemChunk* chunk = pool->chunks;
 	while (chunk) {
@@ -322,8 +359,16 @@ gu_pool_free(GuPool* pool)
 		chunk = next;
 	}
 	VG(VALGRIND_DESTROY_MEMPOOL(pool));
-	if (!pool->flags & GU_POOL_LOCAL) {
+	if (pool->type == GU_POOL_HEAP) {
 		gu_mem_buf_free(pool);
+	} else if (pool->type == GU_POOL_MMAP) {
+#if !defined(_WIN32) && !defined(_WIN64)
+		uint8_t* pfd = pool->init_buf;
+		int fd = *(pfd);
+
+		munmap(pool->curr_buf, pool->curr_size);
+		close(fd);
+#endif
 	}
 }
 
