@@ -1,37 +1,12 @@
 #include <gu/out.h>
 #include <gu/seq.h>
 #include <gu/fun.h>
-#include <gu/str.h>
 #include <gu/assert.h>
 #include <stdlib.h>
-#ifdef __MINGW32__
+#if defined(__MINGW32__) || defined(_MSC_VER)
 #include <malloc.h>
 #endif
 
-struct GuSeq {
-	size_t len;
-	uint8_t data[0];
-};
-
-struct GuBuf {
-	GuSeq* seq;
-	size_t elem_size;
-	size_t avail_len;
-	GuFinalizer fin;
-};
-
-size_t
-gu_buf_length(GuBuf* buf)
-{
-	return buf->seq->len;
-}
-
-size_t
-gu_buf_avail(GuBuf* buf)
-{
-	return buf->avail_len;
-}
-
 static void
 gu_buf_fini(GuFinalizer* fin)
 {
@@ -40,7 +15,7 @@ gu_buf_fini(GuFinalizer* fin)
 		gu_mem_buf_free(buf->seq);
 }
 
-GuBuf*
+GU_API GuBuf*
 gu_make_buf(size_t elem_size, GuPool* pool)
 {
 	GuBuf* buf = gu_new(GuBuf, pool);
@@ -52,26 +27,35 @@ gu_make_buf(size_t elem_size, GuPool* pool)
 	return buf;
 }
 
-size_t
-gu_seq_length(GuSeq* seq)
-{
-	return seq->len;
-}
+extern size_t
+gu_buf_length(GuBuf* buf);
 
-void*
-gu_seq_data(GuSeq* seq)
-{
-	return seq->data;
-}
+extern size_t
+gu_buf_avail(GuBuf* buf);
+
+extern void*
+gu_buf_data(GuBuf* buf);
+
+extern GuSeq*
+gu_buf_data_seq(GuBuf* buf);
+
+extern void*
+gu_buf_extend(GuBuf* buf);
+
+extern const void*
+gu_buf_trim(GuBuf* buf);
+
+extern void
+gu_buf_flush(GuBuf* buf);
 
 static GuSeq gu_empty_seq_ = {0};
 
-GuSeq*
+GU_API GuSeq*
 gu_empty_seq() {
 	return &gu_empty_seq_;
 }
 
-GuSeq*
+GU_API GuSeq*
 gu_make_seq(size_t elem_size, size_t length, GuPool* pool)
 {
 	GuSeq* seq = gu_malloc(pool, sizeof(GuSeq) + elem_size * length);
@@ -79,7 +63,13 @@ gu_make_seq(size_t elem_size, size_t length, GuPool* pool)
 	return seq;
 }
 
-GuSeq*
+extern size_t
+gu_seq_length(GuSeq* seq);
+
+extern void*
+gu_seq_data(GuSeq* seq);
+
+GU_API GuSeq*
 gu_alloc_seq_(size_t elem_size, size_t length)
 {
 	if (length == 0)
@@ -91,7 +81,7 @@ gu_alloc_seq_(size_t elem_size, size_t length)
 	return seq;
 }
 
-GuSeq*
+GU_API GuSeq*
 gu_realloc_seq_(GuSeq* seq, size_t elem_size, size_t length)
 {
 	size_t real_size;
@@ -102,7 +92,7 @@ gu_realloc_seq_(GuSeq* seq, size_t elem_size, size_t length)
 	return new_seq;
 }
 
-void
+GU_API void
 gu_seq_free(GuSeq* seq)
 {
 	if (seq == NULL || seq == gu_empty_seq())
@@ -110,7 +100,7 @@ gu_seq_free(GuSeq* seq)
 	gu_mem_buf_free(seq);
 }
 
-static void
+GU_API void
 gu_buf_require(GuBuf* buf, size_t req_len)
 {
 	if (req_len <= buf->avail_len) {
@@ -130,19 +120,7 @@ gu_buf_require(GuBuf* buf, size_t req_len)
 	buf->avail_len = (real_size - sizeof(GuSeq)) / buf->elem_size;
 }
 
-void*
-gu_buf_data(GuBuf* buf)
-{
-	return &buf->seq->data;
-}
-
-GuSeq*
-gu_buf_data_seq(GuBuf* buf)
-{
-	return buf->seq;
-}
-
-void*
+GU_API void*
 gu_buf_extend_n(GuBuf* buf, size_t n_elems)
 {
 	size_t len = gu_buf_length(buf);
@@ -152,20 +130,14 @@ gu_buf_extend_n(GuBuf* buf, size_t n_elems)
 	return &buf->seq->data[buf->elem_size * len];
 }
 
-void*
-gu_buf_extend(GuBuf* buf)
-{
-	return gu_buf_extend_n(buf, 1);
-}
-
-void
+GU_API void
 gu_buf_push_n(GuBuf* buf, const void* data, size_t n_elems)
 {
 	void* p = gu_buf_extend_n(buf, n_elems);
 	memcpy(p, data, buf->elem_size * n_elems);
 }
 
-const void*
+GU_API const void*
 gu_buf_trim_n(GuBuf* buf, size_t n_elems)
 {
 	gu_require(n_elems <= gu_buf_length(buf));
@@ -174,26 +146,14 @@ gu_buf_trim_n(GuBuf* buf, size_t n_elems)
 	return &buf->seq->data[buf->elem_size * new_len];
 }
 
-const void*
-gu_buf_trim(GuBuf* buf)
-{
-	return gu_buf_trim_n(buf, 1);
-}
-
-void
-gu_buf_flush(GuBuf* buf)
-{
-	buf->seq->len = 0;
-}
-
-void
+GU_API void
 gu_buf_pop_n(GuBuf* buf, size_t n_elems, void* data_out)
 {
 	const void* p = gu_buf_trim_n(buf, n_elems);
 	memcpy(data_out, p, buf->elem_size * n_elems);
 }
 
-GuSeq*
+GU_API GuSeq*
 gu_buf_freeze(GuBuf* buf, GuPool* pool)
 {
 	size_t len = gu_buf_length(buf);
@@ -204,7 +164,7 @@ gu_buf_freeze(GuBuf* buf, GuPool* pool)
 	return seq;
 }
 
-void*
+GU_API void*
 gu_buf_insert(GuBuf* buf, size_t index)
 {
 	size_t len = buf->seq->len;
@@ -265,14 +225,14 @@ gu_quick_sort(GuBuf *buf, GuOrder *order, int left, int right)
 		gu_quick_sort(buf, order, index+1, right);
 }
 
-void
+GU_API void
 gu_buf_sort(GuBuf *buf, GuOrder *order)
 {
 	gu_quick_sort(buf, order, 0, gu_buf_length(buf) - 1);
 }
 
-void*
-gu_seq_binsearch_(GuSeq *seq, GuOrder *order, size_t elem_size, void *key)
+GU_API void*
+gu_seq_binsearch_(GuSeq *seq, GuOrder *order, size_t elem_size, const void *key)
 {
 	int i = 0;
 	int j = seq->len-1;
@@ -294,9 +254,9 @@ gu_seq_binsearch_(GuSeq *seq, GuOrder *order, size_t elem_size, void *key)
 	return NULL;
 }
 
-bool
+GU_API bool
 gu_seq_binsearch_index_(GuSeq *seq, GuOrder *order, size_t elem_size,
-                        void *key, size_t *pindex)
+                        const void *key, size_t *pindex)
 {
 	size_t i = 0;
 	size_t j = seq->len-1;
@@ -364,14 +324,14 @@ gu_heap_siftup(GuBuf *buf, GuOrder *order,
    gu_heap_siftdown(buf, order, value, startpos, pos);
 }
 
-void
+GU_API void
 gu_buf_heap_push(GuBuf *buf, GuOrder *order, void *value)
 {
 	gu_buf_extend(buf);
 	gu_heap_siftdown(buf, order, value, 0, gu_buf_length(buf)-1);
 }
 
-void
+GU_API void
 gu_buf_heap_pop(GuBuf *buf, GuOrder *order, void* data_out)
 {
 	const void* last = gu_buf_trim(buf); // raises an error if empty
@@ -384,7 +344,7 @@ gu_buf_heap_pop(GuBuf *buf, GuOrder *order, void* data_out)
 	}
 }
 
-void
+GU_API void
 gu_buf_heap_replace(GuBuf *buf, GuOrder *order, void *value, void *data_out)
 {
 	gu_require(gu_buf_length(buf) > 0);
@@ -393,7 +353,7 @@ gu_buf_heap_replace(GuBuf *buf, GuOrder *order, void *value, void *data_out)
 	gu_heap_siftup(buf, order, value, 0);
 }
 
-void
+GU_API void
 gu_buf_heapify(GuBuf *buf, GuOrder *order)
 {
 	size_t middle = gu_buf_length(buf) / 2;
@@ -404,69 +364,3 @@ gu_buf_heapify(GuBuf *buf, GuOrder *order)
 		gu_heap_siftup(buf, order, value, i);
 	}
 }
-
-typedef struct GuBufOut GuBufOut;
-struct GuBufOut
-{
-	GuOutStream stream;
-	GuBuf* buf;
-};
-
-static size_t
-gu_buf_out_output(GuOutStream* stream, const uint8_t* src, size_t sz,
-		  GuExn* err)
-{
-	(void) err;
-	GuBufOut* bout = gu_container(stream, GuBufOut, stream);
-	GuBuf* buf = bout->buf;
-	gu_assert(sz % buf->elem_size == 0);
-	size_t len = sz / buf->elem_size;
-	gu_buf_push_n(bout->buf, src, len);
-	return len;
-}
-
-static uint8_t*
-gu_buf_outbuf_begin(GuOutStream* stream, size_t req, size_t* sz_out, GuExn* err)
-{
-	(void) req;
-	(void) err;
-	GuBufOut* bout = gu_container(stream, GuBufOut, stream);
-	GuBuf* buf = bout->buf;
-	size_t esz = buf->elem_size;
-	size_t len = gu_buf_length(buf);
-	gu_buf_require(buf, len + (req + esz - 1) / esz);
-	size_t avail = buf->avail_len;
-	gu_assert(len < avail);
-	*sz_out = esz * (avail - len);
-	return &buf->seq->data[len * esz];
-}
-
-static void
-gu_buf_outbuf_end(GuOutStream* stream, size_t sz, GuExn* err)
-{
-	(void) err;
-	GuBufOut* bout = gu_container(stream, GuBufOut, stream);
-	GuBuf* buf = bout->buf;
-	size_t len = gu_buf_length(buf);
-	size_t elem_size = buf->elem_size;
-	gu_require(sz % elem_size == 0);
-	gu_require(sz < elem_size * (len - buf->avail_len));
-	buf->seq->len = len + (sz / elem_size);
-}
-
-GuOut*
-gu_buf_out(GuBuf* buf, GuPool* pool)
-{
-	GuBufOut* bout = gu_new(GuBufOut, pool);
-	bout->stream.output = gu_buf_out_output;
-	bout->stream.begin_buf = gu_buf_outbuf_begin;
-	bout->stream.end_buf = gu_buf_outbuf_end;
-	bout->stream.flush = NULL;
-	bout->buf = buf;
-	return gu_new_out(&bout->stream, pool);
-}
-
-#include <gu/type.h>
-
-GU_DEFINE_KIND(GuSeq, GuOpaque);
-GU_DEFINE_KIND(GuBuf, abstract);
