@@ -27,7 +27,6 @@
 
 #include <gu/defs.h>
 #include <gu/mem.h>
-#include <gu/type.h>
 
 /** @name Variants
  * @{
@@ -36,10 +35,12 @@
 typedef uintptr_t GuVariant;
 
 
+GU_API_DECL
 void* gu_alloc_variant(uint8_t tag, 
 		       size_t size, size_t align, 
 		       GuVariant* variant_out, GuPool* pool);
 
+GU_API_DECL
 GuVariant gu_make_variant(uint8_t tag, 
 			  size_t size, size_t align, 
 			  const void* init, GuPool* pool);
@@ -69,8 +70,10 @@ enum {
 	GU_VARIANT_NULL = -1
 };
 
+GU_API_DECL
 int gu_variant_tag(GuVariant variant);
 
+GU_API_DECL
 void* gu_variant_data(GuVariant variant);
 
 
@@ -81,8 +84,8 @@ struct GuVariantInfo {
 	void* data;
 };
 
-GuVariantInfo gu_variant_open(GuVariant variant);
-GuVariant gu_variant_close(GuVariantInfo info);
+GU_API_DECL GuVariantInfo gu_variant_open(GuVariant variant);
+GU_API_DECL GuVariant gu_variant_close(GuVariantInfo info);
 
 /** @} */
 
@@ -98,70 +101,11 @@ gu_variant_from_ptr(const void* p)
 	return (uintptr_t) p;
 }
 
-extern const GuVariant gu_null_variant;
+GU_API_DECL extern const GuVariant gu_null_variant;
 
 static inline bool
 gu_variant_is_null(GuVariant v) {
 	return ((void*)v == NULL);
 }
 
-
-// variant
-
-typedef const struct GuConstructor GuConstructor;
-
-struct GuConstructor {
-	int c_tag;
-	const char* c_name;
-	const GuType* type;
-};
-
-#define GU_CONSTRUCTOR_V(ctag, c_type) {		\
-		.c_tag = ctag,	 \
-		.c_name = #ctag, \
-		.type = c_type \
-}
-
-#define GU_CONSTRUCTOR(ctag, t_) \
-	GU_CONSTRUCTOR_V(ctag, gu_type(t_))
-
-#define GU_CONSTRUCTOR_P(ctag, t_) \
-	GU_CONSTRUCTOR_V(ctag, gu_ptr_type(t_))
-
-#define GU_CONSTRUCTOR_S(ctag, t_, ...)		\
-	GU_CONSTRUCTOR_V(ctag, GU_TYPE_LIT(struct, t_, __VA_ARGS__))
-
-#define GU_CONSTRUCTOR_S1(ctag, t_, mem1_, type1_)			\
-	GU_CONSTRUCTOR_S(ctag, t_,					\
-			 GU_MEMBER(t_, mem1_, type1_))
-
-#define GU_CONSTRUCTOR_S2(ctag, t_, mem1_, type1_, mem2_, type2_)	\
-	GU_CONSTRUCTOR_S(ctag, t_,					\
-			 GU_MEMBER(t_, mem1_, type1_),			\
-			 GU_MEMBER(t_, mem2_, type2_))
-
-
-
-typedef struct {
-	int len;
-	GuConstructor* elems;
-} GuConstructors;
-
-typedef const struct GuVariantType GuVariantType, GuType_GuVariant;
-
-struct GuVariantType {
-	GuType_repr repr_base;
-	GuConstructors ctors;
-};
-
-#define GU_TYPE_INIT_GuVariant(k_, t_, ...) {			\
-	.repr_base = GU_TYPE_INIT_repr(k_, GuVariant, _),	\
-	.ctors = {								\
-		.len = GU_ARRAY_LEN(GuConstructor,GU_ID({__VA_ARGS__})),		\
-		.elems = ((GuConstructor[]){__VA_ARGS__})			\
-	} \
-}
-
-extern GU_DECLARE_KIND(GuVariant);
-
 #endif // GU_VARIANT_H_
