@@ -4,18 +4,23 @@
 #include <gu/variant.h>
 #include <gu/map.h>
 #include <gu/string.h>
-#include <gu/type.h>
 #include <gu/seq.h>
 #include <pgf/pgf.h>
 
 typedef struct PgfCCat PgfCCat;
-extern GU_DECLARE_TYPE(PgfCCat, abstract);
 
 typedef GuSeq PgfCCats;
 
-#define PgfCIdMap GuStringMap			 
-typedef PgfCIdMap PgfFlags; // PgfCId -> PgfLiteral
-extern GU_DECLARE_TYPE(PgfFlags, GuMap);
+#define PgfCIdMap GuStringMap
+
+typedef struct {
+	PgfCId name;
+	PgfLiteral value;
+} PgfFlag;
+
+typedef GuSeq PgfFlags;
+
+PGF_INTERNAL_DECL extern GuOrder pgf_flag_order[1];
 
 // PgfPatt
 
@@ -34,7 +39,7 @@ typedef enum {
 
 typedef	struct {
 	PgfCId ctor;
-	GuLength n_args;
+	size_t n_args;
 	PgfPatt args[];
 } PgfPattApp;
 
@@ -51,7 +56,7 @@ typedef struct {
 	PgfPatt patt;
 } PgfPattAs;
 
-typedef void PgfPattWild;
+typedef char PgfPattWild;
 
 typedef struct {
 	PgfPatt patt;
@@ -63,25 +68,35 @@ typedef struct {
 
 typedef struct {
 	PgfExpr body;
-	GuLength n_patts;
+	size_t n_patts;
 	PgfPatt patts[];
 } PgfEquation;
 
 typedef GuSeq PgfEquations;
 
+typedef void *PgfFunction;
+
 typedef struct {
 	PgfCId name;
 	PgfType* type;
 	int arity;
 	PgfEquations* defns; // maybe null
 	PgfExprProb ep;
-	void* predicate;
+	PgfFunction predicate;
+	struct {
+		PgfFunction code;
+		union {
+			size_t caf_offset;
+			PgfFunction* con;
+		};
+	} closure;
 } PgfAbsFun;
 
-extern GU_DECLARE_TYPE(PgfAbsFun, abstract);
+typedef GuSeq PgfAbsFuns;
+
+PGF_INTERNAL_DECL extern GuOrder pgf_absfun_order[1];
 
 typedef GuMap PgfMetaChildMap;
-extern GU_DECLARE_TYPE(PgfMetaChildMap, GuMap);
 
 typedef struct {
 	PgfCId name;
@@ -92,22 +107,55 @@ typedef struct {
 	void* predicate;
 } PgfAbsCat;
 
-extern GU_DECLARE_TYPE(PgfAbsCat, abstract);
+typedef GuSeq PgfAbsCats;
+
+PGF_INTERNAL_DECL extern GuOrder pgf_abscat_order[1];
+
+
+typedef struct PgfEvalGates PgfEvalGates;
 
 typedef struct {
 	PgfCId name;
 	PgfFlags* aflags;
-	PgfCIdMap* funs; // |-> PgfAbsFun*
-	PgfCIdMap* cats; // |-> PgfAbsCat*
+	PgfAbsFuns* funs;
+	PgfAbsCats* cats;
 	PgfAbsFun* abs_lin_fun;
+	PgfEvalGates* eval_gates;
 } PgfAbstr;
 
+typedef enum {
+  PGF_INSTR_CHECK_ARGS  =  0,
+  PGF_INSTR_CASE        =  1,
+  PGF_INSTR_CASE_LIT    =  2,
+  PGF_INSTR_SAVE        =  3,
+  PGF_INSTR_ALLOC       =  4,
+  PGF_INSTR_PUT_CONSTR  =  5,
+  PGF_INSTR_PUT_CLOSURE =  6,
+  PGF_INSTR_PUT_LIT     =  7,
+  PGF_INSTR_SET         =  8,
+  PGF_INSTR_SET_PAD     =  9,
+  PGF_INSTR_PUSH_FRAME  = 10,
+  PGF_INSTR_PUSH        = 11,
+  PGF_INSTR_TUCK        = 12,
+  PGF_INSTR_EVAL        = 13,
+  PGF_INSTR_DROP        = 16,
+  PGF_INSTR_JUMP        = 17,
+  PGF_INSTR_FAIL        = 18,
+  PGF_INSTR_PUSH_ACCUM  = 19,
+  PGF_INSTR_POP_ACCUM   = 20,
+  PGF_INSTR_ADD         = 21,
+} PgfInstruction;
+
+typedef GuSeq PgfConcrs;
+
+PGF_INTERNAL_DECL extern GuOrder pgf_concr_order[1];
+
 struct PgfPGF {
 	uint16_t major_version;
 	uint16_t minor_version;
 	PgfFlags* gflags;
 	PgfAbstr abstract;
-	PgfCIdMap* concretes; // |-> PgfConcr*
+	PgfConcrs* concretes;
 	GuPool* pool;         // the pool in which the grammar is allocated
 };
 
@@ -125,11 +173,9 @@ typedef struct {
 	 * represents. */
 } PgfCncCat;
 
-extern GU_DECLARE_TYPE(PgfCncCat, abstract);
-
 typedef GuSeq PgfTokens;
 
-bool
+PGF_INTERNAL_DECL bool
 pgf_tokens_equal(PgfTokens* t1, PgfTokens* t2);
 
 typedef GuSeq PgfSymbols;
@@ -146,19 +192,13 @@ typedef struct {
 typedef struct PgfItemConts PgfItemConts;
 
 typedef PgfCIdMap PgfPrintNames;
-extern GU_DECLARE_TYPE(PgfPrintNames, GuStringMap);
 
 typedef GuStringMap PgfCncFunOverloadMap;
-extern GU_DECLARE_TYPE(PgfCncFunOverloadMap, GuStringMap);
 
 typedef GuMap PgfCncOverloadMap;
-extern GU_DECLARE_TYPE(PgfCncOverloadMap, GuMap);
 
 typedef struct PgfItem PgfItem;
 
-typedef GuMap PgfCallbacksMap;
-extern GU_DECLARE_TYPE(PgfCallbacksMap, GuMap);
-
 typedef GuVariant PgfSymbol;
 
 typedef enum {
@@ -169,7 +209,10 @@ typedef enum {
 	PGF_SYMBOL_KP,
 	PGF_SYMBOL_BIND,
 	PGF_SYMBOL_SOFT_BIND,
-	PGF_SYMBOL_NE
+	PGF_SYMBOL_NE,
+	PGF_SYMBOL_SOFT_SPACE,
+	PGF_SYMBOL_CAPIT,
+	PGF_SYMBOL_ALL_CAPIT,
 } PgfSymbolTag;
 
 typedef struct {
@@ -191,18 +234,24 @@ typedef struct PgfSymbolKP
 	/**< Default form that this symbol takes if none of of the
 	 * variant forms is triggered. */
 
-	GuLength n_forms;
+	size_t n_forms;
 	PgfAlternative forms[]; 
 	/**< Variant forms whose choise depends on the following
 	 * symbol. */
 } PgfSymbolKP;
 
 typedef struct {
+	char nothing[0]; // Empty struct
 } PgfSymbolNE;
 
 typedef struct {
+	char nothing[0]; // Empty struct
 } PgfSymbolBIND;
 
+typedef struct {
+	char nothing[0]; // Empty struct
+} PgfSymbolCAPIT;
+
 typedef GuBuf PgfProductionIdx;
 
 typedef struct {
@@ -216,7 +265,7 @@ typedef struct {
 	PgfAbsFun* absfun;
 	PgfExprProb *ep;
     int funid;
-	GuLength n_lins;
+	size_t n_lins;
 	PgfSequence* lins[];
 } PgfCncFun;
 
@@ -232,16 +281,13 @@ struct PgfConcr {
 	PgfCncOverloadMap* coerce_idx;
     PgfCncFuns* cncfuns;
     PgfSequences* sequences;
-    GuBuf* pre_sequences;
 	PgfCIdMap* cnccats;
-	PgfCallbacksMap* callbacks;
 	int total_cats;
 	
 	GuPool* pool;     // if the language is loaded separately then this is the pool
 	GuFinalizer fin;  // and this is the finalizer in the pool of the whole grammar
 };
 
-extern GU_DECLARE_TYPE(PgfConcr, abstract);
 
 
 // PgfProduction
@@ -251,8 +297,7 @@ typedef GuVariant PgfProduction;
 typedef enum {
 	PGF_PRODUCTION_APPLY,
 	PGF_PRODUCTION_COERCE,
-	PGF_PRODUCTION_EXTERN,
-	PGF_PRODUCTION_META
+	PGF_PRODUCTION_EXTERN
 } PgfProductionTag;
 
 typedef struct {
@@ -286,7 +331,6 @@ typedef struct {
 } PgfProductionMeta;
 
 typedef GuSeq PgfProductionSeq;
-extern GU_DECLARE_TYPE(PgfProductionSeq, abstract);
 
 typedef struct {
 	PgfCCat* ccat;
