@@ -1,12 +1,13 @@
 #include "pgf.h"
 #include <gu/assert.h>
 #include <gu/utf8.h>
+#include <gu/seq.h>
 #include <ctype.h>
 #include <stdio.h>
 #include <stdlib.h>
 
 
-PgfExpr
+static PgfExpr
 pgf_expr_unwrap(PgfExpr expr)
 {
 	while (true) {
@@ -28,7 +29,7 @@ pgf_expr_unwrap(PgfExpr expr)
 	}
 }
 
-int
+PGF_API int
 pgf_expr_arity(PgfExpr expr)
 {
 	int n = 0;
@@ -50,7 +51,7 @@ pgf_expr_arity(PgfExpr expr)
 	}
 }
 
-PgfApplication*
+PGF_API PgfApplication*
 pgf_expr_unapply(PgfExpr expr, GuPool* pool)
 {
 	int arity = pgf_expr_arity(expr);
@@ -73,56 +74,130 @@ pgf_expr_unapply(PgfExpr expr, GuPool* pool)
 	return appl;
 }
 
-GU_DEFINE_TYPE(PgfBindType, enum,
-	       GU_ENUM_C(PgfBindType, PGF_BIND_TYPE_EXPLICIT),
-	       GU_ENUM_C(PgfBindType, PGF_BIND_TYPE_IMPLICIT));
+PGF_API PgfExpr
+pgf_expr_apply(PgfApplication* app, GuPool* pool)
+{
+	PgfExpr expr;
 
-GU_DEFINE_TYPE(PgfLiteral, GuVariant,
-	       GU_CONSTRUCTOR_S(PGF_LITERAL_STR, PgfLiteralStr,
-				GU_MEMBER(PgfLiteralStr, val, GuString)),
-	       GU_CONSTRUCTOR_S(PGF_LITERAL_INT, PgfLiteralInt,
-				GU_MEMBER(PgfLiteralInt, val, int)),
-	       GU_CONSTRUCTOR_S(PGF_LITERAL_FLT, PgfLiteralFlt,
-				GU_MEMBER(PgfLiteralFlt, val, double)));
+	size_t len = strlen(app->fun);
+	PgfExprFun *efun =
+		gu_new_flex_variant(PGF_EXPR_FUN,
+					        PgfExprFun,
+					        fun, len+1,
+					        &expr, pool);
+	strcpy(efun->fun, app->fun);
 
-GU_DECLARE_TYPE(PgfType, struct);
+	for (int i = 0; i < app->n_args; i++) {
+		expr = gu_new_variant_i(pool, 
+				                PGF_EXPR_APP, PgfExprApp,
+						        .fun = expr,
+						        .arg = app->args[i]);
+	}
+	
+	return expr;
+}
 
-GU_DEFINE_TYPE(PgfHypo, struct,
-	       GU_MEMBER(PgfHypo, bind_type, PgfBindType),
-	       GU_MEMBER(PgfHypo, cid, PgfCId),
-	       GU_MEMBER_P(PgfHypo, type, PgfType));
+PGF_API PgfExpr
+pgf_expr_abs(PgfBindType bind_type, PgfCId id, PgfExpr body, GuPool* pool)
+{
+	return gu_new_variant_i(pool, 
+				            PGF_EXPR_ABS, PgfExprAbs,
+						    .bind_type = bind_type,
+						    .id = id,
+						    .body = body);
+}
 
-GU_DEFINE_TYPE(
-	PgfExpr, GuVariant,
-	GU_CONSTRUCTOR_S(
-		PGF_EXPR_ABS, PgfExprAbs,
-		GU_MEMBER(PgfExprAbs, bind_type, PgfBindType),
-		GU_MEMBER(PgfExprAbs, id, GuString),
-		GU_MEMBER(PgfExprAbs, body, PgfExpr)),
-	GU_CONSTRUCTOR_S(
-		PGF_EXPR_APP, PgfExprApp,
-		GU_MEMBER(PgfExprApp, fun, PgfExpr),
-		GU_MEMBER(PgfExprApp, arg, PgfExpr)),
-	GU_CONSTRUCTOR_S(
-		PGF_EXPR_LIT, PgfExprLit, 
-		GU_MEMBER(PgfExprLit, lit, PgfLiteral)),
-	GU_CONSTRUCTOR_S(
-		PGF_EXPR_META, PgfExprMeta,
-		GU_MEMBER(PgfExprMeta, id, int)),
-	GU_CONSTRUCTOR_S(
-		PGF_EXPR_FUN, PgfExprFun,
-		GU_MEMBER(PgfExprFun, fun, GuString)),
-	GU_CONSTRUCTOR_S(
-		PGF_EXPR_VAR, PgfExprVar,
-		GU_MEMBER(PgfExprVar, var, int)),
-	GU_CONSTRUCTOR_S(
-		PGF_EXPR_TYPED, PgfExprTyped,
-		GU_MEMBER(PgfExprTyped, expr, PgfExpr),
-		GU_MEMBER_P(PgfExprTyped, type, PgfType)),
-	GU_CONSTRUCTOR_S(
-		PGF_EXPR_IMPL_ARG, PgfExprImplArg,
-		GU_MEMBER(PgfExprImplArg, expr, PgfExpr)));
+PGF_API PgfExprAbs*
+pgf_expr_unabs(PgfExpr expr)
+{
+	GuVariantInfo i = gu_variant_open(expr);
+	if (i.tag == PGF_EXPR_ABS) {
+		return (PgfExprAbs*) i.data;
+	}
 
+	return NULL;
+}
+
+PGF_API PgfExpr
+pgf_expr_meta(int id, GuPool* pool)
+{
+	return gu_new_variant_i(pool, 
+				            PGF_EXPR_META, PgfExprMeta,
+						    .id = id);
+}
+
+PGF_API PgfExprMeta*
+pgf_expr_unmeta(PgfExpr expr)
+{
+	GuVariantInfo i = gu_variant_open(expr);
+	if (i.tag == PGF_EXPR_META) {
+		return (PgfExprMeta*) i.data;
+	}
+
+	return NULL;
+}
+
+PGF_API PgfExpr
+pgf_expr_string(GuString str, GuPool* pool)
+{
+	PgfLiteral lit;
+	PgfLiteralStr* plit = 
+		gu_new_flex_variant(PGF_LITERAL_STR,
+		                    PgfLiteralStr,
+		                    val, strlen(str)+1,
+		                    &lit, pool);
+	strcpy(plit->val, str);
+	return gu_new_variant_i(pool,
+	                        PGF_EXPR_LIT,
+	                        PgfExprLit,
+	                        lit);
+}
+
+PGF_API PgfExpr
+pgf_expr_int(int val, GuPool* pool)
+{
+	PgfLiteral lit;
+	PgfLiteralInt* plit = 
+		gu_new_variant(PGF_LITERAL_INT,
+		               PgfLiteralInt,
+		               &lit, pool);
+	plit->val = val;
+	return gu_new_variant_i(pool,
+	                        PGF_EXPR_LIT,
+	                        PgfExprLit,
+	                        lit);
+}
+
+PGF_API PgfExpr
+pgf_expr_float(double val, GuPool* pool)
+{
+	PgfLiteral lit;
+	PgfLiteralFlt* plit = 
+		gu_new_variant(PGF_LITERAL_FLT,
+		               PgfLiteralFlt,
+		               &lit, pool);
+	plit->val = val;
+	return gu_new_variant_i(pool,
+	                        PGF_EXPR_LIT,
+	                        PgfExprLit,
+	                        lit);
+}
+
+PGF_API void*
+pgf_expr_unlit(PgfExpr expr, int lit_tag)
+{
+	expr = pgf_expr_unwrap(expr);
+	GuVariantInfo i = gu_variant_open(expr);
+	if (i.tag == PGF_EXPR_LIT) {
+		PgfExprLit* elit = i.data;
+		GuVariantInfo i2 = gu_variant_open(elit->lit);
+		if (i2.tag == lit_tag) {
+			return i2.data;
+		}
+	}
+		
+	return NULL;
+}
 
 typedef struct PgfExprParser PgfExprParser;
 
@@ -138,6 +213,7 @@ typedef enum {
 	PGF_TOKEN_RTRIANGLE,
 	PGF_TOKEN_COMMA,
 	PGF_TOKEN_COLON,
+	PGF_TOKEN_SEMI,
 	PGF_TOKEN_WILD,
 	PGF_TOKEN_IDENT,
 	PGF_TOKEN_INT,
@@ -153,14 +229,14 @@ struct PgfExprParser {
 	GuPool* expr_pool;
 	GuPool* tmp_pool;
 	PGF_TOKEN_TAG token_tag;
-	GuBuf* token_value;
-	int ch;
+	GuStringBuf* token_value;
+	GuUCS ch;
 };
 
 static void
 pgf_expr_parser_getc(PgfExprParser* parser)
 {
-	parser->ch = gu_in_u8(parser->in, parser->err);
+	parser->ch = gu_in_utf8(parser->in, parser->err);
 	if (!gu_ok(parser->err)) {
 		gu_exn_clear(parser->err);
 		parser->ch = EOF;
@@ -267,54 +343,58 @@ pgf_expr_parser_token(PgfExprParser* parser)
 		pgf_expr_parser_getc(parser);
 		parser->token_tag = PGF_TOKEN_COLON;
 		break;
+	case ';':
+		pgf_expr_parser_getc(parser);
+		parser->token_tag = PGF_TOKEN_SEMI;
+		break;
 	case '\'':
 		pgf_expr_parser_getc(parser);
 
-		GuBuf* chars = gu_new_buf(char, parser->tmp_pool);
+		GuStringBuf* chars = gu_new_string_buf(parser->tmp_pool);
 		while (parser->ch != '\'' && parser->ch != EOF) {
 			if (parser->ch == '\\') {
 				pgf_expr_parser_getc(parser);
 			}
-			gu_buf_push(chars, char, parser->ch);
+			gu_out_utf8(parser->ch, gu_string_buf_out(chars), parser->err);
 			pgf_expr_parser_getc(parser);
 		}
 		if (parser->ch == '\'') {
 			pgf_expr_parser_getc(parser);
-			gu_buf_push(chars, char, 0);
+			gu_out_utf8(0, gu_string_buf_out(chars), parser->err);
 			parser->token_tag   = PGF_TOKEN_IDENT;
 			parser->token_value = chars;
 		}
 		break;
 	default: {
-		GuBuf* chars = gu_new_buf(char, parser->tmp_pool);
+		GuStringBuf* chars = gu_new_string_buf(parser->tmp_pool);
 
 		if (pgf_is_ident_first(parser->ch)) {
 			do {
-				gu_buf_push(chars, char, parser->ch);
+				gu_out_utf8(parser->ch, gu_string_buf_out(chars), parser->err);
 				pgf_expr_parser_getc(parser);
 			} while (pgf_is_ident_rest(parser->ch));
-			gu_buf_push(chars, char, 0);
+			gu_out_utf8(0, gu_string_buf_out(chars), parser->err);
 			parser->token_tag   = PGF_TOKEN_IDENT;
 			parser->token_value = chars;
 		} else if (isdigit(parser->ch)) {
 			while (isdigit(parser->ch)) {
-				gu_buf_push(chars, char, parser->ch);
+				gu_out_utf8(parser->ch, gu_string_buf_out(chars), parser->err);
 				pgf_expr_parser_getc(parser);
 			}
 			
 			if (parser->ch == '.') {
-				gu_buf_push(chars, char, parser->ch);
+				gu_out_utf8(parser->ch, gu_string_buf_out(chars), parser->err);
 				pgf_expr_parser_getc(parser);
 				
 				while (isdigit(parser->ch)) {
-					gu_buf_push(chars, char, parser->ch);
+					gu_out_utf8(parser->ch, gu_string_buf_out(chars), parser->err);
 					pgf_expr_parser_getc(parser);
 				}
-				gu_buf_push(chars, char, 0);
+				gu_out_utf8(0, gu_string_buf_out(chars), parser->err);
 				parser->token_tag   = PGF_TOKEN_FLT;
 				parser->token_value = chars;
 			} else {
-				gu_buf_push(chars, char, 0);
+				gu_out_utf8(0, gu_string_buf_out(chars), parser->err);
 				parser->token_tag   = PGF_TOKEN_INT;
 				parser->token_value = chars;
 			}
@@ -322,13 +402,39 @@ pgf_expr_parser_token(PgfExprParser* parser)
 			pgf_expr_parser_getc(parser);
 
 			while (parser->ch != '"' && parser->ch != EOF) {
-				gu_buf_push(chars, char, parser->ch);
+				if (parser->ch == '\\') {
+					pgf_expr_parser_getc(parser);
+					switch (parser->ch) {
+					case '\\':
+						gu_out_utf8('\\', gu_string_buf_out(chars), parser->err);
+						break;
+					case '"':
+						gu_out_utf8('\"', gu_string_buf_out(chars), parser->err);
+						break;
+					case 'n':
+						gu_out_utf8('\n', gu_string_buf_out(chars), parser->err);
+						break;
+					case 'r':
+						gu_out_utf8('\r', gu_string_buf_out(chars), parser->err);
+						break;
+					case 'b':
+						gu_out_utf8('\b', gu_string_buf_out(chars), parser->err);
+						break;
+					case 't':
+						gu_out_utf8('\t', gu_string_buf_out(chars), parser->err);
+						break;
+					default:
+						return;
+					}
+				} else {
+					gu_out_utf8(parser->ch, gu_string_buf_out(chars), parser->err);
+				}
 				pgf_expr_parser_getc(parser);
 			}
 			
 			if (parser->ch == '"') {
 				pgf_expr_parser_getc(parser);
-				gu_buf_push(chars, char, 0);
+				gu_out_utf8(0, gu_string_buf_out(chars), parser->err);
 				parser->token_tag   = PGF_TOKEN_STR;
 				parser->token_value = chars;
 			}
@@ -371,6 +477,8 @@ pgf_expr_parser_term(PgfExprParser* parser)
 	case PGF_TOKEN_LTRIANGLE: {
 		pgf_expr_parser_token(parser);
 		PgfExpr expr = pgf_expr_parser_expr(parser);
+		if (gu_variant_is_null(expr))
+			return gu_null_variant;
 		
 		if (parser->token_tag != PGF_TOKEN_COLON) {
 			return gu_null_variant;
@@ -378,6 +486,8 @@ pgf_expr_parser_term(PgfExprParser* parser)
 		pgf_expr_parser_token(parser);
 
 		PgfType* type = pgf_expr_parser_type(parser);
+		if (type == NULL)
+			return gu_null_variant;
 
 		if (parser->token_tag != PGF_TOKEN_RTRIANGLE) {
 			return gu_null_variant;
@@ -391,13 +501,21 @@ pgf_expr_parser_term(PgfExprParser* parser)
 	}
 	case PGF_TOKEN_QUESTION: {
 		pgf_expr_parser_token(parser);
+
+		PgfMetaId id = 0;
+		if (parser->token_tag == PGF_TOKEN_INT) {
+			char* str =
+				gu_string_buf_data(parser->token_value);
+			id = atoi(str);
+			pgf_expr_parser_token(parser);
+		}
 		return gu_new_variant_i(parser->expr_pool,
 					            PGF_EXPR_META,
 					            PgfExprMeta,
-					            0);
+					            id);
 	}
 	case PGF_TOKEN_IDENT: {
-		PgfCId id = gu_buf_data(parser->token_value);
+		PgfCId id = gu_string_buf_data(parser->token_value);
 		pgf_expr_parser_token(parser);
 		PgfExpr e;
 		PgfExprFun* fun =
@@ -410,7 +528,7 @@ pgf_expr_parser_term(PgfExprParser* parser)
 	}
 	case PGF_TOKEN_INT: {
 		char* str = 
-			gu_buf_data(parser->token_value);
+			gu_string_buf_data(parser->token_value);
 		int n = atoi(str);
 		pgf_expr_parser_token(parser);
 		PgfLiteral lit = 
@@ -425,24 +543,16 @@ pgf_expr_parser_term(PgfExprParser* parser)
 	}
 	case PGF_TOKEN_STR: {
 		char* str =
-			gu_buf_data(parser->token_value);
+			gu_string_buf_data(parser->token_value);
 		pgf_expr_parser_token(parser);
-		PgfLiteral lit;
-		PgfLiteralStr* plit = 
-			gu_new_flex_variant(PGF_LITERAL_STR,
-			                    PgfLiteralStr,
-			                    val, strlen(str)+1,
-			                    &lit, parser->expr_pool);
-		strcpy(plit->val, str);
-		return gu_new_variant_i(parser->expr_pool,
-		                        PGF_EXPR_LIT,
-		                        PgfExprLit,
-		                        lit);
+		return pgf_expr_string(str, parser->expr_pool);
 	}
 	case PGF_TOKEN_FLT: {
 		char* str = 
-			gu_buf_data(parser->token_value);
-		double d = atof(str);
+			gu_string_buf_data(parser->token_value);
+		double d;
+		if (!gu_string_to_double(str,&d))
+			return gu_null_variant;
 		pgf_expr_parser_token(parser);
 		PgfLiteral lit = 
 			gu_new_variant_i(parser->expr_pool,
@@ -502,7 +612,7 @@ pgf_expr_parser_bind(PgfExprParser* parser, GuBuf* binds)
 	for (;;) {
 		if (parser->token_tag == PGF_TOKEN_IDENT) {
 			var =
-				gu_string_copy(gu_buf_data(parser->token_value), parser->expr_pool);
+				gu_string_copy(gu_string_buf_data(parser->token_value), parser->expr_pool);
 			pgf_expr_parser_token(parser);
 		} else if (parser->token_tag == PGF_TOKEN_WILD) {
 			var = "_";
@@ -591,7 +701,9 @@ pgf_expr_parser_expr(PgfExprParser* parser)
 		       parser->token_tag != PGF_TOKEN_RPAR &&
 		       parser->token_tag != PGF_TOKEN_RCURLY &&
 		       parser->token_tag != PGF_TOKEN_RTRIANGLE &&
-		       parser->token_tag != PGF_TOKEN_COLON) {
+		       parser->token_tag != PGF_TOKEN_COLON &&
+		       parser->token_tag != PGF_TOKEN_COMMA &&
+		       parser->token_tag != PGF_TOKEN_SEMI) {
 			PgfExpr arg = pgf_expr_parser_arg(parser);
 			if (gu_variant_is_null(arg))
 				return gu_null_variant;
@@ -621,7 +733,7 @@ pgf_expr_parser_hypos(PgfExprParser* parser, GuBuf* hypos)
 
 		if (parser->token_tag == PGF_TOKEN_IDENT) {
 			var =
-				gu_string_copy(gu_buf_data(parser->token_value), parser->expr_pool);
+				gu_string_copy(gu_string_buf_data(parser->token_value), parser->expr_pool);
 			pgf_expr_parser_token(parser);
 		} else if (parser->token_tag == PGF_TOKEN_WILD) {
 			var = "_";
@@ -661,7 +773,7 @@ pgf_expr_parser_atom(PgfExprParser* parser)
 		return NULL;
 
 	PgfCId cid =
-		gu_string_copy(gu_buf_data(parser->token_value), parser->expr_pool);
+		gu_string_copy(gu_string_buf_data(parser->token_value), parser->expr_pool);
 	pgf_expr_parser_token(parser);
 
 	GuBuf* args = gu_new_buf(PgfExpr, parser->tmp_pool);
@@ -778,11 +890,11 @@ pgf_new_parser(GuIn* in, GuPool* pool, GuPool* tmp_pool, GuExn* err)
 	return parser;
 }
 
-PgfExpr
+PGF_API PgfExpr
 pgf_read_expr(GuIn* in, GuPool* pool, GuExn* err)
 {
 	GuPool* tmp_pool = gu_new_pool();
-	PgfExprParser* parser = 
+	PgfExprParser* parser =
 		pgf_new_parser(in, pool, tmp_pool, err);
 	PgfExpr expr = pgf_expr_parser_expr(parser);
 	if (parser->token_tag != PGF_TOKEN_EOF)
@@ -791,7 +903,95 @@ pgf_read_expr(GuIn* in, GuPool* pool, GuExn* err)
 	return expr;
 }
 
-PgfType*
+PGF_API int
+pgf_read_expr_tuple(GuIn* in,
+                    size_t n_exprs, PgfExpr exprs[],
+                    GuPool* pool, GuExn* err)
+{
+	GuPool* tmp_pool = gu_new_pool();
+	PgfExprParser* parser =
+		pgf_new_parser(in, pool, tmp_pool, err);
+	if (parser->token_tag != PGF_TOKEN_LTRIANGLE)
+		goto fail;
+	pgf_expr_parser_token(parser);
+	for (size_t i = 0; i < n_exprs; i++) {
+		if (i > 0) {
+			if (parser->token_tag != PGF_TOKEN_COMMA)
+				goto fail;
+			pgf_expr_parser_token(parser);
+		}
+
+		exprs[i] = pgf_expr_parser_expr(parser);
+		if (gu_variant_is_null(exprs[i]))
+			goto fail;
+	}
+	if (parser->token_tag != PGF_TOKEN_RTRIANGLE)
+		goto fail;
+	pgf_expr_parser_token(parser);
+	if (parser->token_tag != PGF_TOKEN_EOF)
+		goto fail;
+	gu_pool_free(tmp_pool);
+
+	return 1;
+	
+fail:
+	gu_pool_free(tmp_pool);
+	return 0;
+}
+
+PGF_API GuSeq*
+pgf_read_expr_matrix(GuIn* in,
+                     size_t n_exprs,
+                     GuPool* pool, GuExn* err)
+{
+	GuPool* tmp_pool = gu_new_pool();
+	PgfExprParser* parser =
+		pgf_new_parser(in, pool, tmp_pool, err);
+	if (parser->token_tag != PGF_TOKEN_LTRIANGLE)
+		goto fail;
+	pgf_expr_parser_token(parser);
+
+	GuBuf* buf = gu_new_buf(PgfExpr, pool);
+	
+	if (parser->token_tag != PGF_TOKEN_RTRIANGLE) {	
+		for (;;) {
+			PgfExpr* exprs = gu_buf_extend_n(buf, n_exprs);
+
+			for (size_t i = 0; i < n_exprs; i++) {
+				if (i > 0) {
+					if (parser->token_tag != PGF_TOKEN_COMMA)
+						goto fail;
+					pgf_expr_parser_token(parser);
+				}
+
+				exprs[i] = pgf_expr_parser_expr(parser);
+				if (gu_variant_is_null(exprs[i]))
+					goto fail;
+			}
+
+			if (parser->token_tag != PGF_TOKEN_SEMI)
+				break;
+
+			pgf_expr_parser_token(parser);
+		}
+
+		if (parser->token_tag != PGF_TOKEN_RTRIANGLE)
+			goto fail;
+	}
+
+	pgf_expr_parser_token(parser);
+	if (parser->token_tag != PGF_TOKEN_EOF)
+		goto fail;
+	gu_pool_free(tmp_pool);
+
+	return gu_buf_data_seq(buf);
+
+fail:
+	gu_pool_free(tmp_pool);
+	return NULL;
+}
+
+PGF_API PgfType*
 pgf_read_type(GuIn* in, GuPool* pool, GuExn* err)
 {
 	GuPool* tmp_pool = gu_new_pool();
@@ -804,7 +1004,7 @@ pgf_read_type(GuIn* in, GuPool* pool, GuExn* err)
 	return type;
 }
 
-bool
+PGF_API bool
 pgf_literal_eq(PgfLiteral lit1, PgfLiteral lit2)
 {
 	GuVariantInfo ei1 = gu_variant_open(lit1);
@@ -836,7 +1036,7 @@ pgf_literal_eq(PgfLiteral lit1, PgfLiteral lit2)
     return false;
 }
 
-bool
+PGF_API bool
 pgf_expr_eq(PgfExpr e1, PgfExpr e2)
 {
 	GuVariantInfo ei1 = gu_variant_open(e1);
@@ -896,7 +1096,7 @@ pgf_expr_eq(PgfExpr e1, PgfExpr e2)
 	return false;
 }
 
-GuHash
+PGF_API GuHash
 pgf_literal_hash(GuHash h, PgfLiteral lit)
 {
 	GuVariantInfo i = gu_variant_open(lit);
@@ -924,7 +1124,7 @@ pgf_literal_hash(GuHash h, PgfLiteral lit)
 	return h;
 }
 
-GuHash
+PGF_API GuHash
 pgf_expr_hash(GuHash h, PgfExpr e)
 {
 	GuVariantInfo ei = gu_variant_open(e);
@@ -976,7 +1176,7 @@ pgf_expr_hash(GuHash h, PgfExpr e)
 	return h;
 }
 
-void
+PGF_API void
 pgf_print_cid(PgfCId id,
 			  GuOut* out, GuExn* err)
 {
@@ -1000,7 +1200,7 @@ pgf_print_cid(PgfCId id,
 	}
 }
 
-void
+PGF_API void
 pgf_print_literal(PgfLiteral lit,
 			      GuOut* out, GuExn* err)
 {
@@ -1009,7 +1209,32 @@ pgf_print_literal(PgfLiteral lit,
 	case PGF_LITERAL_STR: {
 		PgfLiteralStr* lit = ei.data;
         gu_putc('"', out, err);
-		gu_string_write(lit->val, out, err);
+        const uint8_t* s = (uint8_t*) lit->val;
+        while (*s) {
+			GuUCS c = gu_utf8_decode(&s);
+			switch (c) {
+			case '\\':
+				gu_puts("\\\\", out, err);
+				break;
+			case '"':
+				gu_puts("\\\"", out, err);
+				break;
+			case '\n':
+				gu_puts("\\n", out, err);
+				break;
+			case '\r':
+				gu_puts("\\r", out, err);
+				break;
+			case '\b':
+				gu_puts("\\b", out, err);
+				break;
+			case '\t':
+				gu_puts("\\t", out, err);
+				break;
+			default:
+				gu_out_utf8(c, out, err);
+			}
+		}
         gu_putc('"', out, err);
 		break;
     }
@@ -1020,7 +1245,7 @@ pgf_print_literal(PgfLiteral lit,
     }
 	case PGF_LITERAL_FLT: {
         PgfLiteralFlt* lit = ei.data;
-		gu_printf(out, err, "%lg", lit->val);
+		gu_printf(out, err, "%lf", lit->val);
         break;
     }
 	default:
@@ -1028,7 +1253,7 @@ pgf_print_literal(PgfLiteral lit,
     }
 }
 
-void
+PGF_API void
 pgf_print_expr(PgfExpr expr, PgfPrintContext* ctxt, int prec,
                GuOut* out, GuExn* err)
 {
@@ -1099,9 +1324,13 @@ pgf_print_expr(PgfExpr expr, PgfPrintContext* ctxt, int prec,
         pgf_print_literal(lit->lit, out, err);
 		break;
 	}
-	case PGF_EXPR_META:
+	case PGF_EXPR_META: {
+		PgfExprMeta* meta = ei.data;
 		gu_putc('?', out, err);
+		if (meta->id > 0)
+			gu_printf(out, err, "%d", meta->id);
 		break;
+	}
 	case PGF_EXPR_FUN: {
 		PgfExprFun* fun = ei.data;
 		pgf_print_cid(fun->fun, out, err);
@@ -1114,6 +1343,7 @@ pgf_print_expr(PgfExpr expr, PgfPrintContext* ctxt, int prec,
 		PgfPrintContext* c = ctxt;
 		while (c != NULL && var > 0) {
 			c = ctxt->next;
+			var--;
 		}
 		
 		if (c == NULL) {
@@ -1144,7 +1374,7 @@ pgf_print_expr(PgfExpr expr, PgfPrintContext* ctxt, int prec,
 	}
 }
 
-PgfPrintContext*
+PGF_API PgfPrintContext*
 pgf_print_hypo(PgfHypo *hypo, PgfPrintContext* ctxt, int prec,
                GuOut *out, GuExn *err)
 {
@@ -1176,7 +1406,7 @@ pgf_print_hypo(PgfHypo *hypo, PgfPrintContext* ctxt, int prec,
 	return new_ctxt;
 }
 
-void
+PGF_API void
 pgf_print_type(PgfType *type, PgfPrintContext* ctxt, int prec,
                GuOut *out, GuExn *err)
 {
@@ -1223,7 +1453,20 @@ pgf_print_type(PgfType *type, PgfPrintContext* ctxt, int prec,
 	}
 }
 
-bool
+PGF_API void
+pgf_print_expr_tuple(size_t n_exprs, PgfExpr exprs[], PgfPrintContext* ctxt,
+                     GuOut* out, GuExn* err)
+{
+	gu_putc('<', out, err);
+	for (size_t i = 0; i < n_exprs; i++) {
+		if (i > 0)
+			gu_putc(',', out, err);
+		pgf_print_expr(exprs[i], ctxt, 0, out, err);
+	}
+	gu_putc('>', out, err);
+}
+
+PGF_API bool
 pgf_type_eq(PgfType* t1, PgfType* t2)
 {
 	if (gu_seq_length(t1->hypos) != gu_seq_length(t2->hypos))
