@@ -1,10 +1,10 @@
 #ifndef PGF_GRAPHVIZ_H_
 #define PGF_GRAPHVIZ_H_
 
-void
+PGF_API_DECL void
 pgf_graphviz_abstract_tree(PgfPGF* pgf, PgfExpr expr, GuOut* out, GuExn* err);
 
-void
+PGF_API_DECL void
 pgf_graphviz_parse_tree(PgfConcr* concr, PgfExpr expr, GuOut* out, GuExn* err);
 
 #endif
