@@ -49,7 +49,6 @@ struct {
 } jit_flags;
 
 struct jit_local_state {
-    int		 reglist;
     int		 framesize;
     int		 nextarg_get;
     int		 nextarg_put;
@@ -61,6 +60,7 @@ struct jit_local_state {
     jit_thumb_t	thumb;
     jit_insn* thumb_pc;
     jit_insn* thumb_tmp;
+    int tmp;
     /* hackish mostly to make test cases work; use arm instruction
      * set in jmpi if did not yet see a prolog */
     int		 after_prolog;
@@ -132,20 +132,14 @@ jit_v_order[JIT_V_NUM] = {
 #endif
 
 #define jit_movi_i(r0, i0) \
-{ \
-    int		i; \
-	if (!jit_no_set_flags() && r0 < 8 && !(i0 & 0xffffff80)) \
-	    T1_MOVI(r0, i0); \
-	else if ((i = encode_thumb_immediate(i0)) != -1) \
-	    T2_MOVI(r0, i); \
-	else if ((i = encode_thumb_immediate(~i0)) != -1) \
-	    T2_MVNI(r0, i); \
-	else { \
-	    T2_MOVWI(r0, _jit_US(i0)); \
-	    if (i0 & 0xffff0000) \
-			T2_MOVTI(r0, _jit_US((unsigned)i0 >> 16)); \
-	} \
-}
+	((!jit_no_set_flags() && r0 < 8 && !(i0 & 0xffffff80)) ? \
+	     T1_MOVI(r0, i0) : \
+	 ((_jitl.tmp = encode_thumb_immediate(i0)) != -1) ? \
+	     T2_MOVI(r0, _jitl.tmp) : \
+	 ((_jitl.tmp = encode_thumb_immediate(~i0)) != -1) ? \
+	     T2_MVNI(r0, _jitl.tmp) : \
+	 ( T2_MOVWI(r0, _jit_US(i0)), \
+	     ((i0 & 0xffff0000) ? T2_MOVTI(r0, _jit_US((unsigned)i0 >> 16)) : 0) ))
 
 #ifdef USE_THUMB_CODE
 #define jit_movi_p(r0, i0) \
@@ -362,18 +356,11 @@ jit_v_order[JIT_V_NUM] = {
 }
 #else
 #define jit_addi_i(r0, r1, i0) \
-{ \
-	int i; \
-	if ((i = encode_arm_immediate(i0)) != -1) \
-		_ADDI(r0, r1, i); \
-	else if ((i = encode_arm_immediate(-i0)) != -1) \
-		_SUBI(r0, r1, i); \
-	else { \
-		jit_gpr_t reg = r0 != r1 ? r0 : JIT_TMP; \
-		jit_movi_i(reg, i0); \
-		_ADD(r0, r1, reg); \
-	} \
-}
+	(encode_arm_immediate(i0) != -1) ? \
+	    _ADDI(r0, r1, encode_arm_immediate(i0)) : \
+	    ((encode_arm_immediate(-i0) != -1) ? \
+	        _SUBI(r0, r1, encode_arm_immediate(-i0)) : \
+	        (jit_movi_i(JIT_TMP, i0), _ADD(r0, r1, JIT_TMP)))
 #endif
 
 #ifdef USE_THUMB_CODE
@@ -653,7 +640,7 @@ jit_v_order[JIT_V_NUM] = {
 }
 #endif
 
-#define jit_mulr_i(r0, r1, r2)		jit_mulr_ui(_jit, r0, r1, r2)
+#define jit_mulr_i(r0, r1, r2)		jit_mulr_ui(r0, r1, r2)
 #ifdef USE_THUMB_CODE
 #define jit_mulr_ui(r0, r1, r2) \
 { \
@@ -1172,18 +1159,18 @@ next: \
 #define jit_jmpi(i0) \
 	(_jitl.thumb_tmp = _jit.x.pc, \
 	 (_jitl.after_prolog) ? \
-	     ((_s20P((((long)i0 - (long)l) >> 1) - 2)) ? \
-	          T2_B(encode_thumb_jump((((long)i0 - (long)l) >> 1) - 2)) : \
+	     ((_s20P((((long)i0 - (long)_jitl.thumb_tmp) >> 1) - 2)) ? \
+	          T2_B(encode_thumb_jump((((long)i0 - (long)_jitl.thumb_tmp) >> 1) - 2)) : \
 	          (jit_movi_p(JIT_TMP, i0), jit_jmpr(JIT_TMP))) : \
-         (assert(_s24P((((long)i0 - (long)l) >> 2) - 2)), \
-	      _CC_B(ARM_CC_AL, ((((long)i0 - (long)l) >> 2) - 2) & 0x00ffffff)) \
+         (assert(_s24P((((long)i0 - (long)_jitl.thumb_tmp) >> 2) - 2)), \
+	      _CC_B(ARM_CC_AL, ((((long)i0 - (long)_jitl.thumb_tmp) >> 2) - 2) & 0x00ffffff)) \
      _jitl.thumb_tmp)
 #else
 #define jit_jmpi(i0) \
-    (_jit.thumb_tmp = _jit.x.pc, \
-	 assert(_s24P((((long)i0 - (long)l) >> 2) - 2)), \
-	 _CC_B(ARM_CC_AL, ((((long)i0 - (long)l) >> 2) - 2) & 0x00ffffff), \
-	 _jit.thumb_tmp)
+    (_jitl.thumb_tmp = _jit.x.pc, \
+	 assert(_s24P((((long)i0 - (long)_jitl.thumb_tmp) >> 2) - 2)), \
+	 _CC_B(ARM_CC_AL, ((((long)i0 - (long)_jitl.thumb_tmp) >> 2) - 2) & 0x00ffffff), \
+	 _jitl.thumb_tmp)
 #endif
 
 #ifdef USE_THUMB_CODE
@@ -1214,7 +1201,7 @@ next: \
 	                (jit_movi_i(JIT_TMP, i1), T2_CMP(r0, JIT_TMP))))), \
 	 /* use only thumb2 conditional as does not know if will be patched */ \
 	 _jitl.thumb_tmp = _jit.x.pc, \
-	 assert(_s20P((((long)i0 - (long)l) >> 1) - 2)), \
+	 assert(_s20P((((long)i0 - (long)_jitl.thumb_tmp) >> 1) - 2)), \
 	 T2_CC_B(cc, encode_thumb_cc_jump((((long)i0 - (long)l) >> 1) - 2)), \
      _jitl.thumb_tmp)
 #else
@@ -1225,8 +1212,8 @@ next: \
 	           _CMNI(r0, encode_arm_immediate(-i1)) : \
 	           (jit_movi_i(JIT_TMP, i1), _CMP(r0, JIT_TMP)))), \
 	 _jitl.thumb_tmp = _jit.x.pc, \
-	 assert(_s24P((((long)i0 - (long)l) >> 2) - 2)), \
-	 _CC_B(cc, ((((long)i0 - (long)l) >> 2) - 2) & 0x00ffffff), \
+	 assert(_s24P((((long)i0 - (long)_jitl.thumb_tmp) >> 2) - 2)), \
+	 _CC_B(cc, ((((long)i0 - (long)_jitl.thumb_tmp) >> 2) - 2) & 0x00ffffff), \
      _jitl.thumb_tmp)
 #endif
 
@@ -1818,7 +1805,7 @@ next: \
 #endif
 
 #ifdef USE_THUMB_CODE
-#define jit_stxi_i(r0, r1, i0) \
+#define jit_stxi_i(i0, r0, r1) \
 { \
 	if ((r0|r1) < 8 && i0 >= 0 && !(i0 & 3) && (i0 >> 2) < 0x20) \
 		T1_STRI(r1, r0, i0 >> 2); \
@@ -1840,7 +1827,7 @@ next: \
 	} \
 }
 #else
-#define jit_stxi_i(r0, r1, i0) \
+#define jit_stxi_i(i0, r0, r1) \
 { \
 	if (i0 >= 0 && i0 <= 4095) \
 	    _STRI(r1, r0, i0); \
@@ -2016,39 +2003,36 @@ next: \
 	\
     /* patch alloca and stack adjustment */ \
     _jitl.stack = (int *)_jit.x.pc; \
-	\
-    if (jit_swf_p()) \
-	/* 6 soft double precision float registers */ \
-		_jitl.alloca_offset = 48; \
-    else \
-		_jitl.alloca_offset = 0; \
-	\
-	jit_movi_p(JIT_TMP, (void *)_jitl.alloca_offset); \
-    jit_subr_i(JIT_SP, JIT_SP, JIT_TMP); \
     _jitl.stack_length = _jitl.stack_offset = 0; \
 }
 #endif
 
 #ifdef USE_THUMB_CODE
-#define jit_callr(r0) T1_BLX(r0)
+#define jit_callr(r0) \
+    (T1_ADDI3(JIT_LR, JIT_PC, 4), \
+     T2_PUSH((1<<JIT_LR)), \
+     T1_MOV(JIT_PC, r0))
 #else
-#define jit_callr(r0) _BLX(r0)
+#define jit_callr(r0) \
+    (_ADDI(JIT_LR, JIT_PC, 4), \
+     _PUSH((1<<JIT_LR)), \
+     _MOV(JIT_PC, r0))
 #endif
 
 #ifdef USE_THUMB_CODE
 #define jit_calli(i0) \
     (_jitl.thumb_tmp = jit_movi_p(JIT_TMP, i0), \
-	 T1_BLX(JIT_TMP), \
+	 jit_callr(JIT_TMP), \
 	 _jitl.thumb_tmp)
 #else
 #define jit_calli(i0) \
     (_jitl.thumb_tmp = jit_movi_p(JIT_TMP, i0), \
-	 _BLX(JIT_TMP), \
+	 jit_callr(JIT_TMP), \
 	 _jitl.thumb_tmp)
 #endif
 
 #define jit_prepare_i(i0) \
-    (assert((i0) >= 0 && !_jitl.stack_offset && !_jitl.nextarg_put), \
+    (assert((i0) >= 0 && !_jitl.nextarg_put), \
      _jitl.stack_offset = i0 << 2)
 
 #define jit_arg_c()			jit_arg_p()
@@ -2083,81 +2067,53 @@ next: \
 #define jit_getarg_p(r0, i0) \
     jit_ldxi_i(r0, JIT_FP, (((i0) < 4) ? (i0) << 2 : (i0)));  /* arguments are saved in prolog */
 
-#ifdef USE_THUMB_CODE
 #define jit_pusharg_i(r0) \
-{ \
-    int ofs = _jitl.nextarg_put++; \
-    assert(ofs < 256); \
-    _jitl.stack_offset -= sizeof(int); \
-    _jitl.arguments[ofs] = (int *)_jit.x.pc; \
-    _jitl.types[ofs >> 5] &= ~(1 << (ofs & 31)); \
-    /* force 32 bit instruction due to possibly needing to trasform it */ \
-	T2_STRWI(r0, JIT_SP, 0); \
-}
-#else
-#define jit_pusharg_i(r0) \
-{ \
-    int		ofs = _jitl.nextarg_put++; \
-    assert(ofs < 256); \
-    _jitl.stack_offset -= sizeof(int); \
-    _jitl.arguments[ofs] = (int *)_jit.x.pc; \
-    _jitl.types[ofs >> 5] &= ~(1 << (ofs & 31)); \
-	_STRI(r0, JIT_SP, 0); \
-}
-#endif
+    (jit_pushr_i(r0), _jitl.nextarg_put++)
 
 #ifdef USE_THUMB_CODE
 #define jit_finishr(rs) \
 { \
-    assert(!_jitl.stack_offset); \
     if (r0 < 4) { \
 		jit_movr_i(JIT_TMP, r0); \
 		r0 = JIT_TMP; \
     } \
-    arm_patch_arguments(&_jitl); \
-    _jitl.nextarg_put = 0; \
-    if (_jitl.reglist) { \
-		T2_LDMIA(JIT_FP, _jitl.reglist); \
-		_jitl.reglist = 0; \
+    if (_jitl.nextarg_put) { \
+		T2_POP(((1 << _jitl.nextarg_put) - 1) & 0xf); \
+		_jitl.nextarg_put = 0; \
     } \
-    jit_callr(r0); \
+    T1_BLX(r0); \
 }
 #else
 #define jit_finishr(rs) \
 { \
-    assert(!_jitl.stack_offset); \
     if (r0 < 4) { \
 		jit_movr_i(JIT_TMP, r0); \
 		r0 = JIT_TMP; \
     } \
-    arm_patch_arguments(&_jitl); \
-    _jitl.nextarg_put = 0; \
-    if (_jitl.reglist) { \
-		_LDMIA(JIT_FP, _jitl.reglist); \
-		_jitl.reglist = 0; \
+    if (_jitl.nextarg_put) { \
+		_POP(((1 << _jitl.nextarg_put) - 1) & 0xf); \
+		_jitl.nextarg_put = 0; \
     } \
-    jit_callr(r0); \
+    _BLX(r0); \
 }
 #endif
 
 #ifdef USE_THUMB_CODE
 #define jit_finish(i0) \
-	(assert(!_jitl.stack_offset), \
-	 arm_patch_arguments(&_jitl), \
-	 _jitl.nextarg_put = 0, \
-	 (_jitl.reglist) ? \
-		(T2_LDMIA(JIT_FP, _jitl.reglist), \
-		 _jitl.reglist = 0) : 0, \
-     jit_calli(i0))
+	((_jitl.nextarg_put) ? \
+		(T2_POP(((1 << _jitl.nextarg_put) - 1) & 0xf), \
+		 _jitl.nextarg_put = 0) : 0, \
+	 _jitl.thumb_tmp = jit_movi_p(JIT_TMP, i0), \
+	 T1_BLX(JIT_TMP), \
+	 _jitl.thumb_tmp)
 #else
 #define jit_finish(i0) \
-	(assert(!_jitl.stack_offset), \
-	 arm_patch_arguments(&_jitl), \
-	 _jitl.nextarg_put = 0, \
-	 (_jitl.reglist) ? \
-		(_LDMIA(JIT_FP, _jitl.reglist), \
-		 _jitl.reglist = 0) : 0, \
-     jit_calli(i0))
+	((_jitl.nextarg_put) ? \
+		(_POP(((1 << _jitl.nextarg_put) - 1) & 0xf), \
+		 _jitl.nextarg_put = 0) : 0, \
+	 _jitl.thumb_tmp = jit_movi_p(JIT_TMP, i0), \
+	 _BLX(JIT_TMP), \
+	 _jitl.thumb_tmp)
 #endif
 
 #define jit_retval_i(r0)		jit_movr_i(r0, JIT_RET)
@@ -2190,16 +2146,18 @@ next: \
 }
 #endif
 
-/* just to pass make check... */
-#ifdef JIT_NEED_PUSH_POP
-# define jit_pushr_i(r0) \
-    (assert(_jitl.pop < sizeof(_jitl.push) / sizeof(_jitl.push[0])), \
-     _jitl.push[_jitl.pop++] = jit_allocai(4), \
-     jit_stxi_i(_jitl.push[_jitl.pop], JIT_FP, r0))
-
-# define jit_popr_i(r0) \
-	(assert(_jitl.pop > 0), \
-	 _jitl.pop--, \
-	 jit_ldxi_i(r0, JIT_FP, _jitl.push[_jitl.pop]))
+#ifdef USE_THUMB_CODE
+#define jit_bare_ret() T2_POP(1<<JIT_PC)
+#else
+#define jit_bare_ret() _POP(1<<JIT_PC)
 #endif
+
+#ifdef USE_THUMB_CODE
+# define jit_pushr_i(r0) T2_PUSH((1<<r0))
+# define jit_popr_i(r0)  T2_POP((1<<r0))
+#else
+# define jit_pushr_i(r0) _PUSH((1<<r0))
+# define jit_popr_i(r0)  _POP((1<<r0))
+#endif
+
 #endif /* __lightning_core_arm_h */
