@@ -7,19 +7,14 @@
 #include <gu/file.h>
 #include <gu/string.h>
 #include <gu/assert.h>
+#include <gu/utf8.h>
+#include <gu/ucs.h>
 #include <pgf/expr.h>
 #include <pgf/literals.h>
 #include <stdio.h>
 
 //#define PGF_LINEARIZER_DEBUG
 
-GU_DEFINE_TYPE(PgfCncOverloadMap, GuMap,
-               gu_type(PgfCCat), NULL,
-               gu_ptr_type(GuBuf), &gu_null_struct);
-
-GU_DEFINE_TYPE(PgfCncFunOverloadMap, GuStringMap, gu_ptr_type(PgfCncOverloadMap),
-               &gu_null_struct);
-
 static void
 pgf_lzr_add_overl_entry(PgfCncOverloadMap* overl_table,
 			            PgfCCat* ccat, void* entry,
@@ -35,9 +30,10 @@ pgf_lzr_add_overl_entry(PgfCncOverloadMap* overl_table,
 	gu_buf_push(entries, void*, entry);
 }
 
-void
+PGF_INTERNAL void
 pgf_lzr_index(PgfConcr* concr, 
               PgfCCat* ccat, PgfProduction prod,
+              bool is_lexical,
               GuPool *pool)
 {
 	void* data = gu_variant_data(prod);
@@ -48,7 +44,7 @@ pgf_lzr_index(PgfConcr* concr,
 			gu_map_get(concr->fun_indices, papply->fun->absfun->name,
 				PgfCncOverloadMap*);
 		if (!overl_table) {
-			overl_table = gu_map_type_new(PgfCncOverloadMap, pool);
+			overl_table = gu_new_addr_map(PgfCCat*, GuBuf*, &gu_null_struct, pool);
 			gu_map_put(concr->fun_indices,
 				papply->fun->absfun->name, PgfCncOverloadMap*, overl_table);
 		}
@@ -74,46 +70,8 @@ typedef struct {
 } PgfCnc;
 
 
-//
-// PgfCncTree
-//
-
-typedef enum {
-	PGF_CNC_TREE_APP,
-	PGF_CNC_TREE_CHUNKS,
-	PGF_CNC_TREE_LIT,
-} PgfCncTreeTag;
-
-typedef struct {
-	PgfCCat* ccat;
-	PgfCncFun* fun;
-	int fid;
-
-	size_t n_vars;
-	PgfPrintContext* context;
-
-	size_t n_args;
-	PgfCncTree args[];
-} PgfCncTreeApp;
-
-typedef struct {
-	size_t n_vars;
-	PgfPrintContext* context;
-
-	GuLength n_args;
-	PgfCncTree args[];
-} PgfCncTreeChunks;
-
-typedef struct {
-	size_t n_vars;
-	PgfPrintContext* context;
-
-	int fid;
-	PgfLiteral lit;
-} PgfCncTreeLit;
-
 #ifdef PGF_LINEARIZER_DEBUG
-void
+static void
 pgf_print_cnc_tree_vars(size_t n_vars, PgfPrintContext* context,
                         GuOut* out, GuExn* err)
 {
@@ -132,7 +90,7 @@ pgf_print_cnc_tree_vars(size_t n_vars, PgfPrintContext* context,
 	}
 }
 
-void
+PGF_INTERNAL void
 pgf_print_cnc_tree(PgfCncTree ctree, GuOut* out, GuExn* err)
 {
 	GuVariantInfo ti = gu_variant_open(ctree);
@@ -153,7 +111,10 @@ pgf_print_cnc_tree(PgfCncTree ctree, GuOut* out, GuExn* err)
 		PgfCncTreeChunks* chunks = ti.data;
 		if (chunks->n_vars+chunks->n_args > 0) gu_putc('(', out, err);
 		pgf_print_cnc_tree_vars(chunks->n_vars, chunks->context, out, err);
-		gu_putc('?', out, err);
+		if (chunks->id > 0)
+			gu_printf(out, err, "?%d", chunks->id);
+		else
+			gu_putc('?', out, err);
 		for (size_t i = 0; i < chunks->n_args; i++) {
 			gu_putc(' ', out, err);
 			pgf_print_cnc_tree(chunks->args[i], out, err);
@@ -325,7 +286,7 @@ pgf_cnc_cat_resolve_itor(GuMapItor* fn, const void* key, void* value, GuExn* err
 	clo->index--;
 }
 
-PgfCncTree
+PGF_API PgfCncTree
 pgf_lzr_wrap_linref(PgfCncTree ctree, GuPool* pool)
 {
 	GuVariantInfo cti = gu_variant_open(ctree);
@@ -403,6 +364,7 @@ pgf_cnc_resolve(PgfCnc* cnc,
 			goto done;
 		}
 		case PGF_EXPR_META: {
+			PgfExprMeta* emeta = i.data;
 			size_t n_args = gu_buf_length(args);
 
 			PgfCncTree chunks_tree;
@@ -410,9 +372,10 @@ pgf_cnc_resolve(PgfCnc* cnc,
 				gu_new_flex_variant(PGF_CNC_TREE_CHUNKS,
 				                    PgfCncTreeChunks,
 				                    args, n_args, &chunks_tree, pool);
+			chunks->id      = emeta->id;
 			chunks->n_vars  = n_vars;
 			chunks->context = context;
-			chunks->n_args = n_args;
+			chunks->n_args  = n_args;
 
 			for (size_t i = 0; i < n_args; i++) {
 				PgfExpr earg = gu_buf_get(args, PgfExpr, n_args-i-1);
@@ -463,8 +426,8 @@ pgf_cnc_resolve(PgfCnc* cnc,
 				}
 
 				GuPool* tmp_pool = gu_local_pool();
-				GuExn* err = gu_new_exn(NULL, gu_kind(type), tmp_pool);
-				GuStringBuf* sbuf = gu_string_buf(tmp_pool);
+				GuExn* err = gu_new_exn(tmp_pool);
+				GuStringBuf* sbuf = gu_new_string_buf(tmp_pool);
 				GuOut* out = gu_string_buf_out(sbuf);
 
 				gu_putc('[', out, err);
@@ -600,7 +563,7 @@ pgf_cnc_tree_enum_next(GuEnum* self, void* to, GuPool* pool)
 #ifdef PGF_LINEARIZER_DEBUG
     GuPool* tmp_pool = gu_new_pool();
     GuOut* out = gu_file_out(stderr, tmp_pool);
-    GuExn* err = gu_exn(NULL, type, tmp_pool);
+    GuExn* err = gu_exn(tmp_pool);
     if (gu_variant_is_null(*toc))
 		gu_puts("*nil*\n", out, err);
 	else {
@@ -615,7 +578,7 @@ pgf_cnc_tree_enum_next(GuEnum* self, void* to, GuPool* pool)
 	}
 }
 
-PgfCncTreeEnum*
+PGF_API PgfCncTreeEnum*
 pgf_lzr_concretize(PgfConcr* concr, PgfExpr expr, GuExn* err, GuPool* pool)
 {
 	if (concr->fun_indices == NULL ||
@@ -656,6 +619,8 @@ typedef enum {
 	PGF_CACHED_BEGIN,
 	PGF_CACHED_END,
 	PGF_CACHED_BIND,
+	PGF_CACHED_CAPIT,
+	PGF_CACHED_ALL_CAPIT,
 	PGF_CACHED_NE
 } PgfLzrCachedTag;
 
@@ -720,6 +685,16 @@ pgf_lzr_cache_flush(PgfLzrCache* cache, PgfSymbols* form)
 				(*cache->lzr->funcs)->symbol_bind(cache->lzr->funcs);
 			}
 			break;
+		case PGF_CACHED_CAPIT:
+			if ((*cache->lzr->funcs)->symbol_capit) {
+				(*cache->lzr->funcs)->symbol_capit(cache->lzr->funcs, PGF_CAPIT_FIRST);
+			}
+			break;
+		case PGF_CACHED_ALL_CAPIT:
+			if ((*cache->lzr->funcs)->symbol_capit) {
+				(*cache->lzr->funcs)->symbol_capit(cache->lzr->funcs, PGF_CAPIT_ALL);
+			}
+			break;
 		case PGF_CACHED_NE:
 			if ((*cache->lzr->funcs)->symbol_ne) {
 				(*cache->lzr->funcs)->symbol_ne(cache->lzr->funcs);
@@ -795,12 +770,33 @@ pgf_lzr_cache_symbol_bind(PgfLinFuncs** funcs)
 	event->tag     = PGF_CACHED_BIND;
 }
 
+static void
+pgf_lzr_cache_symbol_capit(PgfLinFuncs** funcs, PgfCapitState capit)
+{
+	PgfLzrCache*  cache = gu_container(funcs, PgfLzrCache, funcs);
+	PgfLzrCached* event = gu_buf_extend(cache->events);
+	event->tag     = (capit == PGF_CAPIT_ALL) ? PGF_CACHED_ALL_CAPIT : PGF_CACHED_CAPIT;
+}
+
+static void
+pgf_lzr_cache_symbol_meta(PgfLinFuncs** funcs, PgfMetaId id)
+{
+	PgfLzrCache* cache = gu_container(funcs, PgfLzrCache, funcs);
+
+	pgf_lzr_cache_flush(cache, cache->kp->default_form);
+	if ((*cache->lzr->funcs)->symbol_meta) {
+		(*cache->lzr->funcs)->symbol_meta(cache->lzr->funcs, id);
+	}
+}
+
 static PgfLinFuncs pgf_lzr_cache_funcs = {
 	.symbol_token = pgf_lzr_cache_symbol_token,
 	.begin_phrase = pgf_lzr_cache_begin_phrase,
 	.end_phrase   = pgf_lzr_cache_end_phrase,
 	.symbol_ne    = pgf_lzr_cache_symbol_ne,
-	.symbol_bind  = pgf_lzr_cache_symbol_bind
+	.symbol_bind  = pgf_lzr_cache_symbol_bind,
+	.symbol_capit = pgf_lzr_cache_symbol_capit,
+	.symbol_meta  = pgf_lzr_cache_symbol_meta
 };
 
 static void
@@ -911,6 +907,20 @@ pgf_lzr_linearize_symbols(PgfLzr* lzr, PgfCncTreeApp* fapp,
 			}
 			break;
 		}
+		case PGF_SYMBOL_SOFT_SPACE: {
+			// SOFT_SPACE should be just ignored in linearization
+			break;
+		}
+		case PGF_SYMBOL_CAPIT:
+			if ((*lzr->funcs)->symbol_capit) {
+				(*lzr->funcs)->symbol_capit(lzr->funcs, PGF_CAPIT_FIRST);
+			}
+			break;
+		case PGF_SYMBOL_ALL_CAPIT:
+			if ((*lzr->funcs)->symbol_capit) {
+				(*lzr->funcs)->symbol_capit(lzr->funcs, PGF_CAPIT_ALL);
+			}
+			break;
 		default:
 			gu_impossible(); 
 		}
@@ -950,8 +960,8 @@ pgf_lzr_linearize_tree(PgfLzr* lzr, PgfCncTree ctree, size_t lin_idx)
 		PgfCncTreeChunks* fchunks = cti.data;
 
 		if (fchunks->n_args == 0) {
-			if ((*lzr->funcs)->symbol_token) {
-				(*lzr->funcs)->symbol_token(lzr->funcs, "?");
+			if ((*lzr->funcs)->symbol_meta) {
+				(*lzr->funcs)->symbol_meta(lzr->funcs, fchunks->id);
 			}
 		} else {
 			for (size_t i = 0; i < fchunks->n_args; i++) {
@@ -1012,7 +1022,7 @@ pgf_lzr_linearize_tree(PgfLzr* lzr, PgfCncTree ctree, size_t lin_idx)
 	}
 }
 
-void
+PGF_API void
 pgf_lzr_linearize(PgfConcr* concr, PgfCncTree ctree, size_t lin_idx, 
                   PgfLinFuncs** funcs, GuPool* tmp_pool)
 {
@@ -1030,21 +1040,11 @@ typedef struct PgfSimpleLin PgfSimpleLin;
 struct PgfSimpleLin {
 	PgfLinFuncs* funcs;
 	bool bind;
+	PgfCapitState capit;
 	GuOut* out;
 	GuExn* err;
 };
 
-GU_DEFINE_TYPE(PgfLinNonExist, abstract, _);
-
-static void
-pgf_file_lzn_put_space(PgfSimpleLin* flin)
-{
-	if (flin->bind)
-		flin->bind = false;
-	else
-		gu_putc(' ', flin->out, flin->err);
-}
-
 static void
 pgf_file_lzn_symbol_token(PgfLinFuncs** funcs, PgfToken tok)
 {
@@ -1053,8 +1053,39 @@ pgf_file_lzn_symbol_token(PgfLinFuncs** funcs, PgfToken tok)
 		return;
 	}
 
-	pgf_file_lzn_put_space(flin);
-	gu_string_write(tok, flin->out, flin->err);
+	if (flin->bind)
+		flin->bind = false;
+	else {
+		gu_putc(' ', flin->out, flin->err);
+		if (flin->capit == PGF_CAPIT_NEXT)
+			flin->capit = PGF_CAPIT_NONE;
+	}
+
+	switch (flin->capit) {
+	case PGF_CAPIT_NONE:
+		gu_string_write(tok, flin->out, flin->err);
+		break;
+	case PGF_CAPIT_FIRST: {
+		GuUCS c = gu_utf8_decode((const uint8_t**) &tok);
+		c = gu_ucs_to_upper(c);
+		gu_out_utf8(c, flin->out, flin->err);
+		gu_string_write(tok, flin->out, flin->err);
+		flin->capit = PGF_CAPIT_NONE;
+		break;
+	}
+	case PGF_CAPIT_ALL:
+		flin->capit = PGF_CAPIT_NEXT;
+		// continue
+	case PGF_CAPIT_NEXT: {
+		const uint8_t* p = (uint8_t*) tok;
+		while (*p) {
+			GuUCS c = gu_utf8_decode(&p);
+			c = gu_ucs_to_upper(c);
+			gu_out_utf8(c, flin->out, flin->err);
+		}
+		break;
+	}
+	}
 }
 
 static void
@@ -1071,15 +1102,43 @@ pgf_file_lzn_symbol_bind(PgfLinFuncs** funcs)
 	flin->bind = true;
 }
 
+static void
+pgf_file_lzn_symbol_capit(PgfLinFuncs** funcs, PgfCapitState capit)
+{
+	PgfSimpleLin* flin = gu_container(funcs, PgfSimpleLin, funcs);
+	flin->capit = capit;
+}
+
+static void
+pgf_file_lzn_symbol_meta(PgfLinFuncs** funcs, PgfMetaId id)
+{
+	PgfSimpleLin* flin = gu_container(funcs, PgfSimpleLin, funcs);
+	if (!gu_ok(flin->err)) {
+		return;
+	}
+
+	if (flin->bind)
+		flin->bind = false;
+	else {
+		gu_putc(' ', flin->out, flin->err);
+		if (flin->capit == PGF_CAPIT_NEXT)
+			flin->capit = PGF_CAPIT_NONE;
+	}
+
+	gu_putc('?', flin->out, flin->err);		
+}
+
 static PgfLinFuncs pgf_file_lin_funcs = {
 	.symbol_token = pgf_file_lzn_symbol_token,
 	.begin_phrase = NULL,
 	.end_phrase   = NULL,
 	.symbol_ne    = pgf_file_lzn_symbol_ne,
-	.symbol_bind  = pgf_file_lzn_symbol_bind
+	.symbol_bind  = pgf_file_lzn_symbol_bind,
+	.symbol_capit = pgf_file_lzn_symbol_capit,
+	.symbol_meta  = pgf_file_lzn_symbol_meta
 };
 
-void
+PGF_API void
 pgf_lzr_linearize_simple(PgfConcr* concr, PgfCncTree ctree, size_t lin_idx,
                          GuOut* out, GuExn* err,
                          GuPool* tmp_pool)
@@ -1087,13 +1146,14 @@ pgf_lzr_linearize_simple(PgfConcr* concr, PgfCncTree ctree, size_t lin_idx,
 	PgfSimpleLin flin = {
 		.funcs = &pgf_file_lin_funcs,
 		.bind = true,
+		.capit = PGF_CAPIT_NONE,
 		.out = out,
 		.err = err
 	};
 	pgf_lzr_linearize(concr, ctree, lin_idx, &flin.funcs, tmp_pool);
 }
 
-void
+PGF_API void
 pgf_lzr_get_table(PgfConcr* concr, PgfCncTree ctree, 
                   size_t* n_lins, GuString** labels)
 {
@@ -1122,15 +1182,17 @@ pgf_lzr_get_table(PgfConcr* concr, PgfCncTree ctree,
 
 }
 
-void
+PGF_API void
 pgf_linearize(PgfConcr* concr, PgfExpr expr, GuOut* out, GuExn* err)
 {
 	GuPool* tmp_pool = gu_local_pool();
 
 	GuEnum* cts =
 		pgf_lzr_concretize(concr, expr, err, tmp_pool);
-	if (!gu_ok(err))
+	if (!gu_ok(err)) {
+		gu_pool_free(tmp_pool);
 		return;
+	}
 
 	PgfCncTree ctree = gu_next(cts, PgfCncTree, tmp_pool);
 	if (!gu_variant_is_null(ctree)) {
@@ -1141,17 +1203,18 @@ pgf_linearize(PgfConcr* concr, PgfExpr expr, GuOut* out, GuExn* err)
 	gu_pool_free(tmp_pool);
 }
 
-GuString
+PGF_INTERNAL GuString
 pgf_get_tokens(PgfSymbols* syms, uint16_t sym_idx, GuPool* pool)
 {
 	GuPool* tmp_pool = gu_new_pool();
-	GuExn* err = gu_new_exn(NULL, gu_kind(type), tmp_pool);
-	GuStringBuf* sbuf = gu_string_buf(tmp_pool);
+	GuExn* err = gu_new_exn(tmp_pool);
+	GuStringBuf* sbuf = gu_new_string_buf(tmp_pool);
 	GuOut* out = gu_string_buf_out(sbuf);
 
 	PgfSimpleLin flin = {
 		.funcs = &pgf_file_lin_funcs,
 		.bind = true,
+		.capit = false,
 		.out = out,
 		.err = err
 	};
