@@ -1,8 +1,7 @@
 #include "data.h"
 #include "expr.h"
-#include "literals.h"
+#include "reasoner.h"
 #include "reader.h"
-#include "jit.h"
 
 #include <gu/defs.h>
 #include <gu/map.h>
@@ -13,8 +12,9 @@
 #include <gu/exn.h>
 #include <gu/utf8.h>
 #include <math.h>
+#include <stdio.h>
 #include <stdlib.h>
-#ifdef __MINGW32__
+#if defined(__MINGW32__) || defined(_MSC_VER)
 #include <malloc.h>
 #endif
 
@@ -22,32 +22,19 @@
 // PgfReader
 // 
 
-struct PgfReader {
-	GuIn* in;
-	GuExn* err;
-	GuPool* opool;
-	GuPool* tmp_pool;
-	PgfJitState* jit_state;
-};
-
 typedef struct PgfReadTagExn PgfReadTagExn;
 
 struct PgfReadTagExn {
-	GuType* type;
 	int tag;
 };
 
-static GU_DEFINE_TYPE(PgfReadTagExn, abstract, _);
-
-static GU_DEFINE_TYPE(PgfReadExn, abstract, _);
-
-static uint8_t
+PGF_INTERNAL uint8_t
 pgf_read_tag(PgfReader* rdr)
 {
 	return gu_in_u8(rdr->in, rdr->err);
 }
 
-static uint32_t
+PGF_INTERNAL uint32_t
 pgf_read_uint(PgfReader* rdr)
 {
 	uint32_t u = 0;
@@ -62,14 +49,14 @@ pgf_read_uint(PgfReader* rdr)
 	return u;
 }
 
-static int32_t
+PGF_INTERNAL int32_t
 pgf_read_int(PgfReader* rdr)
 {
 	uint32_t u = pgf_read_uint(rdr);
 	return gu_decode_2c32(u, rdr->err);
 }
 
-static GuLength
+PGF_INTERNAL size_t
 pgf_read_len(PgfReader* rdr)
 {
 	int32_t len = pgf_read_int(rdr);
@@ -81,30 +68,35 @@ pgf_read_len(PgfReader* rdr)
 		GuExnData* err_data = gu_raise(rdr->err, PgfReadTagExn);
 		if (err_data) {
 			PgfReadTagExn* rtag = gu_new(PgfReadTagExn, err_data->pool);
-			rtag->type = gu_type(GuLength);
 			rtag->tag  = len;
 			err_data->data = rtag;
 		}
 
 		return 0;
 	}
-	return (GuLength) len;
+	return len;
 }
 
-static PgfCId
+PGF_INTERNAL PgfCId
 pgf_read_cid(PgfReader* rdr, GuPool* pool)
 {
 	size_t len = pgf_read_len(rdr);
 	return gu_string_read_latin1(len, pool, rdr->in, rdr->err);
 }
 
-static GuString
+PGF_INTERNAL GuString
 pgf_read_string(PgfReader* rdr)
 {	
-	GuLength len = pgf_read_len(rdr);
+	size_t len = pgf_read_len(rdr);
 	return gu_string_read(len, rdr->opool, rdr->in, rdr->err);
 }
 
+PGF_INTERNAL double
+pgf_read_double(PgfReader* rdr)
+{
+	return gu_in_f64be(rdr->in, rdr->err);
+}
+
 static void
 pgf_read_tag_error(PgfReader* rdr)
 {
@@ -119,7 +111,7 @@ pgf_read_literal(PgfReader* rdr)
 	uint8_t tag = pgf_read_tag(rdr);
 	switch (tag) {
 	case PGF_LITERAL_STR: {
-		GuLength len = pgf_read_len(rdr);
+		size_t len = pgf_read_len(rdr);
 		uint8_t* buf = alloca(len*6+1);
 		uint8_t* p   = buf;
 		for (size_t i = 0; i < len; i++) {
@@ -149,7 +141,7 @@ pgf_read_literal(PgfReader* rdr)
 			gu_new_variant(PGF_LITERAL_FLT,
 						   PgfLiteralFlt,
 						   &lit, rdr->opool);
-		lit_flt->val = gu_in_f64be(rdr->in, rdr->err);
+		lit_flt->val = pgf_read_double(rdr);
 		break;
 	}
 	default:
@@ -161,19 +153,18 @@ pgf_read_literal(PgfReader* rdr)
 static PgfFlags*
 pgf_read_flags(PgfReader* rdr)
 {
-	PgfFlags* flags = gu_map_type_new(PgfFlags, rdr->opool);
-
-	GuLength len = pgf_read_len(rdr);
+	size_t n_flags = pgf_read_len(rdr);
 	gu_return_on_exn(rdr->err, NULL);
 
-	for (size_t i = 0; i < len; i++) {
-		PgfCId name = pgf_read_cid(rdr, rdr->opool);
+	PgfFlags* flags = gu_new_seq(PgfFlag, n_flags, rdr->opool);
+	for (size_t i = 0; i < n_flags; i++) {
+		PgfFlag* flag = gu_seq_index(flags, PgfFlag, i);
+
+		flag->name = pgf_read_cid(rdr, rdr->opool);
 		gu_return_on_exn(rdr->err, NULL);
 
-		PgfLiteral value = pgf_read_literal(rdr);
+		flag->value = pgf_read_literal(rdr);
 		gu_return_on_exn(rdr->err, NULL);
-		
-		gu_map_put(flags, name, PgfLiteral, value);
 	}
 
 	return flags;
@@ -417,10 +408,8 @@ pgf_read_patt(PgfReader* rdr)
 }
 
 static PgfAbsFun*
-pgf_read_absfun(PgfReader* rdr)
+pgf_read_absfun(PgfReader* rdr, PgfAbstr* abstr, PgfAbsFun* absfun)
 {
-	PgfAbsFun* absfun = gu_new(PgfAbsFun, rdr->opool);
-
 	size_t len = pgf_read_len(rdr);
 
 	PgfExprFun *efun =
@@ -444,22 +433,28 @@ pgf_read_absfun(PgfReader* rdr)
 	switch (tag) {
 	case 0:
 		absfun->defns = NULL;
+		if (absfun->arity == 0) {
+			absfun->closure.code = abstr->eval_gates->evaluate_value;
+			absfun->closure.con  = &absfun->closure.code;
+		} else {
+			absfun->closure.code = NULL;
+		}
 		break;
 	case 1: {
-        GuLength length = pgf_read_len(rdr);
+        size_t length = pgf_read_len(rdr);
         gu_return_on_exn(rdr->err, NULL);
 
         absfun->defns = gu_new_seq(PgfEquation*, length, rdr->opool);
         PgfEquation** data = gu_seq_data(absfun->defns);
         for (size_t i = 0; i < length; i++) {
-            GuLength n_patts = pgf_read_len(rdr);
+            size_t n_patts = pgf_read_len(rdr);
             gu_return_on_exn(rdr->err, NULL);
 
             PgfEquation *equ =
                 gu_malloc(rdr->opool, 
                           sizeof(PgfEquation)+sizeof(PgfPatt)*n_patts);
             equ->n_patts = n_patts;
-            for (GuLength j = 0; j < n_patts; j++) {
+            for (size_t j = 0; j < n_patts; j++) {
                 equ->patts[j] = pgf_read_patt(rdr);
                 gu_return_on_exn(rdr->err, NULL);
             }
@@ -468,6 +463,8 @@ pgf_read_absfun(PgfReader* rdr)
 
             data[i] = equ;
         }
+        
+       	// pgf_jit_function(rdr, abstr, absfun);
 		break;
     }
 	default:
@@ -475,38 +472,31 @@ pgf_read_absfun(PgfReader* rdr)
 		break;
 	}
 
-	absfun->ep.prob = - log(gu_in_f64be(rdr->in, rdr->err));
+	absfun->ep.prob = - log(pgf_read_double(rdr));
 
 	return absfun;
 }
 					    
-static PgfCIdMap*
-pgf_read_absfuns(PgfReader* rdr)
+static PgfAbsFuns*
+pgf_read_absfuns(PgfReader* rdr, PgfAbstr* abstr)
 {
-	GuMapType* map_type = (GuMapType*)
-		GU_TYPE_LIT(GuStringMap, _,
-			 		gu_ptr_type(PgfAbsFun),
-					&gu_null_struct);
-	PgfCIdMap* absfuns = gu_map_type_make(map_type, rdr->opool);
-
-	size_t len = pgf_read_len(rdr);
+	size_t n_funs = pgf_read_len(rdr);
 	gu_return_on_exn(rdr->err, NULL);
 
-	for (size_t i = 0; i < len; i++) {
-		PgfAbsFun* absfun = pgf_read_absfun(rdr);
+	PgfAbsFuns* absfuns = gu_new_seq(PgfAbsFun, n_funs, rdr->opool);
+
+	for (size_t i = 0; i < n_funs; i++) {
+		PgfAbsFun* absfun = gu_seq_index(absfuns, PgfAbsFun, i);
+		pgf_read_absfun(rdr, abstr, absfun);
 		gu_return_on_exn(rdr->err, NULL);
-		
-		gu_map_put(absfuns, absfun->name, PgfAbsFun*, absfun);
 	}
 
 	return absfuns;
 }
 
 static PgfAbsCat*
-pgf_read_abscat(PgfReader* rdr, PgfAbstr* abstr, PgfCIdMap* abscats)
+pgf_read_abscat(PgfReader* rdr, PgfAbstr* abstr, PgfAbsCat* abscat)
 {
-	PgfAbsCat* abscat = gu_new(PgfAbsCat, rdr->opool);
-
 	abscat->name = pgf_read_cid(rdr, rdr->opool);
 	gu_return_on_exn(rdr->err, NULL);
 
@@ -519,48 +509,25 @@ pgf_read_abscat(PgfReader* rdr, PgfAbstr* abstr, PgfCIdMap* abscats)
 		gu_return_on_exn(rdr->err, NULL);
 	}
 
-    GuBuf* functions = gu_new_buf(PgfAbsFun*, rdr->tmp_pool);
+	pgf_jit_predicate(rdr, abstr, abscat);
 
-	size_t n_functions = pgf_read_len(rdr);
-	gu_return_on_exn(rdr->err, NULL);
-
-	for (size_t i = 0; i < n_functions; i++) {
-		gu_in_f64be(rdr->in, rdr->err);  // ignore
-		gu_return_on_exn(rdr->err, NULL);
-
-		PgfCId name = pgf_read_cid(rdr, rdr->tmp_pool);
-		gu_return_on_exn(rdr->err, NULL);
-		
-		PgfAbsFun* absfun =
-			gu_map_get(abstr->funs, name, PgfAbsFun*);
-		assert(absfun != NULL);
-		gu_buf_push(functions, PgfAbsFun*, absfun);
-	}
-
-	abscat->prob = - log(gu_in_f64be(rdr->in, rdr->err));
-
-	pgf_jit_predicate(rdr->jit_state, abscats, abscat, functions);
+	abscat->prob = - log(pgf_read_double(rdr));
 
 	return abscat;
 }
 
-static PgfCIdMap*
+static PgfAbsCats*
 pgf_read_abscats(PgfReader* rdr, PgfAbstr* abstr)
 {
-	GuMapType* map_type = (GuMapType*)
-		GU_TYPE_LIT(GuStringMap, _,
-			 		gu_ptr_type(PgfAbsCat),
-					&gu_null_struct);
-	PgfCIdMap* abscats = gu_map_type_make(map_type, rdr->opool);
-	
-	size_t len = pgf_read_len(rdr);
+	size_t n_cats = pgf_read_len(rdr);
 	gu_return_on_exn(rdr->err, NULL);
+	
+	PgfAbsCats* abscats = gu_new_seq(PgfAbsCat, n_cats, rdr->opool);
 
-	for (size_t i = 0; i < len; i++) {
-		PgfAbsCat* abscat = pgf_read_abscat(rdr, abstr, abscats);
+	for (size_t i = 0; i < n_cats; i++) {
+		PgfAbsCat* abscat = gu_seq_index(abscats, PgfAbsCat, i);
+		pgf_read_abscat(rdr, abstr, abscat);
 		gu_return_on_exn(rdr->err, NULL);
-
-		gu_map_put(abscats, abscat->name, PgfAbsCat*, abscat);
 	}
 
 	return abscats;
@@ -574,8 +541,10 @@ pgf_read_abstract(PgfReader* rdr, PgfAbstr* abstract)
 
 	abstract->aflags = pgf_read_flags(rdr);
 	gu_return_on_exn(rdr->err, );
-	
-	abstract->funs = pgf_read_absfuns(rdr);
+
+	abstract->eval_gates = pgf_jit_gates(rdr);
+		
+	abstract->funs = pgf_read_absfuns(rdr, abstract);
 	gu_return_on_exn(rdr->err, );
 	
 	abstract->cats = pgf_read_abscats(rdr, abstract);
@@ -596,11 +565,7 @@ pgf_read_abstract(PgfReader* rdr, PgfAbstr* abstract)
 static PgfCIdMap*
 pgf_read_printnames(PgfReader* rdr)
 {
-	GuMapType* map_type = (GuMapType*)
-		GU_TYPE_LIT(GuStringMap, _,
-			 		gu_type(GuString),
-					&"");
-	PgfCIdMap* printnames = gu_map_type_make(map_type, rdr->opool);
+	PgfCIdMap* printnames = gu_new_string_map(GuString, &gu_null_struct, rdr->opool);
 
 	size_t len = pgf_read_len(rdr);
 	gu_return_on_exn(rdr->err, NULL);
@@ -686,7 +651,7 @@ pgf_read_symbol(PgfReader* rdr)
 		break;
 	}
 	case PGF_SYMBOL_KS: {
-		GuLength len = pgf_read_len(rdr);
+		size_t len = pgf_read_len(rdr);
 		uint8_t* buf = alloca(len*6+1);
 		uint8_t* p   = buf;
 		for (size_t i = 0; i < len; i++) {
@@ -744,6 +709,27 @@ pgf_read_symbol(PgfReader* rdr)
 		gu_return_on_exn(rdr->err, gu_null_variant);
 		break;
 	}
+	case PGF_SYMBOL_SOFT_SPACE: {
+		gu_new_variant(PGF_SYMBOL_SOFT_SPACE,
+		               PgfSymbolBIND,
+		               &sym, rdr->opool);
+		gu_return_on_exn(rdr->err, gu_null_variant);
+		break;
+	}
+	case PGF_SYMBOL_CAPIT: {
+		gu_new_variant(PGF_SYMBOL_CAPIT,
+		               PgfSymbolCAPIT,
+		               &sym, rdr->opool);
+		gu_return_on_exn(rdr->err, gu_null_variant);
+		break;
+	}
+	case PGF_SYMBOL_ALL_CAPIT: {
+		gu_new_variant(PGF_SYMBOL_ALL_CAPIT,
+		               PgfSymbolCAPIT,
+		               &sym, rdr->opool);
+		gu_return_on_exn(rdr->err, gu_null_variant);
+		break;
+	}
 	default:
 		pgf_read_tag_error(rdr);
 	}
@@ -797,7 +783,7 @@ pgf_read_cncfun(PgfReader* rdr, PgfAbstr* abstr, PgfConcr* concr, int funid)
 	gu_return_on_exn(rdr->err, NULL);
 
 	PgfAbsFun* absfun =
-		gu_map_get(abstr->funs, name, PgfAbsFun*);
+		gu_seq_binsearch(abstr->funs, pgf_absfun_order, PgfAbsFun, name);
 
 	PgfCncFun* cncfun = gu_new_flex(rdr->opool, PgfCncFun, lins, len);
 	cncfun->absfun = absfun;
@@ -953,18 +939,56 @@ pgf_read_pargs(PgfReader* rdr, PgfConcr* concr)
 extern void
 pgf_parser_index(PgfConcr* concr, 
                  PgfCCat* ccat, PgfProduction prod,
+                 bool is_lexical,
                  GuPool *pool);
 
 extern void
 pgf_lzr_index(PgfConcr* concr, 
               PgfCCat* ccat, PgfProduction prod,
+              bool is_lexical,
               GuPool *pool);
 
+static bool
+pgf_production_is_lexical(PgfReader* rdr, PgfProductionApply *papp)
+{
+	if (gu_seq_length(papp->args) > 0)
+		return false;
+
+	for (size_t lin_idx = 0; lin_idx < papp->fun->n_lins; lin_idx++) {
+		PgfSequence* seq = papp->fun->lins[lin_idx];
+		
+		if (seq->idx == NULL) {
+			size_t n_syms = gu_seq_length(seq->syms);
+			for (size_t i = 0; i < n_syms; i++) {
+				PgfSymbol sym = gu_seq_get(seq->syms, PgfSymbol, i);
+				GuVariantInfo inf = gu_variant_open(sym);
+				if (inf.tag == PGF_SYMBOL_KP ||
+					inf.tag == PGF_SYMBOL_BIND ||
+					inf.tag == PGF_SYMBOL_NE ||
+					inf.tag == PGF_SYMBOL_SOFT_BIND ||
+					inf.tag == PGF_SYMBOL_SOFT_SPACE ||
+					inf.tag == PGF_SYMBOL_CAPIT ||
+					inf.tag == PGF_SYMBOL_ALL_CAPIT) {
+					seq->idx = rdr->non_lexical_buf;
+					return false;
+				}
+			}
+
+			seq->idx = gu_new_buf(PgfProductionIdxEntry, rdr->opool);
+		} if (seq->idx == rdr->non_lexical_buf) {
+			return false;
+		}
+	}
+
+	return true;
+}
+
 static void
 pgf_read_production(PgfReader* rdr, PgfConcr* concr, 
                     PgfCCat* ccat, size_t* top, size_t* bot)
 {
 	PgfProduction prod = gu_null_variant;
+	bool is_lexical = false;
 
 	uint8_t tag = pgf_read_tag(rdr);
 	switch (tag) {
@@ -980,7 +1004,8 @@ pgf_read_production(PgfReader* rdr, PgfConcr* concr,
 		papp->args = pgf_read_pargs(rdr, concr);
 		gu_return_on_exn(rdr->err, );
 
-		if (gu_seq_length(papp->args) > 0)
+		is_lexical = pgf_production_is_lexical(rdr, papp);
+		if (!is_lexical)
 			gu_seq_set(ccat->prods, PgfProduction, (*top)++, prod);
 		else
 			gu_seq_set(ccat->prods, PgfProduction, (*bot)--, prod);
@@ -1002,8 +1027,8 @@ pgf_read_production(PgfReader* rdr, PgfConcr* concr,
 		pgf_read_tag_error(rdr);
 	}
 
-	pgf_parser_index(concr, ccat, prod, rdr->opool);
-	pgf_lzr_index(concr, ccat, prod, rdr->opool);
+	pgf_parser_index(concr, ccat, prod, is_lexical, rdr->opool);
+	pgf_lzr_index(concr, ccat, prod, is_lexical, rdr->opool);
 }
 
 static void
@@ -1015,7 +1040,7 @@ pgf_read_ccats(PgfReader* rdr, PgfConcr* concr)
 	for (size_t i = 0; i < len; i++) {
 		PgfCCat* ccat = pgf_read_fid(rdr, concr);
 
-		GuLength n_prods = pgf_read_len(rdr);
+		size_t n_prods = pgf_read_len(rdr);
 		gu_return_on_exn(rdr->err,);
 
 		ccat->prods = gu_new_seq(PgfProduction, n_prods, rdr->opool);
@@ -1042,8 +1067,11 @@ pgf_read_cnccat(PgfReader* rdr, PgfAbstr* abstr, PgfConcr* concr, PgfCId name)
 		gu_malloc(rdr->opool, sizeof(PgfCncCat)+n_lins*sizeof(GuString));
 
 	cnccat->abscat = 
-		gu_map_get(abstr->cats, name, PgfAbsCat*);
-	gu_assert(cnccat->abscat != NULL);
+		gu_seq_binsearch(abstr->cats, pgf_abscat_order, PgfAbsCat, name);
+	if (cnccat->abscat == NULL) {
+		fprintf(stderr, "Abstract category %s is missing\n", name);
+		gu_assert(cnccat->abscat != NULL);
+	}
 
 	int len = last + 1 - first;
 	cnccat->cats = gu_new_seq(PgfCCat*, len, rdr->opool);
@@ -1081,11 +1109,7 @@ pgf_read_cnccat(PgfReader* rdr, PgfAbstr* abstr, PgfConcr* concr, PgfCId name)
 static PgfCIdMap*
 pgf_read_cnccats(PgfReader* rdr, PgfAbstr* abstr, PgfConcr* concr)
 {
-	GuMapType* map_type = (GuMapType*)
-		GU_TYPE_LIT(GuStringMap, _,
-			 		gu_ptr_type(PgfCncCat),
-					&gu_null_struct);
-	PgfCIdMap* cnccats = gu_map_type_make(map_type, rdr->opool);
+	PgfCIdMap* cnccats = gu_new_string_map(PgfCncCat, &gu_null_struct, rdr->opool);
 
 	size_t len = pgf_read_len(rdr);
 	gu_return_on_exn(rdr->err, NULL);
@@ -1155,7 +1179,7 @@ pgf_read_ccat_cb(GuMapItor* fn, const void* key, void* value, GuExn* err)
 //	pgf_ccat_set_viterbi_prob(ccat);
 }
 
-void
+static void
 pgf_read_concrete_content(PgfReader* rdr, PgfConcr* concr)
 {
 	concr->printnames =
@@ -1166,21 +1190,18 @@ pgf_read_concrete_content(PgfReader* rdr, PgfConcr* concr)
 		pgf_read_sequences(rdr);
 	gu_return_on_exn(rdr->err,);
 
-	concr->pre_sequences = gu_new_buf(PgfSequence, rdr->opool);
-
 	concr->cncfuns =
 		pgf_read_cncfuns(rdr, concr->abstr, concr);
 	gu_return_on_exn(rdr->err,);
 
 	concr->ccats =
 		gu_new_int_map(PgfCCat*, &gu_null_struct, rdr->opool);
-	concr->fun_indices = gu_map_type_new(PgfCncFunOverloadMap, rdr->opool);
-	concr->coerce_idx  = gu_map_type_new(PgfCncOverloadMap, rdr->opool);
+	concr->fun_indices = gu_new_string_map(PgfCncOverloadMap*, &gu_null_struct, rdr->opool);
+	concr->coerce_idx  = gu_new_addr_map(PgfCCat*, GuBuf*, &gu_null_struct, rdr->opool);
 	pgf_read_lindefs(rdr, concr);
 	pgf_read_linrefs(rdr, concr);
 	pgf_read_ccats(rdr, concr);
 	concr->cnccats = pgf_read_cnccats(rdr, concr->abstr, concr);
-	concr->callbacks = pgf_new_callbacks_map(concr, rdr->opool); 
 	concr->total_cats = pgf_read_int(rdr);
 
 	GuMapItor clo1 = { pgf_read_ccat_cb };
@@ -1192,13 +1213,11 @@ pgf_read_concrete_init_header(PgfConcr* concr)
 {
 	concr->printnames = NULL;
 	concr->sequences  = NULL;
-	concr->pre_sequences = NULL;
 	concr->cncfuns = NULL;
 	concr->ccats = NULL;
 	concr->fun_indices = NULL;
 	concr->coerce_idx = NULL;
 	concr->cnccats = NULL;
-	concr->callbacks = NULL;
 	concr->total_cats = 0;
 }
 
@@ -1215,10 +1234,8 @@ gu_concr_fini(GuFinalizer* fin)
 }
 
 static PgfConcr*
-pgf_read_concrete(PgfReader* rdr, PgfAbstr* abstr, bool with_content)
+pgf_read_concrete(PgfReader* rdr, PgfAbstr* abstr, PgfConcr* concr, bool with_content)
 {
-	PgfConcr* concr = gu_new(PgfConcr, rdr->opool);
-
 	concr->name = 
 		pgf_read_cid(rdr, rdr->opool);
 	gu_return_on_exn(rdr->err, NULL);
@@ -1246,7 +1263,7 @@ pgf_read_concrete(PgfReader* rdr, PgfAbstr* abstr, bool with_content)
 	return concr;
 }
 
-void
+PGF_API void
 pgf_concrete_load(PgfConcr* concr, GuIn* in, GuExn* err)
 {
 	if (concr->fin.fn == NULL || concr->pool != NULL)
@@ -1285,7 +1302,7 @@ end:
 	gu_pool_free(tmp_pool);
 }
 
-void
+PGF_API void
 pgf_concrete_unload(PgfConcr* concr)
 {
 	if (concr->fin.fn == NULL)
@@ -1294,29 +1311,24 @@ pgf_concrete_unload(PgfConcr* concr)
 	gu_concr_fini(&concr->fin);
 }
 
-static PgfCIdMap*
+static PgfConcrs*
 pgf_read_concretes(PgfReader* rdr, PgfAbstr* abstr, bool with_content)
 {
-	GuMapType* map_type = (GuMapType*)
-		GU_TYPE_LIT(GuStringMap, _,
-			 		gu_ptr_type(PgfConcr),
-					&gu_null_struct);
-	PgfCIdMap* concretes = gu_map_type_make(map_type, rdr->opool);
-
-	size_t len = pgf_read_len(rdr);
+	size_t n_concrs = pgf_read_len(rdr);
 	gu_return_on_exn(rdr->err, NULL);
 
-	for (size_t i = 0; i < len; i++) {
-		PgfConcr* concr = pgf_read_concrete(rdr, abstr, with_content);
-		gu_return_on_exn(rdr->err, NULL);
+	PgfConcrs* concretes = gu_new_seq(PgfConcr, n_concrs, rdr->opool);
 
-		gu_map_put(concretes, concr->name, PgfConcr*, concr);
+	for (size_t i = 0; i < n_concrs; i++) {
+		PgfConcr* concr = gu_seq_index(concretes, PgfConcr, i);
+		pgf_read_concrete(rdr, abstr, concr, with_content);
+		gu_return_on_exn(rdr->err, NULL);
 	}
 
 	return concretes;
 }
 
-PgfPGF*
+PGF_INTERNAL PgfPGF*
 pgf_read_pgf(PgfReader* rdr) {
 	PgfPGF* pgf = gu_new(PgfPGF, rdr->opool);
 	
@@ -1333,16 +1345,16 @@ pgf_read_pgf(PgfReader* rdr) {
 	gu_return_on_exn(rdr->err, NULL);
 
 	bool with_content =
-		gu_variant_is_null(gu_map_get(pgf->gflags, "split", PgfLiteral));
+		(gu_seq_binsearch(pgf->gflags, pgf_flag_order, PgfFlag, "split") == NULL);
 	pgf->concretes = pgf_read_concretes(rdr, &pgf->abstract, with_content);
 	gu_return_on_exn(rdr->err, NULL);
-	
+
 	pgf->pool = rdr->opool;
 
 	return pgf;
 }
 
-PgfReader*
+PGF_INTERNAL PgfReader*
 pgf_new_reader(GuIn* in, GuPool* opool, GuPool* tmp_pool, GuExn* err)
 {
 	PgfReader* rdr = gu_new(PgfReader, tmp_pool);
@@ -1350,15 +1362,16 @@ pgf_new_reader(GuIn* in, GuPool* opool, GuPool* tmp_pool, GuExn* err)
 	rdr->tmp_pool = tmp_pool;
 	rdr->err = err;
 	rdr->in = in;
-	rdr->jit_state = pgf_jit_init(tmp_pool, rdr->opool);
+	rdr->non_lexical_buf = gu_new_buf(PgfProductionIdxEntry, opool);
+	rdr->jit_state = pgf_new_jit(rdr);
 	return rdr;
 }
 
-void
+PGF_INTERNAL void
 pgf_reader_done(PgfReader* rdr, PgfPGF* pgf)
 {
 	if (pgf == NULL)
 		return;
 
-	pgf_jit_done(rdr->jit_state, &pgf->abstract);
+	pgf_jit_done(rdr, &pgf->abstract);
 }
