@@ -88,6 +88,7 @@ module PGF(
            Parse.initState, Parse.nextState, Parse.getCompletions, Parse.recoveryStates, 
            Parse.ParseInput(..),  Parse.simpleParseInput, Parse.mkParseInput,
            Parse.ParseOutput(..), Parse.getParseOutput,
+           Parse.getContinuationInfo,
 
            -- ** Generation
            -- | The PGF interpreter allows automatic generation of
@@ -118,20 +119,20 @@ module PGF(
            -- extra:
            morphoKnown, isInMorpho,
 
-           -- ** Tokenizing
-           mkTokenizer,
-
            -- ** Visualizations
            graphvizAbstractTree,
            graphvizParseTree,
+           graphvizParseTreeDep,
            graphvizDependencyTree,
            graphvizBracketedString,
            graphvizAlignment,
            gizaAlignment,
            GraphvizOptions(..),
            graphvizDefaults,
+           conlls2latexDoc,
            -- extra:
-           getDepLabels,
+           Labels, getDepLabels,
+           CncLabels, getCncDepLabels,
  
            -- * Probabilities
            Probabilities,
@@ -164,7 +165,6 @@ import PGF.Expr (Tree)
 import PGF.Morphology
 import PGF.Data
 import PGF.Binary()
-import PGF.Tokenizer
 import qualified PGF.Forest as Forest
 import qualified PGF.Parse as Parse
 import PGF.Utilities(replace)
@@ -292,8 +292,8 @@ categories pgf = [c | (c,hs) <- Map.toList (cats (abstract pgf))]
 
 categoryContext pgf cat =
   case Map.lookup cat (cats (abstract pgf)) of
-    Just (hypos,_,_,_) -> Just hypos
-    Nothing            -> Nothing
+    Just (hypos,_,_) -> Just hypos
+    Nothing          -> Nothing
 
 startCat pgf = DTyp [] (lookStartCat pgf) []
 
@@ -301,13 +301,13 @@ functions pgf = Map.keys (funs (abstract pgf))
 
 functionsByCat pgf cat =
   case Map.lookup cat (cats (abstract pgf)) of
-    Just (_,fns,_,_) -> map snd fns
-    Nothing          -> []
+    Just (_,fns,_) -> map snd fns
+    Nothing        -> []
 
 functionType pgf fun =
   case Map.lookup fun (funs (abstract pgf)) of
-    Just (ty,_,_,_,_) -> Just ty
-    Nothing           -> Nothing
+    Just (ty,_,_,_) -> Just ty
+    Nothing         -> Nothing
 
 -- | Converts an expression to normal form
 compute :: PGF -> Expr -> Expr
@@ -317,20 +317,20 @@ browse :: PGF -> CId -> Maybe (String,[CId],[CId])
 browse pgf id = fmap (\def -> (def,producers,consumers)) definition
   where
     definition = case Map.lookup id (funs (abstract pgf)) of
-                   Just (ty,_,Just eqs,_,_) -> Just $ render (text "fun" <+> ppCId id <+> colon <+> ppType 0 [] ty $$
-                                                              if null eqs
-                                                                then empty
-                                                                else text "def" <+> vcat [let scope = foldl pattScope [] patts
-                                                                                              ds    = map (ppPatt 9 scope) patts
-                                                                                          in ppCId id <+> hsep ds <+> char '=' <+> ppExpr 0 scope res | Equ patts res <- eqs])
-                   Just (ty,_,Nothing, _,_) -> Just $ render (text "data" <+> ppCId id <+> colon <+> ppType 0 [] ty)
+                   Just (ty,_,Just (eqs,_),_) -> Just $ render (text "fun" <+> ppCId id <+> colon <+> ppType 0 [] ty $$
+                                                                if null eqs
+                                                                  then empty
+                                                                  else text "def" <+> vcat [let scope = foldl pattScope [] patts
+                                                                                                ds    = map (ppPatt 9 scope) patts
+                                                                                            in ppCId id <+> hsep ds <+> char '=' <+> ppExpr 0 scope res | Equ patts res <- eqs])
+                   Just (ty,_,Nothing,_)      -> Just $ render (text "data" <+> ppCId id <+> colon <+> ppType 0 [] ty)
                    Nothing   -> case Map.lookup id (cats (abstract pgf)) of
-                                  Just (hyps,_,_,_) -> Just $ render (text "cat" <+> ppCId id <+> hsep (snd (mapAccumL (ppHypo 4) [] hyps)))
-                                  Nothing           -> Nothing
+                                  Just (hyps,_,_) -> Just $ render (text "cat" <+> ppCId id <+> hsep (snd (mapAccumL (ppHypo 4) [] hyps)))
+                                  Nothing         -> Nothing
 
     (producers,consumers) = Map.foldWithKey accum ([],[]) (funs (abstract pgf))
       where
-        accum f (ty,_,_,_,_) (plist,clist) = 
+        accum f (ty,_,_,_) (plist,clist) = 
           let !plist' = if id `elem` ps then f : plist else plist
               !clist' = if id `elem` cs then f : clist else clist
           in (plist',clist')
@@ -348,12 +348,12 @@ browse pgf id = fmap (\def -> (def,producers,consumers)) definition
     expIds _            ids = ids
 
 -- | A type for plain applicative trees
-data ATree = Other Tree | App CId  [ATree]  deriving Show
-data Trie  = Oth   Tree | Ap  CId [[Trie ]] deriving Show
+data ATree t = Other t    | App CId  [ATree t]  deriving Show
+data Trie    = Oth   Tree | Ap  CId [[Trie   ]] deriving Show
 -- ^ A type for tries of plain applicative trees
 
 -- | Convert a 'Tree' to an 'ATree'
-toATree :: Tree -> ATree
+toATree :: Tree -> ATree Tree
 toATree e = maybe (Other e) app (unApp e)
   where
     app (f,es) = App f (map toATree es)
