@@ -21,6 +21,7 @@ module PGF.Expr(Tree, BindType(..), Expr(..), Literal(..), Patt(..), Equation(..
 
 import PGF.CId
 import PGF.Type
+import PGF.ByteCode
 
 import Data.Char
 --import Data.Maybe
@@ -30,12 +31,6 @@ import Control.Monad
 import qualified Text.PrettyPrint as PP
 import qualified Text.ParserCombinators.ReadP as RP
 
-data Literal = 
-   LStr String                      -- ^ string constant
- | LInt Int                         -- ^ integer constant
- | LFlt Double                      -- ^ floating point constant
- deriving (Eq,Ord,Show)
-
 type MetaId = Int
 
 data BindType = 
@@ -273,10 +268,6 @@ pattScope scope (PTilde e)   = scope
 ppBind Explicit x = ppCId x
 ppBind Implicit x = PP.braces (ppCId x)
 
-ppLit (LStr s) = PP.text (show s)
-ppLit (LInt n) = PP.int n
-ppLit (LFlt d) = PP.double d
-
 ppMeta :: MetaId -> PP.Doc
 ppMeta n
   | n == 0    = PP.char '?'
@@ -324,21 +315,22 @@ data Value
   | VClosure Env Expr
   | VImplArg Value
 
-type Sig = ( Map.Map CId (Type,Int,Maybe [Equation],Double,Int) -- type and def of a fun
-           , Int -> Maybe Expr                                  -- lookup for metavariables
+type Sig = ( Map.Map CId (Type,Int,Maybe ([Equation],[[Instr]]),Double) -- type and def of a fun
+           , Int -> Maybe Expr                                          -- lookup for metavariables
            )
 type Env = [Value]
 
 eval :: Sig -> Env -> Expr -> Value
 eval sig env (EVar i)     = env !! i
 eval sig env (EFun f)     = case Map.lookup f (fst sig) of
-                              Just (_,a,meqs,_,_) -> case meqs of
-                                                       Just eqs -> if a == 0
-                                                                     then case eqs of
-                                                                            Equ [] e : _ -> eval sig [] e
-                                                                            _            -> VConst f []
-                                                                     else VApp f []
-                                                       Nothing  -> VApp f []
+                              Just (_,a,meqs,_) -> case meqs of
+                                                       Just (eqs,_)
+                                                               -> if a == 0
+                                                                    then case eqs of
+                                                                           Equ [] e : _ -> eval sig [] e
+                                                                           _            -> VConst f []
+                                                                    else VApp f []
+                                                       Nothing -> VApp f []
                               Nothing             -> error ("unknown function "++showCId f)
 eval sig env (EApp e1 e2) = apply sig env e1 [eval sig env e2]
 eval sig env (EAbs b x e) = VClosure env (EAbs b x e)
@@ -353,11 +345,11 @@ apply :: Sig -> Env -> Expr -> [Value] -> Value
 apply sig env e            []     = eval sig env e
 apply sig env (EVar i)     vs     = applyValue sig (env !! i) vs
 apply sig env (EFun f)     vs     = case Map.lookup f (fst sig) of
-                                      Just (_,a,meqs,_,_) -> case meqs of
-                                                               Just eqs -> if a <= length vs
-                                                                             then match sig f eqs vs
-                                                                             else VApp f vs
-                                                               Nothing  -> VApp f vs
+                                      Just (_,a,meqs,_) -> case meqs of
+                                                             Just (eqs,_) -> if a <= length vs
+                                                                               then match sig f eqs vs
+                                                                               else VApp f vs
+                                                             Nothing      -> VApp f vs
                                       Nothing           -> error ("unknown function "++showCId f)
 apply sig env (EApp e1 e2) vs     = apply sig env e1 (eval sig env e2 : vs)
 apply sig env (EAbs b x e) (v:vs) = case (b,v) of
