@@ -1,3 +1,4 @@
+{-# LANGUAGE FlexibleContexts #-}
 module PGF.Linearize
             ( linearize
             , linearizeAll
@@ -101,7 +102,7 @@ linTree pgf cnc e = nub (map snd (lin Nothing 0 e [] [] e []))
             Nothing        -> concat [toApp fid prod | (fid,set) <- IntMap.toList prods, prod <- Set.toList set]
           where
             toApp fid (PApply funid pargs) =
-              let Just (ty,_,_,_,_) = Map.lookup f (funs (abstract pgf))
+              let Just (ty,_,_,_) = Map.lookup f (funs (abstract pgf))
                   (args,res) = catSkeleton ty
               in [(funid,(res,fid),zip args [fid | PArg _ fid <- pargs])]
             toApp _   (PCoerce fid) = 
@@ -123,5 +124,5 @@ linTree pgf cnc e = nub (map snd (lin Nothing 0 e [] [] e []))
                                   def (Just (cat,fid)) n_fid e0 ys xs s
     def Nothing          n_fid e0 ys xs s = []
 
-amapWithIndex :: (IArray a e1, IArray a e2, Ix i) => (i -> e1 -> e2) -> a i e1 -> a i e2
-amapWithIndex f arr = listArray (bounds arr) (map (uncurry f) (assocs arr))
+--amapWithIndex :: (IArray a e1, IArray a e2, Ix i) => (i -> e1 -> e2) -> a i e1 -> a i e2
+--amapWithIndex f arr = listArray (bounds arr) (map (uncurry f) (assocs arr))
