@@ -1,4 +1,4 @@
-{-# LANGUAGE BangPatterns, RankNTypes #-}
+{-# LANGUAGE BangPatterns, RankNTypes, FlexibleContexts #-}
 module PGF.Parse
           ( ParseState
           , ErrorState
@@ -10,11 +10,12 @@ module PGF.Parse
           , ParseOutput(..), getParseOutput
           , parse
           , parseWithRecovery
+          , getContinuationInfo
           ) where
 
 import Data.Array.IArray
 import Data.Array.Base (unsafeAt)
-import Data.List (isPrefixOf, foldl')
+import Data.List (isPrefixOf, foldl', intercalate)
 import Data.Maybe (fromMaybe, maybeToList)
 import qualified Data.Map as Map
 import qualified PGF.TrieMap as TrieMap
@@ -310,10 +311,18 @@ process flit ftok cnc (item@(Active j ppos funid seqid args key0):items) acc cha
       	SymBIND    -> let !acc' = ftok_ ["&+"] (Active j (ppos+1) funid seqid args key0) acc
       	              in process flit ftok cnc items acc' chart
       	SymSOFT_BIND->process flit ftok cnc ((Active j (ppos+1) funid seqid args key0):items) acc chart
+      	SymSOFT_SPACE->process flit ftok cnc ((Active j (ppos+1) funid seqid args key0):items) acc chart
+      	SymCAPIT   -> let !acc' = ftok_ ["&|"] (Active j (ppos+1) funid seqid args key0) acc
+      	              in process flit ftok cnc items acc' chart
+      	SymALL_CAPIT->let !acc' = ftok_ ["&|"] (Active j (ppos+1) funid seqid args key0) acc
+      	              in process flit ftok cnc items acc' chart
       	SymKP syms vars
       	           -> let to_tok (SymKS t)    = [t]
       	                  to_tok SymBIND      = ["&+"]
       	                  to_tok SymSOFT_BIND = []
+      	                  to_tok SymSOFT_SPACE= []
+      	                  to_tok SymCAPIT     = ["&|"]
+      	                  to_tok SymALL_CAPIT = ["&|"]
       	                  to_tok _            = []
 
       	                  !acc' = foldl (\acc syms -> ftok_ (concatMap to_tok syms) (Active j (ppos+1) funid seqid args key0) acc) acc
@@ -503,6 +512,29 @@ data Chart
 
 type Continuation = TrieMap.TrieMap Token ActiveSet
 
+-- | Return the Continuation of a Parsestate with exportable types
+--   Used by PGFService
+getContinuationInfo :: ParseState -> Map.Map [Token] [(FunId, CId, String)]
+getContinuationInfo pstate = Map.map (map f . Set.toList) contMap
+  where
+    PState _abstr concr _chart cont = pstate
+    contMap = Map.fromList (TrieMap.toList cont) -- always get [([], _::ActiveSet)]
+    f :: Active -> (FunId,CId,String)
+    f (Active int dotpos funid seqid pargs ak) = (funid, cid, seq)
+      where
+        CncFun cid _ = cncfuns concr ! funid
+        seq = showSeq dotpos (sequences concr ! seqid)
+        
+    showSeq :: DotPos -> Sequence -> String
+    showSeq pos seq = intercalate " " $ scan (drop (pos-1) (elems seq))
+      where
+        -- Scan left-to-right, stop at first non-token
+        scan :: [Symbol] -> [String]
+        scan [] = []
+        scan (sym:syms) = case sym of
+          SymKS token -> token : scan syms
+          _           -> []
+
 ----------------------------------------------------------------
 -- Error State
 ----------------------------------------------------------------
