@@ -37,6 +37,7 @@ import Data.Map as Map
 import Data.IntMap as IntMap
 import Data.Maybe as Maybe
 import Data.List as List
+import Control.Applicative
 import Control.Monad
 --import Control.Monad.Identity
 import Control.Monad.State
@@ -92,10 +93,18 @@ class Selector s where
   splitSelector :: s -> (s,s)
   select        :: CId -> Scope -> Maybe Int -> TcM s (Expr,TType)
 
+instance Applicative (TcM s) where
+  pure = return
+  (<*>) = ap
+
 instance Monad (TcM s) where
   return x = TcM (\abstr k h -> k x)
   f >>= g  = TcM (\abstr k h -> unTcM f abstr (\x -> unTcM (g x) abstr k h) h)
 
+instance Selector s => Alternative (TcM s) where
+  empty = mzero
+  (<|>) = mplus
+
 instance Selector s => MonadPlus (TcM s) where
   mzero = TcM (\abstr k h ms s -> id)
   mplus f g = TcM (\abstr k h ms s -> let (s1,s2) = splitSelector s
@@ -121,13 +130,13 @@ runTcM abstr f ms s = unTcM f abstr (\x ms s cp b -> let (es,xs) = cp b
 
 lookupCatHyps :: CId -> TcM s [Hypo]
 lookupCatHyps cat = TcM (\abstr k h ms -> case Map.lookup cat (cats abstr) of
-                                            Just (hyps,_,_,_) -> k hyps ms
-                                            Nothing           -> h (UnknownCat cat))
+                                            Just (hyps,_,_) -> k hyps ms
+                                            Nothing         -> h (UnknownCat cat))
 
 lookupFunType :: CId -> TcM s Type
 lookupFunType fun = TcM (\abstr k h ms -> case Map.lookup fun (funs abstr) of
-                                            Just (ty,_,_,_,_) -> k ty ms
-                                            Nothing           -> h (UnknownFun fun))
+                                            Just (ty,_,_,_) -> k ty ms
+                                            Nothing         -> h (UnknownFun fun))
 
 typeGenerators :: Scope -> CId -> TcM s [(Double,Expr,TType)]
 typeGenerators scope cat = fmap normalize (liftM2 (++) x y)
@@ -143,8 +152,8 @@ typeGenerators scope cat = fmap normalize (liftM2 (++) x y)
       | cat == cidString = return [(1.0,ELit (LStr "Foo"),TTyp [] (DTyp [] cat []))]
       | otherwise        = TcM (\abstr k h ms ->
                                     case Map.lookup cat (cats abstr) of
-                                      Just (_,fns,_,_) -> unTcM (mapM helper fns) abstr k h ms
-                                      Nothing          -> h (UnknownCat cat))
+                                      Just (_,fns,_) -> unTcM (mapM helper fns) abstr k h ms
+                                      Nothing        -> h (UnknownCat cat))
 
     helper (p,fn) = do
       ty <- lookupFunType fn
@@ -510,7 +519,7 @@ eqValue fail suspend k v1 v2 = do
 
     bind i scope cs env vs0 v = do
       let k  = scopeSize scope
-          vs  = reverse (take k env) ++ vs0
+          vs  = reverse (List.take k env) ++ vs0
           xs  = nub [i | VGen i [] <- vs]
       if length vs /= length xs
         then suspend i (\e -> apply env e vs0 >>= \iv -> eqValue fail suspend k iv v)
@@ -591,6 +600,7 @@ generateForForest :: (Scope -> TType -> TcM FId Expr) -> Expr -> TcM FId Expr
 generateForForest prove e = do
 --  fillinVariables
   refineExpr e
+{-
   where
     fillinVariables = do
       fvs <- TcM (\abstr k h ms -> k [(i,s,scope,tty,cs) | (i,MUnbound s scope tty cs) <- IntMap.toList ms] ms)
@@ -605,6 +615,7 @@ generateForForest prove e = do
                                                                     sequence_ [c e | c <- cs]
                                                                     fillinVariables
                                                                 ) abstr k h ms s)
+-}
 
 -----------------------------------------------------
 -- evalType
