@@ -3,7 +3,7 @@ package org.grammaticalframework.pgf;
 import java.util.Iterator;
 
 public interface LiteralCallback {
-	public CallbackResult match(int lin_idx, String sentence, int start_offset);
+	public CallbackResult match(int lin_idx, int start_offset);
 
 	public Iterator<TokenProb> predict(int lin_idx, String prefix);
 
