@@ -1,13 +1,18 @@
 {-# LANGUAGE CPP #-}
 module PGFService(cgiMain,cgiMain',getPath,
                   logFile,stderrToFile,
-                  newPGFCache,flushPGFCache,listPGFCache) where
+                  Caches,pgfCache,newPGFCache,flushPGFCache,listPGFCache) where
 
-import PGF (PGF)
+import PGF (PGF,Labels,CncLabels)
 import qualified PGF
 import PGF.Lexing
 import Cache
-import FastCGIUtils
+import CGIUtils(outputJSONP,outputPlain,outputHTML,outputText,
+                outputBinary,outputBinary',
+                logError,handleCGIErrors,throwCGIError,stderrToFile)
+import CGI(CGI,readInput,getInput,getVarWithDefault,
+           CGIResult,requestAcceptLanguage,handleErrors,setHeader,
+           Accept(..),Language(..),negotiate,liftIO)
 import URLEncoding
 
 #if C_RUNTIME
@@ -17,8 +22,11 @@ import qualified PGF2 as C
 
 import Data.Time.Clock(UTCTime)
 import Data.Time.Format(formatTime)
+#if MIN_VERSION_time(1,5,0)
+import Data.Time.Format(defaultTimeLocale,rfc822DateFormat)
+#else
 import System.Locale(defaultTimeLocale,rfc822DateFormat)
-import Network.CGI
+#endif
 import Text.JSON
 import Text.PrettyPrint as PP(render, text, (<+>))
 import qualified Codec.Binary.UTF8.String as UTF8 (decodeString)
@@ -28,9 +36,10 @@ import Control.Concurrent
 import qualified Control.Exception as E
 import Control.Monad
 import Control.Monad.State(State,evalState,get,put)
+import Control.Monad.Catch(bracket_)
 import Data.Char
-import Data.Function (on)
-import Data.List (sortBy,intersperse,mapAccumL,nub,isSuffixOf)
+--import Data.Function (on)
+import Data.List ({-sortBy,-}intersperse,mapAccumL,nub,isSuffixOf,nubBy)
 import qualified Data.Map as Map
 import Data.Maybe
 import System.Random
@@ -39,35 +48,63 @@ import System.Exit
 import System.IO
 import System.IO.Error(isDoesNotExistError)
 import System.Directory(removeFile)
+import System.FilePath(dropExtension,takeDirectory,(</>),(<.>))
 import System.Mem(performGC)
 import Fold(fold) -- transfer function for OpenMath LaTeX
 
 catchIOE :: IO a -> (E.IOException -> IO a) -> IO a
 catchIOE = E.catch
 
+withQSem qsem = bracket_ (liftIO $ waitQSem qsem) (liftIO $ signalQSem qsem)
+
 logFile :: FilePath
 logFile = "pgf-error.log"
 
 #ifdef C_RUNTIME
-type Caches = (Cache PGF,Cache (C.PGF,({-MVar ParseCache-})))
+data Caches = Caches { pgfCache::Cache PGF,
+                       labelsCache::Cache Labels,
+                       cncLabelsCache::Cache CncLabels,
+                       cpgfCache::(Cache (C.PGF,({-MVar ParseCache-})),QSem) }
+--type Caches = (Cache PGF,Cache (C.PGF,({-MVar ParseCache-})))
 --type ParseCache = Map.Map (String,String) (ParseResult,UTCTime)
 --type ParseResult = Either String [(C.Expr,Float)]
 
-newPGFCache = do pgfCache <- newCache PGF.readPGF
-                 cCache <- newCache $ \ path -> do pgf <- C.readPGF path
-                                                 --pc <- newMVar Map.empty
-                                                   return (pgf,({-pc-}))
-                 return (pgfCache,cCache)
-flushPGFCache (c1,c2) = flushCache c1 >> flushCache c2
-listPGFCache (c1,c2) = (,) # listCache c1 % listCache c2
+newPGFCache jobs = do pgfCache <- newCache' PGF.readPGF
+                      lblCache <- newCache' (fmap PGF.getDepLabels . readFile)
+                      clblCache <- newCache'(fmap PGF.getCncDepLabels .readFile)
+                      let n = maybe 4 id jobs
+                      qsem <- newQSem n
+                      cCache <- newCache' $ \ path -> do pgf <- C.readPGF path
+                                                       --pc <- newMVar Map.empty
+                                                         return (pgf,({-pc-}))
+                      return $ Caches pgfCache lblCache clblCache (cCache,qsem)
+flushPGFCache c = do flushCache (pgfCache c)
+                     flushCache (labelsCache c)
+                     flushCache (fst (cpgfCache c))
+listPGFCache c = (,) # listCache (pgfCache c) % listCache (fst (cpgfCache c))
 #else
-type Caches = (Cache PGF,())
-newPGFCache = do pgfCache <- newCache PGF.readPGF
-                 return (pgfCache,())
-flushPGFCache (c1,_) = flushCache c1
-listPGFCache (c1,_) = (,) # listCache c1 % return []
+data Caches = Caches { pgfCache::Cache PGF,
+                       labelsCache::Cache Labels,
+                       cncLabelsCache::Cache CncLabels }
+newPGFCache _ = do pgfCache <- newCache' PGF.readPGF
+                   lblCache <- newCache' (fmap PGF.getDepLabels . readFile)
+                   clblCache <- newCache'(fmap PGF.getCncDepLabels .readFile)
+                   return $ Caches pgfCache lblCache clblCache
+flushPGFCache c = flushCache (pgfCache c)
+
+listPGFCache :: Caches -> IO ([(FilePath,UTCTime)],[(FilePath,UTCTime)])
+listPGFCache c = (,) # listCache (pgfCache c) % return []
 #endif
 
+labelsCaches c = (labelsCache c,cncLabelsCache c)
+
+newCache' rd = do c <- newCache rd
+                  forkIO $ forever $ clean c
+                  return c
+  where
+    clean c = do threadDelay 2000000000 -- 2000 seconds, i.e. ~33 minutes
+                 expireCache (24*60*60) c -- 24 hours
+
 getPath =
     do path <- getVarWithDefault "PATH_TRANSLATED" "" -- apache mod_fastcgi
        if null path
@@ -83,14 +120,21 @@ cgiMain' cache path =
     do command <- liftM (maybe "grammar" (urlDecodeUnicode . UTF8.decodeString))
                         (getInput "command")
        case command of
-         "download" -> outputBinary    =<< getFile BS.readFile path
-         'c':'-':_  ->
+         "download" -> outputBinary =<< getFile BS.readFile path
+         'c':'-':_  -> optionalCpgfMain cache path command
+         _ -> do tpgf <- getFile (readCache' (pgfCache cache)) path
+                 pgfMain (labelsCaches cache) path command tpgf
+
+optionalCpgfMain cache path command =
 #ifdef C_RUNTIME
-                   cpgfMain command =<< getFile (readCache' (snd cache)) path
+    cpgfMain (snd (cpgfCache cache)) command
+       =<< getFile (readCache' (fst (cpgfCache cache))) path
 #else
-                   serverError "Server configured without C run-time support" ""
+    serverError "Server configured without C run-time support" ""
+
+serverError = throw 500
+
 #endif
-         _          -> pgfMain command =<< getFile (readCache' (fst cache)) path
 
 getFile get path =
    either failed return =<< liftIO (E.try (get path))
@@ -104,14 +148,19 @@ getFile get path =
 
 #ifdef C_RUNTIME
 --cpgfMain :: String -> (C.PGF,MVar ParseCache) -> CGI CGIResult
-cpgfMain command (t,(pgf,pc)) =
+cpgfMain qsem command (t,(pgf,pc)) =
   case command of
-    "c-parse"       -> out t=<< join (parse # input % start % limit % trie)
+    "c-parse"       -> withQSem qsem $
+                       out t=<< join (parse # input % start % limit % treeopts)
     "c-linearize"   -> out t=<< lin # tree % to
-    "c-translate"   -> out t=<< join (trans # input % to % start % limit % trie)
+    "c-linearizeAll"-> out t=<< linAll # tree % to
+    "c-translate"   -> withQSem qsem $
+                       out t=<<join(trans # input % to % start % limit%treeopts)
     "c-lookupmorpho"-> out t=<< morpho # from1 % textInput
     "c-flush"       -> out t=<< flush
     "c-grammar"     -> out t grammar
+    "c-abstrtree"   -> outputGraphviz=<< C.graphvizAbstractTree pgf # tree
+    "c-parsetree"   -> outputGraphviz=<< C.graphvizParseTree . snd # from1 %tree
     "c-wordforword" -> out t =<< wordforword # input % to
     _               -> badRequest "Unknown command" command
   where
@@ -125,25 +174,29 @@ cpgfMain command (t,(pgf,pc)) =
     grammar = showJSON $ makeObj
                  ["name".=C.abstractName pgf,
                   "lastmodified".=show t,
-                  "startcat".=C.startCat pgf,
+                  "startcat".=C.showType [] (C.startCat pgf),
                   "languages".=languages]
       where
         languages = [makeObj ["name".= l] | (l,_)<-Map.toList langs]
 
-    parse input@((from,_),_) start mlimit trie =
+    parse input@((from,_),_) start mlimit (trie,json) =
         do r <- parse' start mlimit input
-           return $ showJSON [makeObj ("from".=from:jsonParseResult r)]
+           return $ showJSON [makeObj ("from".=from:jsonParseResult json r)]
 
-    jsonParseResult = either bad good
+    jsonParseResult json = either bad good
       where
         bad err = ["parseFailed".=err]
         good trees = "trees".=map tp trees :[]  -- :addTrie trie trees
-        tp (tree,prob) = makeObj ["tree".=tree,"prob".=prob]
+        tp (tree,prob) = makeObj (addTree json tree++["prob".=prob])
 
     -- Without caching parse results:
-    parse' start mlimit ((_,concr),input) =
-      return $
-      maybe id take mlimit . drop start # C.parse concr cat input
+    parse' start mlimit ((from,concr),input) =
+        return $ maybe id take mlimit . drop start # cparse
+      where
+      --cparse = C.parse concr cat input
+        cparse = C.parseWithHeuristics concr cat input (-1) callbacks
+        callbacks = maybe [] cb $ lookup (C.abstractName pgf) C.literalCallbacks
+        cb fs = [(cat,f pgf (from,concr) input)|(cat,f)<-fs]
 {-
     -- Caching parse results:
     parse' start mlimit ((from,concr),input) = 
@@ -161,11 +214,16 @@ cpgfMain command (t,(pgf,pc)) =
             purge r@(_,t') = if diffUTCTime t t'<120 then Just r else Nothing
                              -- remove unused parse results after 2 minutes
 -}
+    linAll tree to = showJSON (linAll' tree to)
+    linAll' tree (tos,unlex) =
+        [makeObj ["to".=to,
+                  "texts".=map unlex (C.linearizeAll c tree)]|(to,c)<-tos]
+
     lin tree to = showJSON (lin' tree to)
     lin' tree (tos,unlex) =
         [makeObj ["to".=to,"text".=unlex (C.linearize c tree)]|(to,c)<-tos]
 
-    trans input@((from,_),_) to start mlimit trie =
+    trans input@((from,_),_) to start mlimit (trie,jsontree) =
       do parses <- parse' start mlimit input
          return $
            showJSON [ makeObj ["from".=from,
@@ -174,9 +232,9 @@ cpgfMain command (t,(pgf,pc)) =
         jsonParses = either bad good
           where
             bad err = [makeObj ["error".=err]]
-            good parses = [makeObj ["tree".=tree,
-                                     "prob".=prob,
-                                     "linearizations".=lin' tree to]
+            good parses = [makeObj (addTree jsontree tree++
+                                    ["prob".=prob,
+                                     "linearizations".=lin' tree to])
                                     | (tree,prob) <- parses]
 
     morpho (from,concr) input =
@@ -235,7 +293,7 @@ cpgfMain command (t,(pgf,pc)) =
     from1 = maybe (missing "from") return =<< from'
     from' = getLang "from"
 
-    to = (,) # getLangs "to" % unlexer
+    to = (,) # getLangs "to" % unlexerC
 
     getLangs = getLangs' readLang
     getLang = getLang' readLang
@@ -262,7 +320,18 @@ instance JSON C.CId where
 -}
 instance JSON C.Expr where
     readJSON x = readJSON x >>= maybe (fail "Bad expression.") return . C.readExpr
-    showJSON = showJSON . C.showExpr
+    showJSON = showJSON . C.showExpr []
+
+
+-- | Convert a 'Tree' to an 'ATree'
+cToATree :: C.Expr -> PGF.ATree C.Expr
+cToATree e = maybe (PGF.Other e) app (C.unApp e)
+  where
+    app (f,es) = PGF.App (read f) (map cToATree es)
+
+instance ToATree C.Expr where
+  showTree = show
+  toATree = cToATree
 
 #endif
 
@@ -272,9 +341,9 @@ instance JSON C.Expr where
 -- | Lexers with a text lexer that tries to be a more clever with the first word
 ilexer good = lexer' uncap
   where
-    uncap s = if good s
-              then s
-              else uncapitInit s
+    uncap s = case span isUpper s of
+                ([c],r) | not (good s) -> toLower c:r
+                _ -> s
 
 -- | Standard lexers
 lexer = lexer' uncapitInit
@@ -291,40 +360,64 @@ lexer' uncap = maybe (return id) lexerfun =<< getInput "lexer"
 
 type Unlexer = String->String
 
-unlexer :: CGI Unlexer
-unlexer = maybe (return id) unlexerfun =<< getInput "unlexer" 
+-- | Unlexing for the C runtime system, &+ is already applied
+unlexerC :: CGI Unlexer
+unlexerC = maybe (return id) unlexerfun =<< getInput "unlexer"
   where
     unlexerfun name =
        case name of
          "text"  -> return (unlexText' . words)
          "code"  -> return (unlexCode . words)
          "mixed" -> return (unlexMixed . words)
+         "none"  -> return id
+         "id"    -> return id
          _ -> badRequest "Unknown lexer" name
 
-    unlexText' ("+":ws) = "+ "++unlexText ws
-    unlexText' ("*":ws) = "* "++unlexText ws
-    unlexText' ws       = unlexText ws
+-- | Unlex text, skipping the quality marker used by the App grammar
+unlexText' ("+":ws) = "+ "++unlexText ws
+unlexText' ("*":ws) = "* "++unlexText ws
+unlexText' ws       = unlexText ws
+
+-- | Unlexing for the Haskell run-time, applying the &+ operator first
+unlexerH :: CGI Unlexer
+unlexerH = maybe (return doBind) unlexerfun =<< getInput "unlexer"
+  where
+    unlexerfun name =
+       case name of
+         "text"  -> return (unlexText' . bind)
+         "code"  -> return (unlexCode . bind)
+         "mixed" -> return (unlexMixed . bind)
+         "none"  -> return id
+         "id"    -> return id
+         "bind"  -> return doBind
+         _ -> badRequest "Unknown lexer" name
+
+    doBind = unwords . bind
+    bind = bindTok . words
 
 --------------------------------------------------------------------------------
 -- * Haskell run-time functionality
 
---pgfMain :: String -> PGF -> CGI CGIResult
-pgfMain command (t,pgf) =
+--pgfMain :: Cache Labels -> FilePath -> String -> PGF -> CGI CGIResult
+pgfMain lcs@(alc,clc) path command tpgf@(t,pgf) =
     case command of
-      "parse"          -> o =<< doParse pgf # input % cat % limit % trie
-      "complete"       -> o =<< doComplete pgf # input % cat % limit
+      "parse"          -> o =<< doParse pgf # input % cat % limit % treeopts
+      "complete"       -> o =<< doComplete pgf # input % cat % limit % full
       "linearize"      -> o =<< doLinearize pgf # tree % to
       "linearizeAll"   -> o =<< doLinearizes pgf # tree % to
       "linearizeTable" -> o =<< doLinearizeTabular pgf # tree % to
       "random"         -> o =<< join (doRandom pgf # cat % depth % limit % to)
       "generate"       -> o =<< doGenerate pgf # cat % depth % limit % to
-      "translate"      -> o =<< doTranslate pgf # input % cat % to % limit %trie
+      "translate"      -> o =<< doTranslate pgf # input % cat %to%limit%treeopts
       "translategroup" -> o =<< doTranslateGroup pgf # input % cat % to % limit
       "lookupmorpho"   -> o =<< doLookupMorpho pgf # from1 % textInput
-      "grammar"        -> o =<< doGrammar t pgf # requestAcceptLanguage
+      "grammar"        -> join $ doGrammar tpgf
+                                       # liftIO (E.try (getLabels alc path pgf))
+                                       % requestAcceptLanguage
       "abstrtree"      -> outputGraphviz =<< abstrTree pgf # graphvizOptions % tree
       "alignment"      -> outputGraphviz =<< alignment pgf # tree % to
       "parsetree"      -> outputGraphviz =<< parseTree pgf # from1 % graphvizOptions % tree
+      "deptree"        -> join $ doDepTree lcs path pgf # format "dot" % to1 % tree
       "abstrjson"      -> o . jsonExpr =<< tree
       "browse"         -> join $ doBrowse pgf # optId % cssClass % href % format "html" % getIncludePrintNames
       "external"       -> do cmd <- getInput "external"
@@ -381,6 +474,7 @@ pgfMain command (t,pgf) =
         PGF.GraphvizOptions # bool "noleaves"
                             % bool "nofun"
                             % bool "nocat"
+                            % bool "nodep"
                             % string "nodefont"
                             % string "leaffont"
                             % string "nodecolor"
@@ -393,7 +487,9 @@ pgfMain command (t,pgf) =
 
     from1 = maybe (missing "from") return =<< from
     from = getLang "from"
-    to = (,) # getLangs "to" % unlexer
+
+    to1 = maybe (missing "to") return =<< getLang "to"
+    to = (,) # getLangs "to" % unlexerH
 
     getLangs = getLangs' readLang
     getLang = getLang' readLang
@@ -405,6 +501,9 @@ pgfMain command (t,pgf) =
         Just lang | lang `elem` PGF.languages pgf -> return lang
                   | otherwise -> badRequest "Unknown language" l
 
+    full :: CGI Bool
+    full = maybe False toBool # getInput "full"
+
 -- * Request parameter access and related auxiliary functions
 
 --out = outputJSONP
@@ -435,9 +534,10 @@ depth = readInput "depth"
 start :: CGI Int
 start = maybe 0 id # readInput "start"
 
-trie :: CGI Bool
-trie = maybe False toBool # getInput "trie"
+treeopts :: CGI TreeOpts
+treeopts = (,) # getBool "trie" % getBool "jsontree"
 
+getBool x = maybe False toBool # getInput x
 toBool s = s `elem` ["","yes","true","True"]
 
 missing = badRequest "Missing parameter"
@@ -445,7 +545,6 @@ errorMissingId = badRequest "Missing identifier" ""
 
 notFound = throw 404 "Not found"
 badRequest = throw 400
-serverError = throw 500
 
 throw code msg extra =
     throwCGIError code msg [msg ++(if null extra then "" else ": "++extra)]
@@ -483,9 +582,10 @@ doLookupMorpho pgf from input =
 
 type From = (Maybe PGF.Language,String)
 type To = ([PGF.Language],Unlexer)
+type TreeOpts = (Bool,Bool) -- (trie,jsontree)
 
-doTranslate :: PGF -> From -> Maybe PGF.Type -> To -> Maybe Int -> Bool -> JSValue
-doTranslate pgf (mfrom,input) mcat (tos,unlex) mlimit trie =
+doTranslate :: PGF -> From -> Maybe PGF.Type -> To -> Maybe Int -> TreeOpts -> JSValue
+doTranslate pgf (mfrom,input) mcat tos mlimit (trie,jsontree) =
   showJSON
      [makeObj ("from".=from : "brackets".=bs : jsonTranslateOutput po)
           | (from,po,bs) <- parse' pgf input mcat mfrom]
@@ -495,11 +595,11 @@ doTranslate pgf (mfrom,input) mcat (tos,unlex) mlimit trie =
         PGF.ParseOk trees ->
             addTrie trie trees++
             ["translations".=
-              [makeObj ["tree".=tree,
-                        "linearizations".=
-                            [makeObj ["to".=to, "text".=unlex text,
+              [makeObj (addTree jsontree tree++
+                       ["linearizations".=
+                            [makeObj ["to".=to, "text".=text,
                                       "brackets".=bs]
-                               | (to,text,bs)<- linearizeAndBind pgf tos tree]]
+                               | (to,text,bs)<- linearizeAndUnlex pgf tos tree]])
                 | tree <- maybe id take mlimit trees]]
         PGF.ParseIncomplete -> ["incomplete".=True]
         PGF.ParseFailed n   -> ["parseFailed".=n]
@@ -511,17 +611,17 @@ jsonTypeErrors errs =
 
 -- used in phrasebook
 doTranslateGroup :: PGF -> From -> Maybe PGF.Type -> To -> Maybe Int -> JSValue
-doTranslateGroup pgf (mfrom,input) mcat (tos,unlex) mlimit =
+doTranslateGroup pgf (mfrom,input) mcat tos mlimit =
   showJSON
     [makeObj ["from".=langOnly (PGF.showLanguage from),
               "to".=langOnly (PGF.showLanguage to),
               "linearizations".=
-                 [toJSObject (("text",unlex alt) : disamb lg from ts)
-                    | (ts,alt) <- output, let lg = length output]
+                 [toJSObject (("text",alt) : disamb lg from ts)
+                    | let lg = length output, (ts,alt) <- output]
               ]
        | 
          (from,po,bs) <- parse' pgf input mcat mfrom,
-         (to,output)  <- groupResults [(t, linearizeAndBind pgf tos t) | t <- case po of {PGF.ParseOk ts -> maybe id take mlimit ts; _ -> []}]
+         (to,output)  <- groupResults [(t, linearizeAndUnlex pgf tos t) | t <- case po of {PGF.ParseOk ts -> maybe id take mlimit ts; _ -> []}]
           ]
   where
    groupResults = Map.toList . foldr more Map.empty . start . collect
@@ -540,7 +640,7 @@ doTranslateGroup pgf (mfrom,input) mcat (tos,unlex) mlimit =
    disamb lg from ts = 
      if lg < 2 
        then [] 
-       else [("tree", "-- " ++ groupDisambs [unlex (disambLang from t) | t <- ts])]
+       else [("tree", "-- " ++ groupDisambs [disambLang from t | t <- ts])]
 
    groupDisambs = unwords . intersperse "/"
 
@@ -558,15 +658,17 @@ doTranslateGroup pgf (mfrom,input) mcat (tos,unlex) mlimit =
 
    notDisamb = (/="Disamb") . take 6 . PGF.showLanguage
 
-doParse :: PGF -> From -> Maybe PGF.Type -> Maybe Int -> Bool -> JSValue
-doParse pgf (mfrom,input) mcat mlimit trie = showJSON $ map makeObj
+doParse :: PGF -> From -> Maybe PGF.Type -> Maybe Int -> TreeOpts -> JSValue
+doParse pgf (mfrom,input) mcat mlimit (trie,jsontree) = showJSON $ map makeObj
      ["from".=from : "brackets".=bs : jsonParseOutput po
         | (from,po,bs) <- parse' pgf input mcat mfrom]
   where
     jsonParseOutput output =
       case output of
-        PGF.ParseOk trees   -> ["trees".=maybe id take mlimit trees]
+        PGF.ParseOk trees   -> ["trees".=trees']
+                               ++["jsontrees".=map jsonExpr trees'|jsontree]
                                ++addTrie trie trees
+          where trees' = maybe id take mlimit trees
         PGF.TypeError errs  -> jsonTypeErrors errs
         PGF.ParseIncomplete -> ["incomplete".=True]
         PGF.ParseFailed n   -> ["parseFailed".=n]
@@ -574,22 +676,48 @@ doParse pgf (mfrom,input) mcat mlimit trie = showJSON $ map makeObj
 addTrie trie trees =
     ["trie".=map head (PGF.toTrie (map PGF.toATree trees))|trie]
 
-doComplete :: PGF -> From -> Maybe PGF.Type -> Maybe Int -> JSValue
-doComplete pgf (mfrom,input) mcat mlimit = showJSON
-    [makeObj ["from".=from, "brackets".=bs, "completions".=cs, "text".=s]
-       | from <- froms, let (bs,s,cs) = complete' pgf from cat mlimit input]
+addTree json tree = "tree".=showTree tree:
+                    ["jsontree".= jsonExpr tree | json]
+
+doComplete :: PGF -> From -> Maybe PGF.Type -> Maybe Int -> Bool -> JSValue
+doComplete pgf (mfrom,input) mcat mlimit full = showJSON
+    [makeObj (
+        ["from".=from, "brackets".=bs, "text".=s] ++
+        if full
+          then [ "completions" .= Map.elems (Map.mapWithKey (completionInfo pgf) cs) ]
+          else [ "completions" .= Map.keys cs ]
+        )
+    | from <- froms, let (bs,s,cs) = complete' pgf from cat mlimit input]
   where
     froms = maybe (PGF.languages pgf) (:[]) mfrom
     cat = fromMaybe (PGF.startCat pgf) mcat
 
+completionInfo :: PGF -> PGF.Token -> PGF.ParseState -> JSValue
+completionInfo pgf token pstate =
+  makeObj
+  ["token".= token
+  ,"funs" .= (map mkFun (nubBy ignoreFunIds funs))
+  ]
+  where
+    contInfo = PGF.getContinuationInfo pstate
+    funs = snd . head $ Map.toList contInfo -- always get [([],_)] ; funs :: [(fid,cid,seq)]
+    ignoreFunIds (_,cid1,seq1) (_,cid2,seq2) = (cid1,seq1) == (cid2,seq2)
+    mkFun (funid,cid,seq) = case PGF.functionType pgf cid of
+      Just typ ->
+        makeObj [ {-"fid".=funid,-} "fun".=cid, "hyps".=hyps', "cat".=cat, "seq".=seq ]
+        where
+          (hyps,cat,_es) = PGF.unType typ
+          hyps' = [ PGF.showType [] typ | (_,_,typ) <- hyps ]
+      Nothing -> makeObj [ "error".=("Function "++show cid++" not found") ] -- shouldn't happen
+
 doLinearize :: PGF -> PGF.Tree -> To -> JSValue
-doLinearize pgf tree (tos,unlex) = showJSON
-    [makeObj ["to".=to, "text".=unlex text,"brackets".=bs]
-      | (to,text,bs) <- linearizeAndBind pgf tos tree]
+doLinearize pgf tree tos = showJSON
+    [makeObj ["to".=to, "text".=text,"brackets".=bs]
+      | (to,text,bs) <- linearizeAndUnlex pgf tos tree]
 
 doLinearizes :: PGF -> PGF.Tree -> To -> JSValue
 doLinearizes pgf tree (tos,unlex) = showJSON
-    [makeObj ["to".=to, "texts".=map (unlex . doBind) texts]
+    [makeObj ["to".=to, "texts".=map unlex texts]
        | (to,texts) <- linearizes' pgf tos tree]
   where
     linearizes' pgf tos tree =
@@ -599,9 +727,9 @@ doLinearizes pgf tree (tos,unlex) = showJSON
         lins to = nub . concatMap (map snd) . PGF.tabularLinearizes pgf to
 
 doLinearizeTabular :: PGF -> PGF.Tree -> To -> JSValue
-doLinearizeTabular pgf tree (tos,unlex) = showJSON
+doLinearizeTabular pgf tree tos = showJSON
     [makeObj ["to".=to,
-              "table".=[makeObj ["params".=ps,"texts".=map unlex ts]
+              "table".=[makeObj ["params".=ps,"texts".=ts]
                          | (ps,ts)<-texts]]
        | (to,texts) <- linearizeTabular pgf tos tree]
 
@@ -619,11 +747,11 @@ doRandom pgf mcat mdepth mlimit to =
         depth = fromMaybe 4 mdepth
 
 doGenerate :: PGF -> Maybe PGF.Type -> Maybe Int -> Maybe Int -> To -> JSValue
-doGenerate pgf mcat mdepth mlimit (tos,unlex) =
+doGenerate pgf mcat mdepth mlimit tos =
     showJSON [makeObj ["tree".=PGF.showExpr [] tree,
                        "linearizations".=
-                          [makeObj ["to".=to, "text".=unlex text]
-                             | (to,text,bs) <- linearizeAndBind pgf tos tree]]
+                          [makeObj ["to".=to, "text".=text]
+                             | (to,text,bs) <- linearizeAndUnlex pgf tos tree]]
                 | tree <- limit trees]
   where
     trees = PGF.generateAllDepth pgf cat (Just depth)
@@ -631,16 +759,18 @@ doGenerate pgf mcat mdepth mlimit (tos,unlex) =
     limit = take (fromMaybe 1 mlimit)
     depth = fromMaybe 4 mdepth
 
-doGrammar :: UTCTime -> PGF -> Maybe (Accept Language) -> JSValue
-doGrammar t pgf macc = showJSON $ makeObj
+doGrammar :: (UTCTime,PGF) -> Either IOError (UTCTime,l) -> Maybe (Accept Language) -> CGI CGIResult
+doGrammar (t1,pgf) elbls macc = out t $ showJSON $ makeObj
              ["name".=PGF.abstractName pgf,
               "lastmodified".=show t,
+              "hasDependencyLabels".=either (const False) (const True) elbls,
               "userLanguage".=selectLanguage pgf macc,
               "startcat".=PGF.showType [] (PGF.startCat pgf),
               "categories".=categories,
               "functions".=functions,
               "languages".=languages]
   where
+    t = either (const t1) (max t1 . fst) elbls
     languages =
        [makeObj ["name".= l, 
                   "languageCode".= fromMaybe "" (PGF.languageCode pgf l)]
@@ -654,9 +784,7 @@ outputGraphviz code =
        "gv" -> outputPlain code
        _ -> outputFPS' fmt =<< liftIO (pipeIt2graphviz fmt code)
   where
-    outputFPS' fmt bs =
-      do setHeader "Content-Type" (mimeType fmt)
-         outputFPS bs
+    outputFPS' = outputBinary' . mimeType
 
     mimeType fmt =
       case fmt of
@@ -671,6 +799,39 @@ abstrTree pgf      opts tree = PGF.graphvizAbstractTree pgf opts' tree
 
 parseTree pgf lang opts tree = PGF.graphvizParseTree pgf lang opts tree
 
+doDepTree (alc,clc) path pgf fmt lang tree =
+  do (_,lbls) <- liftIO $ getLabels alc path pgf
+     clbls <- liftIO $ getCncLabels clc path pgf lang
+     let vis = PGF.graphvizDependencyTree fmt False (Just lbls) clbls pgf lang tree
+     if fmt `elem` ["png","gif","gv"]
+       then outputGraphviz vis
+       else if fmt=="svg"
+            then outputText "image/svg+xml" vis
+            else outputPlain vis
+
+getLabels lc path pgf =
+    msum [readCache' lc path | path<-[{-path1,-}path2,path3]]
+  where
+    dir = takeDirectory path
+  --path1 = dir</> ...labels flag from abstract syntax...
+    path2 = dir</>PGF.showCId (PGF.abstractName pgf)<.>"labels"
+    path3 = dropExtension path <.> "labels"
+
+getCncLabels lc path pgf lang =
+    either fail ok =<< tryIO (readCache lc path2)
+  where
+    ok ls  = do logError ("Found "++show (length ls)++" CncLabels for "++show lang++" in "++path2)
+                return (Just ls)
+    fail _ = do logError ("No CncLabels for "++show lang++" in "++path2)
+                return Nothing
+    dir = takeDirectory path
+  --path1 = dir</> ...labels flag from concrete syntax...
+    path2 = dir</>PGF.showCId lang<.>"labels"
+  --path3 = ...
+
+tryIO :: IO a -> IO (Either IOError a)
+tryIO = E.try
+
 alignment pgf tree (tos,unlex) = PGF.graphvizAlignment pgf tos' tree
   where tos' = if null tos then PGF.languages pgf else tos
 
@@ -806,8 +967,16 @@ doBrowse pgf (Just id) cssClass href _ pn = -- default to "html" format
     annotatePrintNames = "<DL>"++(unwords pns)++"</DL>"
       where pns = ["<DT>"++(show lang)++"</DT><DD>"++(PGF.showPrintName pgf lang id)++"</DD>" | lang <- PGF.languages pgf ]
 
+class ToATree a where 
+  showTree :: a -> String
+  toATree :: a -> PGF.ATree a
+
+instance ToATree PGF.Expr where
+  showTree = PGF.showExpr []
+  toATree = PGF.toATree
+
 -- | Render trees as JSON with numbered functions
-jsonExpr e = evalState (expr (PGF.toATree e)) 0
+jsonExpr e = evalState (expr (toATree e)) 0
   where
     expr e =
       case e of
@@ -842,10 +1011,10 @@ instance JSON PGF.BracketedString where
     showJSON (PGF.Leaf s) = makeObj ["token".=s]
 
 -- * PGF utilities
-
+{-
 cat :: PGF -> Maybe PGF.Type -> PGF.Type
 cat pgf mcat = fromMaybe (PGF.startCat pgf) mcat
-
+-}
 parse' :: PGF -> String -> Maybe PGF.Type -> Maybe PGF.Language -> [(PGF.Language,PGF.ParseOutput,PGF.BracketedString)]
 parse' pgf input mcat mfrom = 
    [(from,po,bs) | from <- froms, (po,bs) <- [PGF.parse_ pgf from cat Nothing input]]
@@ -853,17 +1022,17 @@ parse' pgf input mcat mfrom =
         cat = fromMaybe (PGF.startCat pgf) mcat
 
 complete' :: PGF -> PGF.Language -> PGF.Type -> Maybe Int -> String
-         -> (PGF.BracketedString, String, [String])
+         -> (PGF.BracketedString, String, Map.Map PGF.Token PGF.ParseState)
 complete' pgf from typ mlimit input =
   let (ws,prefix) = tokensAndPrefix input
       ps0 = PGF.initState pgf from typ
       (ps,ws') = loop ps0 ws
       bs       = snd (PGF.getParseOutput ps typ Nothing)
   in if not (null ws')
-       then (bs, unwords (if null prefix then ws' else ws'++[prefix]), [])
-       else (bs, prefix, maybe id take mlimit $ order $ Map.keys (PGF.getCompletions ps prefix))
+       then (bs, unwords (if null prefix then ws' else ws'++[prefix]), Map.empty)
+       else (bs, prefix, PGF.getCompletions ps prefix)
   where
-    order = sortBy (compare `on` map toLower)
+  --order = sortBy (compare `on` map toLower)
 
     tokensAndPrefix :: String -> ([String],String)
     tokensAndPrefix s | not (null s) && isSpace (last s) = (ws, "")
@@ -882,25 +1051,23 @@ transfer lang = if "LaTeX" `isSuffixOf` show lang
 
 -- | tabulate all variants and their forms
 linearizeTabular
-  :: PGF -> [PGF.Language] -> PGF.Tree -> [(PGF.Language,[(String,[String])])]
-linearizeTabular pgf tos tree =
+  :: PGF -> To -> PGF.Tree -> [(PGF.Language,[(String,[String])])]
+linearizeTabular pgf (tos,unlex) tree =
     [(to,lintab to (transfer to tree)) | to <- langs]
   where
     langs = if null tos then PGF.languages pgf else tos
-    lintab to t = [(p,map doBind (nub [t|(p',t)<-vs,p'==p]))|p<-ps]
+    lintab to t = [(p,map unlex (nub [t|(p',t)<-vs,p'==p]))|p<-ps]
       where
         ps = nub (map fst vs)
         vs = concat (PGF.tabularLinearizes pgf to t)
 
-linearizeAndBind pgf mto tree =
+linearizeAndUnlex pgf (mto,unlex) tree =
     [(to,s,bss) | to<-langs,
                  let bss = PGF.bracketedLinearize pgf to (transfer to tree)
-                     s   = unwords . bindTok $ concatMap PGF.flattenBracketedString bss]
+                     s   = unlex . unwords $ concatMap PGF.flattenBracketedString bss]
   where
     langs = if null mto then PGF.languages pgf else mto
 
-doBind = unwords . bindTok . words
-
 selectLanguage :: PGF -> Maybe (Accept Language) -> PGF.Language
 selectLanguage pgf macc = case acceptable of
                             []  -> case PGF.languages pgf of
