@@ -2,53 +2,23 @@ package org.grammaticalframework.ui.android;
 
 import android.content.Context;
 import android.content.SharedPreferences;
+import android.content.pm.PackageManager.NameNotFoundException;
 import android.util.Log;
 import android.util.Pair;
+import android.net.Uri;
 import android.view.inputmethod.CompletionInfo;
+import android.database.sqlite.SQLiteDatabase;
+import android.database.Cursor;
 
-import org.grammaticalframework.pgf.Concr;
-import org.grammaticalframework.pgf.Expr;
-import org.grammaticalframework.pgf.ExprProb;
-import org.grammaticalframework.pgf.FullFormEntry;
-import org.grammaticalframework.pgf.MorphoAnalysis;
-import org.grammaticalframework.pgf.NercLiteralCallback;
-import org.grammaticalframework.pgf.PGF;
-import org.grammaticalframework.pgf.ParseError;
-
-import java.io.FileNotFoundException;
-import java.io.IOException;
-import java.io.InputStream;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.Comparator;
-import java.util.List;
-import java.util.PriorityQueue;
+import org.grammaticalframework.sg.*;
+import org.grammaticalframework.pgf.*;
+import java.io.*;
+import java.util.*;
 
 public class Translator {
 
     private static final String TAG = "Translator";
 
-    /*
-
-    // old
-
-    // TODO: allow changing
-    private String mGrammar = "ParseEngAbs.pgf";
-
-    // TODO: build dynamically?
-    private Language[] mLanguages = {
-	    new Language("en-US", "English", "ParseEng", R.xml.qwerty),
-        new Language("bg-BG", "Bulgarian", "ParseBul", R.xml.cyrillic),
-        new Language("cmn-Hans-CN", "Chinese", "ParseChi", R.xml.qwerty),   
-        new Language("fr-FR", "French", "ParseFre", R.xml.qwerty),  
-        new Language("de-DE", "German", "ParseGer", R.xml.qwerty), 
-        new Language("hi-IN", "Hindi", "ParseHin", R.xml.devanagari_page1, R.xml.devanagari_page2), 
-        new Language("sv-SE", "Swedish", "ParseSwe", R.xml.qwerty), 
-        new Language("fi-FI", "Finnish", "ParseFin", R.xml.qwerty),
-    };
-     */
-
-    // /*
     // new
 
     // TODO: allow changing
@@ -56,19 +26,26 @@ public class Translator {
 
     // TODO: build dynamically?
     private Language[] mLanguages = {
-    	new Language("en-US", "English", "AppEng", R.xml.qwerty),
     	new Language("bg-BG", "Bulgarian", "AppBul", R.xml.cyrillic),
+        new Language("ca-ES", "Catalan", "AppCat", R.xml.qwerty),
     	new Language("cmn-Hans-CN", "Chinese", "AppChi", R.xml.qwerty),
+	//    	new Language("zh-CN", "Chinese", "AppChi", R.xml.qwerty),
+	//    	new Language("yue", "Chinese (Cantonese)", "AppChi", R.xml.qwerty),
+	//    	new Language("cmn-Hans-CN", "Chinese (Mandarin)", "AppChi", R.xml.qwerty),
         new Language("nl-NL", "Dutch", "AppDut", R.xml.qwerty),
-        new Language("fi-FI", "Finnish", "AppFin", R.xml.qwerty),
-        new Language("fr-FR", "French",  "AppFre", R.xml.qwerty),
+    	new Language("en-US", "English", "AppEng", R.xml.qwerty),
+        new Language("et-EE", "Estonian","AppEst", R.xml.nordic),
+        new Language("fi-FI", "Finnish", "AppFin", R.xml.nordic),
+        new Language("fr-FR", "French",  "AppFre", R.xml.qwerty), ////
         new Language("de-DE", "German",  "AppGer", R.xml.qwerty),
         new Language("hi-IN", "Hindi",   "AppHin", R.xml.devanagari_page1, R.xml.devanagari_page2),
         new Language("it-IT", "Italian", "AppIta", R.xml.qwerty),
+        new Language("ja-JP", "Japanese","AppJpn", R.xml.qwerty),
+        new Language("ru-RU", "Russian", "AppRus", R.xml.cyrillic),
         new Language("es-ES", "Spanish", "AppSpa", R.xml.qwerty),
-        new Language("sv-SE", "Swedish", "AppSwe", R.xml.qwerty),
+        new Language("sv-SE", "Swedish", "AppSwe", R.xml.nordic),
+        new Language("th-TH", "Thai",    "AppTha", R.xml.thai_page1, R.xml.thai_page2)
     };
-    // */
 
     private Context mContext;
 
@@ -76,10 +53,14 @@ public class Translator {
     private ConcrLoader mSourceLoader;
     private ConcrLoader mTargetLoader;
     private ConcrLoader mOtherLoader;
+    private SemanticGraphManager mSGManager;
 
 	private static final String SOURCE_LANG_KEY = "source_lang";
 	private static final String TARGET_LANG_KEY = "target_lang";
 	
+	public static final String WORDS     = "words";
+	public static final String SENTENCES = "sentences";
+	
 	private static final int NUM_ALT_TRANSLATIONS = 10;
 	
 	private SharedPreferences mSharedPref;
@@ -125,6 +106,8 @@ public class Translator {
         }
 
         mOtherLoader = null;
+
+        mSGManager = new SemanticGraphManager(context);
     }
 
     public List<Language> getAvailableLanguages() {
@@ -265,16 +248,21 @@ public class Translator {
     	return out;
     }
 
+    private static String implode(String s) {
+		return s.replaceAll("(?<!^[%*+])\\s","");
+    }
+
     private String translateWord(String input) {
 
-    	String output = "[" + input + "]" ;  // if all else fails, return the word itself in brackets
+    	String output = input.toUpperCase() ;  // if all else fails, return the word itself in upper case ///in brackets
     	Concr sourceLang = getSourceConcr() ;
     	Concr targetLang = getTargetConcr() ;
 
     	String lowerinput = input.toLowerCase() ;  // also consider lower-cased versions of the word
 
         try {
-        	Expr expr = sourceLang.parseBest("Chunk", input) ; // try parse as chunk
+        	Iterator<ExprProb> iter = sourceLang.parse("Chunk", input).iterator(); // try parse as chunk
+        	Expr expr = iter.next().getExpr();
             output = targetLang.linearize(expr);
             return output ;
         } catch (ParseError e) {                               	  // if this fails
@@ -303,6 +291,28 @@ public class Translator {
         return output ;
     }
 
+    // lowercase the first word if (1) it is not in the lexicon and (2) its lowercase version is in the lexicon
+    // otherwise it will be left uppercase and treated as a name
+    private String lowercaseIfBetter(String input) {
+    	String[] words = input.split(" ") ;
+	
+	String firstword = words[0] ;
+
+	String lowerfirstword = firstword.toLowerCase() ;
+
+	if (lookupMorpho(firstword).isEmpty() && !(lookupMorpho(lowerfirstword).isEmpty())) {
+	    words[0] = lowerfirstword ;
+	    String output = "" ;
+            for (String w : words) {
+        	output = output + " " + w ; 
+	    }
+            return output ;
+	} else {
+	    return input ;
+	}
+	    
+    }
+
     /**
      * Takes a lot of time. Must not be called on the main thread.
      */
@@ -310,7 +320,9 @@ public class Translator {
         if (getSourceLanguage().getLangCode().equals("cmn-Hans-CN")) {
         	// for Chinese we need to put space after every character
         	input = explode(input);
-        }
+        } else {
+	    input = lowercaseIfBetter(input);
+	} ;
 
         String output = null;
         List<ExprProb> exprs = new ArrayList<ExprProb>();
@@ -319,69 +331,301 @@ public class Translator {
             Concr sourceLang = getSourceConcr();
             Concr targetLang = getTargetConcr();
 
-            Expr expr = sourceLang.parseBest(getGrammar().getStartCat(), input);
+            Map<String,LiteralCallback> callbacks = new HashMap<String,LiteralCallback>();
+            callbacks.put("PN", new NercLiteralCallback(mGrammarLoader.getGrammar(), sourceLang, input));
+            callbacks.put("Symb", new UnknownLiteralCallback(sourceLang, input));
 
             int count = NUM_ALT_TRANSLATIONS;
-            for (ExprProb ep : sourceLang.parse(getGrammar().getStartCat(), input)) {
+            for (ExprProb ep : sourceLang.parseWithHeuristics(getGrammar().getStartCat(), input, -1, callbacks)) {
             	if (count-- <= 0)
             		break;
             	exprs.add(ep);
-            	output = targetLang.linearize(expr);
+            	if (output == null)
+					output = targetLang.linearize(ep.getExpr());
             }
         } catch (ParseError e) {
         	output = translateByLookup(input);
         }
         
+        if (output == null)
+        	output = "% ";     // make sure that we return something
+
+	if (getTargetLanguage().getLangCode().equals("cmn-Hans-CN") ||
+	    getTargetLanguage().getLangCode().equals("ja-JP") ||
+	    getTargetLanguage().getLangCode().equals("th-TH"))
+	    output = implode(output) ;
+		
         return new Pair<String,List<ExprProb>>(output, exprs);
     }
 
     public String linearize(Expr expr) {
     	Concr targetLang = getTargetConcr();
-    	return targetLang.linearize(expr);
+    	String s = targetLang.linearize(expr);
+    	if (s == null)
+    		s = "% ";     // make sure that we return something
+	
+		if (getTargetLanguage().getLangCode().equals("cmn-Hans-CN") ||
+		    getTargetLanguage().getLangCode().equals("ja-JP") ||
+		    getTargetLanguage().getLangCode().equals("th-TH"))
+		  return implode(s);
+		else
+		  return s;
     }
 
-    public String generateLexiconEntry(String lemma) {
+    public String linearizeSource(Expr expr) {
+    	Concr targetLang = getSourceConcr();
+    	String s = targetLang.linearize(expr);
+    	if (s == null)
+    		s = "% ";     // make sure that we return something
+	
+		if (getSourceLanguage().getLangCode().equals("cmn-Hans-CN") ||
+		    getSourceLanguage().getLangCode().equals("ja-JP") ||
+		    getSourceLanguage().getLangCode().equals("th-TH"))
+		  return implode(s);
+		else
+		  return s;
+    }
+
+    public Object[] bracketedLinearize(Expr expr) {
+    	Concr targetLang = getTargetConcr();
+    	return targetLang.bracketedLinearize(expr);
+    }
+
+    public String generateLexiconEntry(Expr lemma) {
         Concr sourceLang = getSourceConcr();
         Concr targetLang = getTargetConcr();
-    	String cat = getGrammar().getFunctionType(lemma).getCategory();
+        String fun = lemma.toString();
+    	String cat = getGrammar().getFunctionType(fun).getCategory();
 		
-    	Expr e1 = Expr.readExpr(lemma);
-    	Expr e2 = Expr.readExpr("MkTag (Inflection"+cat+" "+lemma+")");
+    	Expr e2 = Expr.readExpr("MkTag (Inflection"+cat+" "+fun+")");
 
     	if (targetLang.hasLinearization("Inflection"+cat)) {
-	        if (targetLang.hasLinearization(lemma))
-	        	return sourceLang.linearize(e1) + " - " + targetLang.linearize(e2) + ". " + targetLang.linearize(e1);
+	        if (targetLang.hasLinearization(fun))
+	        	return sourceLang.linearize(lemma) + " - " + targetLang.linearize(e2) + ". " + targetLang.linearize(lemma);
 	        else
-	        	return sourceLang.linearize(e1) + " " + targetLang.linearize(e2)+".";
+	        	return sourceLang.linearize(lemma) + " " + targetLang.linearize(e2)+".";
     	} else {
-    		if (targetLang.hasLinearization(lemma))
-    			return sourceLang.linearize(e1) + " - " + targetLang.linearize(e1);
+    		if (targetLang.hasLinearization(fun))
+    			return sourceLang.linearize(lemma) + " - " + targetLang.linearize(lemma);
     		else
-    			return sourceLang.linearize(e1);
+    			return sourceLang.linearize(lemma);
     	}
     }
 
-	public String getInflectionTable(String lemma) {
+	private static final Expr gloss_pred   = Expr.readExpr("gloss");
+	private static final Expr topic_pred   = Expr.readExpr("topic");
+	private static final Expr example_pred = Expr.readExpr("example");
+	
+    public Expr getDefinition(Expr lemma, boolean withExample) {
+		Expr gloss   = null;
+		Expr example = null;
+		Map<String,Uri.Builder> topics = new TreeMap<String,Uri.Builder>();
+
+		try {
+			TripleResult res = mSGManager.queryTriple(lemma, null, null);
+			while (res.hasNext()) {
+				if (res.getPredicate().equals(gloss_pred))
+					gloss = res.getObject();
+				else if (res.getPredicate().equals(topic_pred))
+					updateWordsMap(res.getObject(), topics);
+				else if (res.getPredicate().equals(example_pred))
+					example = res.getObject();
+			}
+			res.close();
+		} catch (IOException e) {
+			// nothing
+		} catch (SGError e) {
+			// nothing
+		}
+
+		Expr topic = null;
+		if (topics.size() > 0) {
+			StringBuilder builder = new StringBuilder();
+			builder.append('(');
+			buildWordsHtml(topics, builder);
+			builder.append(')');
+			topic = new Expr(builder.toString());
+		}
+		if (gloss == null)
+			return topic;
+		else {
+			if (topic == null)
+				topic = new Expr("");
+			if (withExample && example != null)
+				return new Expr("MkDefinitionEx", topic, gloss, example);
+			else
+				return new Expr("MkDefinition", topic, gloss);
+		}
+	}
+
+	private void updateWordsMap(Expr expr, Map<String,Uri.Builder> map) {
+		String word = getTargetConcr().linearize(expr);
+
+		Uri.Builder builder = map.get(word);
+		if (builder == null) {
+			builder = new Uri.Builder();
+			builder.scheme("gf-translator");
+			builder.authority(WORDS);
+			builder.appendQueryParameter("source", word);	
+			map.put(word,builder);
+		}
+		builder.appendQueryParameter("alternative", expr.toString());
+	}
+
+    private void buildWordsHtml(Map<String,Uri.Builder> map, StringBuilder sbuilder) {
+		boolean first = true;
+		for (Map.Entry<String,Uri.Builder> entry : map.entrySet()) {
+			if (first)
+				first = false;
+			else
+				sbuilder.append(", ");
+
+			sbuilder.append("<a href=\""+entry.getValue().build()+"\">"+entry.getKey()+"</a>");
+		}
+	}
+
+    public List<Expr> getTopicWords(Expr lemma) {
+		TripleResult res = null;
+		List<Expr> words = new ArrayList<Expr>();
+		try {
+			res = mSGManager.queryTriple(null, topic_pred, lemma);
+			while (res.hasNext()) {
+				words.add(res.getSubject());
+			}
+		} catch (IOException e) {
+			// nothing
+		} catch (SGError e) {
+			// nothing
+		} finally {
+			if (res != null)
+				res.close();
+		}
+		return words;
+	}
+
+    private Expr getTopicWordsHtml(Expr lemma) {
+		StringBuilder sbuilder = new StringBuilder();
+		TripleResult res = null;
+		try {
+			res = mSGManager.queryTriple(null, topic_pred, lemma);
+			Map<String,Uri.Builder> map = new TreeMap<String,Uri.Builder>();
+			while (res.hasNext()) {
+				updateWordsMap(res.getSubject(), map);
+			}
+
+			StringBuilder builder = new StringBuilder();
+			buildWordsHtml(map, builder);
+			return new Expr(builder.toString());
+		} catch (IOException e) {
+			// nothing
+		} catch (SGError e) {
+			// nothing
+		} finally {
+			if (res != null)
+				res.close();
+		}
+		return null;
+	}
+
+	public String getInflectionTable(Expr lemma) {
+		boolean withExample =
+			(getSourceLanguage().getLangCode().equals("en-US") ||
+             getTargetLanguage().getLangCode().equals("en-US"));
+		Expr def = 
+			getDefinition(lemma, withExample);
+
+		String fun = lemma.toString();
 		Concr targetLang = getTargetConcr();
-		String cat = getGrammar().getFunctionType(lemma).getCategory();
+		String cat = getGrammar().getFunctionType(fun).getCategory();
 
-		if (!targetLang.hasLinearization(lemma))
+		if (targetLang.hasLinearization(fun) && 
+		    targetLang.hasLinearization("Inflection"+cat)) {
+			if (def == null)
+				def = Expr.readExpr("NoDefinition");
+
+			Expr e = new Expr("MkDocument", 
+			                  def,
+			                  new Expr("Inflection"+cat,lemma),
+			                  getTopicWordsHtml(lemma));
+			String html =
+				"<html><head><meta charset=\"UTF-8\"/><style>a {color: #808080;}</style></head><body>" +
+				targetLang.linearize(e) +
+				"</body>";
+			return html;
+		} else if (def != null) {
+			String html =
+				"<html><head><meta charset=\"UTF-8\"/><style>a {color: #808080;}</style></head><body>" +
+				targetLang.linearize(def) +
+				"</body>";
+			return html;
+		} else {
 			return null;
+		}
+	}
 
-		if (!targetLang.hasLinearization("Inflection"+cat))
-			return null;
+    public List<Expr> getTopicsOf(Expr lemma) {
+		TripleResult res = null;
+		List<Expr> topics = new ArrayList<Expr>();
+		try {
+			res = mSGManager.queryTriple(lemma, topic_pred, null);
+			while (res.hasNext()) {
+				topics.add(res.getObject());
+			}
+		} catch (IOException e) {
+			// nothing
+		} catch (SGError e) {
+			// nothing
+		} finally {
+			if (res != null)
+				res.close();
+		}
+		return topics;
+	}
 
-		Expr e = Expr.readExpr("MkDocument \"\" (Inflection"+cat+" "+lemma+") \"\"");
-		String html =
-			"<html><head><meta charset=\"UTF-8\"/></head><body>" +
-			targetLang.linearize(e) +
-			"</body>";
+	private static String escapeHtml(CharSequence text) {
+		StringBuilder out = new StringBuilder();
 
-		return html;
+		for (int i = 0; i < text.length(); i++) {
+			char c = text.charAt(i);
+
+			if (c == '<') {
+				out.append("&lt;");
+			} else if (c == '>') {
+				out.append("&gt;");
+			} else if (c == '&') {
+				out.append("&amp;");
+			} else if (c == '"') {
+				out.append("&quot;");
+			} else if (c > 0x7E || c < ' ') {
+				out.append("&#").append((int) c).append(";");
+			} else if (c == ' ') {
+				while (i + 1 < text.length() && text.charAt(i + 1) == ' ') {
+					out.append("&nbsp;");
+					i++;
+				}
+
+				out.append(' ');
+			} else {
+				out.append(c);
+			}
+		}
+		
+		return out.toString();
 	}
 
     public List<MorphoAnalysis> lookupMorpho(String sentence) {
-    	return getSourceConcr().lookupMorpho(sentence);
+		List<MorphoAnalysis> ans = getSourceConcr().lookupMorpho(sentence);
+		Collections.sort(ans, new Comparator<MorphoAnalysis>() {
+				public int compare(MorphoAnalysis a1, MorphoAnalysis a2) {
+					if (a1.getProb() > a2.getProb())
+						return 1;
+					else if (a1.getProb() < a2.getProb())
+						return -1;
+					else
+						return 0;
+				}
+			});
+    	return ans;
     }
 
     public CompletionInfo[] lookupWordPrefix(String prefix) {
@@ -495,7 +739,6 @@ public class Translator {
 		        long t1 = System.currentTimeMillis();
 		        mConcr = mGrammarLoader.getGrammar().getLanguages().get(mLanguage.getConcrete());
 		        mConcr.load(in);
-		        mConcr.addLiteral("PN", new NercLiteralCallback(mGrammarLoader.getGrammar(), mConcr));
 		        long t2 = System.currentTimeMillis();
 		        Log.d(TAG, name + " loaded ("+(t2-t1)+" ms)");
 		    } catch (FileNotFoundException e) {
@@ -513,4 +756,21 @@ public class Translator {
 		    }
 		}
 	}
+
+    public boolean isUpgraded(String key) {
+		int old_code = mSharedPref.getInt(key, 0);
+
+		int new_code = 0;
+		try {
+			new_code = mContext.getPackageManager().getPackageInfo(mContext.getPackageName(), 0).versionCode;
+		} catch (NameNotFoundException e) {
+			// Huh? Really?
+		}
+
+		SharedPreferences.Editor editor = mSharedPref.edit();
+		editor.putInt(key, new_code);
+		editor.commit();
+
+		return (old_code != new_code);
+	}
 }
