@@ -7,10 +7,12 @@ var online_options={
   //grammar_list: ["Foods.pgf"], // leave undefined to get list from server
 }
 
+if(window.grammar_list) online_options.grammar_list=grammar_list
+
 var minibar_options= {
     show_abstract: true,
     show_trees: true,
-    tree_img_format: "png", // or "svg"
+//  tree_img_format: "png", // or "svg"
     show_grouped_translations: false,
     show_brackets: true,
     word_replacements: true,
